document.addEventListener("DOMContentLoaded", function() {
    let scannerActive = false;
    let stream = null;

    function stopScanner() {
        Quagga.stop();
        scannerActive = false;
        document.getElementById("scanner-container").style.display = "none";
        document.getElementById("laser-line").style.display = "none";
        document.getElementById("mm-scan-barcode").style.display = "block";
        if (stream) {
            stream.getTracks().forEach(track => track.stop());
            stream = null;
        }
    }

    // Use event delegation to detect clicks on dynamically added elements
    document.body.addEventListener("click", function(event) {
        if (event.target.id === "mm-scan-barcode") {
            document.getElementById("scanner-container").style.display = "block";
            document.getElementById("laser-line").style.display = "block";
            document.getElementById("mm-scan-barcode").style.display = "none"; // Hide the button when scanning starts

            navigator.mediaDevices.getUserMedia({
                video: { facingMode: { ideal: "environment" }, width: 640, height: 480 }
            }).then(function(mediaStream) {
                stream = mediaStream;
                let video = document.getElementById("barcode-scanner");
                video.srcObject = stream;
                video.play();
            }).catch(function(err) {
                console.error("Camera access denied: ", err);
                alert("Error: Camera access denied. Please enable camera permissions.");
            });

            if (scannerActive) return;
            scannerActive = true;

            Quagga.init({
                inputStream: { name: "Live", type: "LiveStream", target: document.querySelector("#barcode-scanner") },
                decoder: { readers: ["upc_reader", "upc_e_reader", "code_39_reader", "ean_reader", "code_128_reader"], willReadFrequently: true },
                locator: { halfSample: true },
                numOfWorkers: 2,
                frequency: 5
            }, function(err) {
                if (err) {
                    console.error("QuaggaJS initialization failed: ", err);
                    alert("Error initializing barcode scanner.");
                    stopScanner();
                    return;
                }
                Quagga.start();
            });
        }

        if (event.target.id === "close-scanner") {
            stopScanner();
        }
    });

    Quagga.onDetected(function(result) {
        var barcode = result.codeResult.code;
        Quagga.stop();
        fetch(mmBarcodeScanner.ajax_url, {  
            method: "POST",
            headers: { 'Content-Type': 'application/x-www-form-urlencoded' },
            body: new URLSearchParams({
                action: "mm_scan_barcode",
                barcode: barcode,
                auto_add: mmBarcodeScanner.auto_add,
                success_text: mmBarcodeScanner.success_text,
                not_found_text: mmBarcodeScanner.not_found_text,
            })
        })
        .then(response => response.json())
        .then(data => {
        	  if (data.success && mmBarcodeScanner.auto_add === 'cart' ) {
                window.location.href = "/cart/";  // if success and auto add, go to cart
            }
            else if (data.success && data.data.product_url) { 
                window.location.href = data.data.product_url;
            } 
            else  {
                window.location.href = "/shop/";  // if not success go to shop
            }
        })
        .catch(() => window.location.reload());
    });
});
