(function($){
    $(document).ready(function(){
        if (wp.media) {
            wp.media.view.Modal.prototype.on('open', function() {
                // Check to avoid duplicate insertion of the button.
                if ( ! $('#mm-camera-button').length ) {
                    var cameraButton = $('<button id="mm-camera-button" class="button">Take a Snapshot</button>');
                    $('.media-toolbar-primary').append(cameraButton);
                    cameraButton.on('click', function(e) {
                        e.preventDefault();
                        mmOpenCamera();
                    });
                }
            });
        }
    });

    /**
     * Opens the camera using getUserMedia.
     * Adds mobile support by setting playsinline and requesting the environment camera.
     */
    function mmOpenCamera() {
        if (!navigator.mediaDevices || !navigator.mediaDevices.getUserMedia) {
            alert("Your device does not support camera access.");
            return;
        }
        var video = document.createElement("video");
        video.setAttribute("playsinline", "true");
        var canvas = document.createElement("canvas");
        var context = canvas.getContext("2d");

        navigator.mediaDevices.getUserMedia({ video: { facingMode: "environment" } })
        .then(function(stream) {
            video.srcObject = stream;
            video.play();

            // Open a new popup window for capturing the image
            var captureWindow = window.open("", "Camera", "width=600,height=500");
            captureWindow.document.write(
                '<html><head><title>Camera Snapshot</title>' +
                '<meta name="viewport" content="width=device-width, initial-scale=1.0">' +
                '</head><body style="display:flex; flex-direction:column; align-items:center; justify-content:center; margin:0; padding:10px;">'
            );
            captureWindow.document.body.appendChild(video);

            var captureButton = document.createElement("button");
            captureButton.innerText = "Capture";
            captureButton.style.marginTop = "10px";
            captureWindow.document.body.appendChild(captureButton);

            captureButton.onclick = function() {
                canvas.width = video.videoWidth;
                canvas.height = video.videoHeight;
                context.drawImage(video, 0, 0, canvas.width, canvas.height);
                var imgData = canvas.toDataURL("image/png");
                mmUploadImage(imgData);
                stream.getTracks().forEach(function(track) {
                    track.stop();
                });
                captureWindow.close();
            };
        })
        .catch(function(err) {
            console.error("Camera access error:", err);
            alert("Error accessing camera: " + err.message);
        });
    }

    /**
     * Uploads the captured image to WordPress via AJAX.
     */
    function mmUploadImage(imageData) {
        var formData = new FormData();
        formData.append("action", "mm_upload_snapshot");
        formData.append("image", imageData);

        fetch(mm_snapshot_ajax.ajax_url, {
            method: "POST",
            body: formData
        })
        .then(function(response) {
            return response.json();
        })
        .then(function(data) {
            if (data.success) {
                alert("Image uploaded successfully!");
                // Optionally add the new attachment to the media selection if applicable.
                if (wp.media && wp.media.frame && wp.media.frame.state && wp.media.frame.state().get("selection")) {
                    wp.media.frame.state().get("selection").add(wp.media.attachment(data.attachment_id));
                }
            } else {
                alert("Upload failed: " + data.message);
            }
        })
        .catch(function(error) {
            console.error("Upload error:", error);
        });
    }
})(jQuery);
