<?php if (!defined('ABSPATH')) {
    exit;
}
// Exit if accessed directly

if (!class_exists('MM_REST_API')) {

    /**
     * Model that houses the logic of MM API.
     *
     * @since 1.0.0
     */
    class MM_REST_API {

        /*
        |--------------------------------------------------------------------------
        | Class Properties
        |--------------------------------------------------------------------------
         */

        /**
         * Property that holds the single main instance of MM_REST_API.
         *
         * @since 1.0.0
         * @access private
         * @var MM_REST_API
         */
        private static $_instance;

        /**
         * Property that holds MM API Controllers.
         *
         * @since 1.12.0
         */
        public $mm_rest_api_customer_controller;
        public $mm_rest_api_indirect_tax_controller;
        public $mm_rest_api_media_controller;
        public $mm_rest_api_settings_controller;
        public $mm_rest_api_products_controller;
        public $mm_rest_api_trade_class_controller;

        /*
        |--------------------------------------------------------------------------
        | Class Methods
        |--------------------------------------------------------------------------
         */

        /**
         * MM_REST_API constructor.
         *
         * @since 1.0.0
         * @access public
         *
         * @param array $dependencies Array of instance objects of all dependencies of MM_REST_API model.
         */
        public function __construct($dependencies = array()) {}

        /**
         * Ensure that only one instance of MM_REST_API is loaded or can be loaded (Singleton Pattern).
         *
         * @since 1.12.0
         * @access public
         *
         * @param array $dependencies Array of instance objects of all dependencies of MM_REST_API model.
         * @return MM_REST_API
         */
        public static function instance($dependencies = array()) {

            if (!self::$_instance instanceof self) {
                self::$_instance = new self($dependencies);
            }

            return self::$_instance;

        }

        /**
         * MM REST API controllers under mm/v1 namespace.
         *
         * @since 1.0.0
         * @access public
         */
        public function load_api_mm_controllers() {

            require_once MM_INCLUDES_PATH . 'api/v1/class-mm-rest-api-customer-v1-controller.php';
            require_once MM_INCLUDES_PATH . 'api/v1/class-mm-rest-api-indirect-tax-v1-controller.php';
            require_once MM_INCLUDES_PATH . 'api/v1/class-mm-rest-api-media-v1-controller.php';
            require_once MM_INCLUDES_PATH . 'api/v1/class-mm-rest-api-settings-v1-controller.php';
            require_once MM_INCLUDES_PATH . 'api/v1/class-mm-rest-api-product-v1-controller.php';
            require_once MM_INCLUDES_PATH . 'api/v1/class-mm-rest-api-trade-class-v1-controller.php';
            $this->mm_rest_api_customer_controller = new MM_REST_Customer_V1_Controller;
            $this->mm_rest_api_indirect_tax_controller = new MM_REST_Indirect_Tax_V1_Controller;
            $this->mm_rest_api_media_controller = new MM_REST_Media_V1_Controller;
            $this->mm_rest_api_settings_controller = new MM_REST_Settings_V1_Controller;
            $this->mm_rest_api_products_controller = new MM_REST_Products_V1_Controller;
            $this->mm_rest_api_trade_class_controller = new MM_REST_Trade_Class_V1_Controller;

        }

        /**
         * Authenticate if user if using MM rest base if api keys are provided.
         *
         * @since 1.0.0
         * @access public
         *
         * @param bool $rest_request
         * @return boolean
         */
        public function authenticate_user($rest_request) {

            $rest_prefix = trailingslashit(rest_get_url_prefix());
            $request_uri = esc_url_raw(wp_unslash($_SERVER['REQUEST_URI']));

            // Authenticate if the request is using the mm/v1 endpoint.
            if ((false !== strpos($request_uri, $rest_prefix . 'mm/'))) {
                return true;
            }

            return $rest_request;

        }

        /**
         * Execute model.
         *
         * @since 1.0.0
         * @access public
         */
        public function run() {

            add_action('woocommerce_loaded', function () {

                // API controllers using mm/v1 namespace
                add_action('rest_api_init', array($this, 'load_api_mm_controllers'), 5);

                // Authenticate users if api keys are provided
                add_action('woocommerce_rest_is_request_to_rest_api', array($this, 'authenticate_user'));

            }, 10);

        }

    }

}
