<?php if (!defined('ABSPATH')) {
    exit;
}
// Exit if accessed directly

if (!class_exists('MM_REST_Indirect_Tax_V1_Controller')) {

    /**
     * Model that houses the logic of REST Interface with Indirect Tax Setup.
     *
     * @since 1.12
     */
    class MM_REST_Indirect_Tax_V1_Controller extends WC_REST_Controller {

        /*
        |--------------------------------------------------------------------------
        | Class Properties
        |--------------------------------------------------------------------------
         */

        /**
         * Property that holds the single main instance of WWP_REST_Wholesale_Roles_V1_Controller.
         *
         * @var WWP_REST_Wholesale_Roles_V1_Controller
         */
        private static $_instance;

        /**
         * Endpoint namespace.
         *
         * @var string
         */
        protected $namespace = 'mm/v1';

        /**
         * Route base.
         *
         * @var string
         */
        protected $rest_base = 'indirect_taxes';



        //Other properties
        private $debug_active = null;

        /*
        |--------------------------------------------------------------------------
        | Class Methods
        |--------------------------------------------------------------------------
         */

        /**
         * MM_REST_Roles_V1_Controller constructor.
         *
         * @since 1.0.0
         * @access public
         */
        public function __construct() {

            
            // Fires when preparing to serve an API request.
            add_action("rest_api_init", array($this, "register_routes"));

        }

        /**
         * Register routes for ManageMore roles API.
         *
         * @since 1.0.0
         * @access public
         */
        public function register_routes() {


            register_rest_route($this->namespace, '/' . $this->rest_base, array(
                array(
                    'methods' => WP_REST_Server::READABLE,
                    'callback' => array($this, 'get_items'),
                    'permission_callback' => array($this, 'get_items_permissions_check'),
                ),
                array(
                    'methods' => WP_REST_Server::EDITABLE,
                    'callback' => array($this, 'update_item'),
                    'permission_callback' => array($this, 'update_item_permissions_check'),
                ),
            ));

            register_rest_route($this->namespace, '/' . $this->rest_base . '/(?P<id>\d+)', array(
                array(
                    'methods' => WP_REST_Server::READABLE,
                    'callback' => array($this, 'get_item'),
                    'permission_callback' => array($this, 'get_item_permissions_check'),
                ),
                array(
                    'methods' => WP_REST_Server::EDITABLE,
                    'callback' => array($this, 'update_item'),
                    'permission_callback' => array($this, 'update_item_permissions_check'),
                ),
            ));

        }

 
        /**
         * Check if a given request has access to read items.
         *
         * @param  WP_REST_Request $request Full details about the request.
         * @return WP_Error|boolean
         */
        public function get_items_permissions_check($request) {

            if (!wc_rest_check_post_permissions('product', 'read')) {
                return new WP_Error('mm_rest_cannot_view', __('Sorry, you cannot list resources.', 'woocommerce'), array('status' => rest_authorization_required_code()));
            }

            return true;
        }

        /**
         * Check if a given request has access to read an item.
         *
         * @param  WP_REST_Request $request Full details about the request.
         * @return WP_Error|boolean
         */
        public function get_item_permissions_check($request) {
            $post = get_post((int) $request['id']);

            if ($post && !wc_rest_check_post_permissions('product', 'read', $post->ID)) {
                return new WP_Error('mm_rest_cannot_view', __('Sorry, you cannot view this resource.', 'woocommerce'), array('status' => rest_authorization_required_code()));
            }

            return true;
        }

        /**
         * Check if a given request has access to update an item.
         *
         * @param  WP_REST_Request $request Full details about the request.
         * @return WP_Error|boolean
         */
        public function update_item_permissions_check($request) {
            $post = get_post((int) $request['id']);

            if ($post && !wc_rest_check_post_permissions('product', 'edit', $post->ID)) {
                return new WP_Error('mm_rest_cannot_edit', __('Sorry, you are not allowed to edit this resource.', 'woocommerce'), array('status' => rest_authorization_required_code()));
            }

            return true;
        }

        /**
         * Get all indirect tax types from the collection.
         *
         * @param WP_REST_Request         $request
         *
         * @since 1.0.0
         * @access public
         * @return WP_REST_Response
         */
        public function get_items($request) {
          
            $posts = get_posts(
                array(
                    'post_type'  => 'mm-indirect-tax-type',
                    'post_status' => 'publish',
                    'numberposts' => -1
                )
            );

            $indirect_taxes= array();
 
            if( $posts ) {

               foreach( $posts as $p )
               {
                   $indirect_tax= array(
                     'id' => $p->ID,
                     'recId' => get_post_meta( $p->ID, 'mm_rec_id', true ),
                     'description' => $p->post_title,
                   );
                   $indirect_taxes[] = $indirect_tax;
                }

            }

            $result = $indirect_taxes;
            return new WP_REST_Response($result, 200); 
              

          
        }


        /**
         * Get one item from the collection.
         *
         * @param WP_REST_Request         $request
         *
         * @since 1.0.0
         * @access public
         * @return WP_Error|WP_REST_Response
         */
        public function get_item($request) {

            global $wpdb; //database required 2.1.0
            
            $indirect_tax_type = $request['id'];
            $post = get_post((int) $indirect_tax_type);
            $post_type = $post->post_type;

            if ($post && $post_type == 'mm-indirect-tax-type') {

                $indirect_taxes = array(
                   'recId' => get_post_meta( $post->ID, 'mm_rec_id', true ),
                   'description' => $post->post_title
                );


               //2.1.0 - Get Indirect Taxes from database file
               $table_name =  $wpdb->prefix . 'mm_indirect_tax';
               $query = 'SELECT * FROM '.$table_name.' WHERE indirect_tax_type='.$indirect_tax_type;
               $results = $wpdb->get_results($query);

               if ($results){

                  $indirect_tax_rates = array();
                  
                  foreach( $results as $indirect_tax_result )
                  {

                      //2.1.0 Changed to use db instead of posts
                      $indirect_tax= array(
                        'id' => $indirect_tax_result->id,
                        'state' => $indirect_tax_result->indirect_tax_state,
                        'rate' => $indirect_tax_result->indirect_tax_rate,
                        'taxCalcMethod' => $indirect_tax_result->indirect_tax_calc_method,
                        'minmaxCalcMethod' => $indirect_tax_result->min_max_calc_method,
                        'maxAmountToTax' => $indirect_tax_result->max_amount_to_tax,
                        'minAmountToTax' => $indirect_tax_result->min_amount_to_tax,
                      );
                      $indirect_tax_rates[] = $indirect_tax;
                   }
                   
                   $indirect_taxes['rates'] = $indirect_tax_rates;

               }

               //4.8.0 - Get Indirect Taxe Jurisdictions from database file
               $table_name =  $wpdb->prefix . 'mm_indirect_tax_jurisdiction';
               $query = 'SELECT * FROM '.$table_name.' WHERE indirect_tax_type='.$indirect_tax_type;
               $results = $wpdb->get_results($query);

               if ($results){

                  $indirect_tax_jurisdictions = array();
                  
                  foreach( $results as $indirect_tax_result )
                  {

                      $indirect_tax= array(
                        'id' => $indirect_tax_result->id,
                        'tax_code' => $indirect_tax_result->indirect_tax_code,
                        'rate_j1' => $indirect_tax_result->indirect_tax_rate_j1,
                        'taxCalcMethod_j1' => $indirect_tax_result->indirect_tax_calc_method_j1,
                        'minmaxCalcMethod_j1' => $indirect_tax_result->min_max_calc_method_j1,
                        'maxAmountToTax_j1' => $indirect_tax_result->max_amount_to_tax_j1,
                        'minAmountToTax_j1' => $indirect_tax_result->min_amount_to_tax_j1,
                        'rate_j2' => $indirect_tax_result->indirect_tax_rate_j2,
                        'taxCalcMethod_j2' => $indirect_tax_result->indirect_tax_calc_method_j2,
                        'minmaxCalcMethod_j2' => $indirect_tax_result->min_max_calc_method_j2,
                        'maxAmountToTax_j2' => $indirect_tax_result->max_amount_to_tax_j2,
                        'minAmountToTax_j2' => $indirect_tax_result->min_amount_to_tax_j2,
                      );
                      $indirect_tax_jurisdictions[] = $indirect_tax;
                   }
                   
                   $indirect_taxes['jurisdictions'] = $indirect_tax_jurisdictions;

               }


               $result = $indirect_taxes;
               return new WP_REST_Response($result, 200);

            }

            $result = array(); // if not found, return empty json
            return new WP_REST_Response($result, 200);

        }

        /**
         * Update one item from the collection.
         *
         * @param WP_REST_Request         $request
         *
         * @since 1.0.0
         * @access public
         * @return WP_Error|WP_REST_Request
         */
        public function update_item($request) {


            $id = isset($request['id']) ? $request['id'] : '';
            $indirect_taxes = array();
            
            
            $this->debug_log('Updating Indirect Tax Type: ' . $id) ; //. print_r($request['rates'],true) .  print_r($request['jurisdictions']);

            // Create an array of state rates ( e.g. $state_rates['FL'] = .025 )
            // Also create an array of indirect tax method (e.g. $state_method['FL'] = 1)
            $tax_rates = $request['rates'];
            $state_rates = array();
            $state_method = array();
            $state_min_max_method = array();
            $state_max_amount = array();
            $state_min_amount = array();
            foreach ($tax_rates as $tax_rate) {
                $state_rates[$tax_rate['state']] = $tax_rate['rate'];
                $state_method[$tax_rate['state']] = $tax_rate['taxCalcMethod'];
                $state_min_max_method[$tax_rate['state']] = $tax_rate['minmaxCalcMethod'];
                $state_max_amount[$tax_rate['state']] = $tax_rate['maxAmountToTax'];
                $state_min_amount[$tax_rate['state']] = $tax_rate['minAmountToTax'];
            } 

            // Create an array of jurisdiction rates ( e.g. $j1_rates['TAXCODE'] = .025 )
            // Also create an array of jurisdiction tax method (e.g. $j1_methods['TAXCODE'] = 1)
            $tax_jurisdictions = $request['jurisdictions'];
            $j1_rates = array();
            $j1_method = array();
            $j1_min_max_method = array();
            $j1_max_amount = array();
            $j1_min_amount = array();
            $j2_rates = array();
            $j2_method = array();
            $j2_min_max_method = array();
            $j2_max_amount = array();
            $j2_min_amount = array();
            foreach ($tax_jurisdictions as $tax_jurisdiction) {
                $j1_rates[$tax_jurisdiction['taxcode']] = $tax_jurisdiction['rate_j1'];
                $j1_method[$tax_jurisdiction['taxcode']] = $tax_jurisdiction['taxCalcMethod_j1'];
                $j1_min_max_method[$tax_jurisdiction['taxcode']] = $tax_jurisdiction['minmaxCalcMethod_j1'];
                $j1_max_amount[$tax_jurisdiction['taxcode']] = $tax_jurisdiction['maxAmountToTax_j1'];
                $j1_min_amount[$tax_jurisdiction['taxcode']] = $tax_jurisdiction['minAmountToTax_j1'];
                $j2_rates[$tax_jurisdiction['taxcode']] = $tax_jurisdiction['rate_j2'];
                $j2_method[$tax_jurisdiction['taxcode']] = $tax_jurisdiction['taxCalcMethod_j2'];
                $j2_min_max_method[$tax_jurisdiction['taxcode']] = $tax_jurisdiction['minmaxCalcMethod_j2'];
                $j2_max_amount[$tax_jurisdiction['taxcode']] = $tax_jurisdiction['maxAmountToTax_j2'];
                $j2_min_amount[$tax_jurisdiction['taxcode']] = $tax_jurisdiction['minAmountToTax_j2'];
            } 


            $mm_rec_id = isset($request['recId']) ? $request['recId'] : '';


            if (!empty($id)) {

               //Update indirect taxes

               // Get indirect tax type post object
               $post = get_post((int) $request['id']);
               $post_type = $post->post_type;
               $indirect_tax_type_id = $post->ID;
               


               if ($post && $post_type == 'mm-indirect-tax-type') {
               	
               	if ($request['remove'] == true){
               	  	 //remove the indirect tax type and related taxes
               	  	 wp_delete_post( $indirect_tax_type_id, true);
               	  	 $this->remove_rates($indirect_tax_type_id) ; //removes rates and jurisdictions
               	}
                  elseif (!empty($request['description'])) {
                     $indirect_tax_type = array(
                        'ID'            => $indirect_tax_type_id,
                        'post_title'    => wp_strip_all_tags( $request['description']),
                        'post_content'  => '',
                        'post_type'     => 'mm-indirect-tax-type',
                        'post_status'   => 'publish',
                        'meta_input'    => array(
                                            'mm_rec_id' => $mm_rec_id,
                                            ),
                     );
                     // update post in the database
                     $indirect_tax_type_id = wp_update_post( $indirect_tax_type );
                     $indirect_taxes['id'] =  $indirect_tax_type_id;
                     
                     //Update Rates
                     if  ($indirect_tax_type_id != 0 ){
                        $this->update_rates($indirect_tax_type_id,$state_rates,$state_method,$state_min_max_method,$state_max_amount,$state_min_amount);
                     } 
                     //Update Jurisdictions
                     if  ($indirect_tax_type_id != 0 ){
                    	   $this->update_jurisdictions($indirect_tax_type_id,$j1_rates,$j1_method,$j1_min_max_method,$j1_max_amount,$j1_min_amount,$j2_rates,$j2_method,$j2_min_max_method,$j2_max_amount,$j2_min_amount);
                     } 
                     
                  } // if remove/description was passed

                  
               } // if the id passed was a valid indirect tax type
                

            
            }
            else {


               //Check if mm_rec_id exists already. If so, update it
               if ($mm_rec_id != ''){
                  $posts = get_posts(
                       array(
                           'post_type'  => 'mm-indirect-tax-type',
                           'post_status' => 'publish',
                           'meta_key' => 'mm_rec_id',
                           'meta_value' => $mm_rec_id,
                           'numberposts' => 1
                          )
                       );
               }
               else {
                 $posts = array();
               }

               if (!empty($posts)){

                  //Update the Post (tax type)
               	  $post = $posts[0];
                  $indirect_tax_type_id = $post->ID;
                  $indirect_taxes['id'] =  $indirect_tax_type_id;
                  $indirect_tax_type = array(
                      'ID'            => $indirect_tax_type_id,
                      'post_title'    => wp_strip_all_tags( $request['description'] ),
                      'post_content'  => '',
                      'post_type'     => 'mm-indirect-tax-type',
                      'post_status'   => 'publish',
                      'meta_input'    => array(
                                            'mm_rec_id' => $mm_rec_id,
                                            ),
                      );
                  $indirect_tax_type_id = wp_update_post( $indirect_tax_type );
                  
                  //Update Rates
                  if  ($indirect_tax_type_id != 0 ){
                 	   $this->update_rates($indirect_tax_type_id,$state_rates,$state_method,$state_min_max_method,$state_max_amount,$state_min_amount);
                  }  
                  //Update Jurisdictions
                  if  ($indirect_tax_type_id != 0 ){
                 	   $this->update_jurisdictions($indirect_tax_type_id,$j1_rates,$j1_method,$j1_min_max_method,$j1_max_amount,$j1_min_amount,$j2_rates,$j2_method,$j2_min_max_method,$j2_max_amount,$j2_min_amount);
                  } 



               }
               else {

                  // Add new tax type
               
                  // Create post object for Indirect Tax Type
                  $indirect_tax_type = array(
                      'post_title'    => wp_strip_all_tags( $request['description'] ),
                      'post_content'  => '',
                      'post_type'     => 'mm-indirect-tax-type',
                      'post_status'   => 'publish',
                      'meta_input'    => array(
                                            'mm_rec_id' => $mm_rec_id,
                                            ),
                      );
    
                  // Add post to the database
                  $indirect_tax_type_id = wp_insert_post( $indirect_tax_type );
                  $indirect_taxes['id'] =  $indirect_tax_type_id;
                  //Update Rates
                  if  ($indirect_tax_type_id != 0 ){
                 	   $this->update_rates($indirect_tax_type_id,$state_rates,$state_method,$state_min_max_method,$state_max_amount,$state_min_amount);
                  } 
                  //Update Jurisdictions
                  if  ($indirect_tax_type_id != 0 ){
                 	   $this->update_jurisdictions($indirect_tax_type_id,$j1_rates,$j1_method,$j1_min_max_method,$j1_max_amount,$j1_min_amount,$j2_rates,$j2_method,$j2_min_max_method,$j2_max_amount,$j2_min_amount);
                  } 


               	
               }




            }


            $result = $indirect_taxes;
            return new WP_REST_Response($result, 200);


        }


        /**
         * Create/Update/Delete Indirect Taxes (rates by State).
         *
         * @param long $indirect_tax_type_id
         * @param array() $state_rates
         * @param array() $state_method
         *
         * @since 1.0.0
         * @access private
         * @return none
         */
        private function update_rates($indirect_tax_type_id,$state_rates,$state_method,$state_min_max_method,$state_max_amount,$state_min_amount){

            $this->debug_log('Updating Rates for '. $indirect_tax_type_id . 'State Rates -> ' . print_r($state_rates,true) . ' State Method ->' . print_r($state_method,true) .' State Min/Max Method '. print_r($state_min_max_method,true) . ' State Max Amount ' . print_r($state_max_amount,true) . ' State Min Amount ' . print_r($state_max_amount,true));
            
            //get indirect taxes to update
            global $wpdb; //2.1.0 - Need database access
            
            //2.1.0 - Get Indirect Taxes from database file
            $table_name =  $wpdb->prefix . 'mm_indirect_tax';
            $query = 'SELECT * FROM '.$table_name.' WHERE indirect_tax_type='.$indirect_tax_type_id;
            $results = $wpdb->get_results($query);
            
            $updated = false;  //flag to indicate an update was made

            foreach ($results as $indirect_tax_result) {
              
               // loop through states for this indirect tax
               //2.1.0 - Changed logic to get values from database result 

               $indirect_tax_state = $indirect_tax_result->indirect_tax_state;
               
               if (array_key_exists($indirect_tax_state,$state_rates)){
                  //The current State was passed.  Update the rate, if necessary
                  $update_record = false;
                  $current_rate = $indirect_tax_result->indirect_tax_rate;
                  $new_rate = $state_rates[$indirect_tax_state];
                  $current_tax_calc_method = $indirect_tax_result->indirect_tax_calc_method;
                  $new_tax_calc_method = $state_method[$indirect_tax_state];
                  $current_min_max_method = $indirect_tax_result->min_max_calc_method;
                  $new_min_max_method = $state_min_max_method[$indirect_tax_state];
                  $current_max_amount = $indirect_tax_result->max_amount_to_tax;
                  $new_max_amount = $state_max_amount[$indirect_tax_state];
                  $current_min_amount = $indirect_tax_result->min_amount_to_tax;
                  $new_min_amount = $state_min_amount[$indirect_tax_state];

                  if ($current_rate != $new_rate || 
                      ($new_tax_calc_method != '' && $new_tax_calc_method != $current_tax_calc_method) || 
                      $new_min_max_method != $current_min_max_method ||
                      $new_max_amount != $current_max_amount ||
                      $new_min_amount != $current_min_amount ) {
                      	
                      $update_record = true;
                  }
                  if ($update_record == true){
                  	//Update record in database
                     $update = $wpdb->update($table_name,
                                             array(
                                                  'indirect_tax_rate'=>$new_rate,
                                                  'indirect_tax_calc_method'=>$new_tax_calc_method,
                                                  'min_max_calc_method'=>$new_min_max_method,
                                                  'max_amount_to_tax'=>$new_max_amount,
                                                  'min_amount_to_tax'=>$new_min_amount
                                                  ),
                                             array(
                                                  'id'=>$indirect_tax_result->id
                                                  ),
                                             array( '%f', '%d' ), array( '%d' )
                                             );
                  }
                  //Remove state from our rate array, so all that will be left are states to add.
                  unset($state_rates[$indirect_tax_state]);
                  $updated = true;
               }
               else {
                  //State was not passed.  Delete it.
                  
                  //2.1.0 - Remove from database file
                  $update = $wpdb->delete( $table_name, array('id' => $indirect_tax_result->id ));
                  $updated = true;

                  
               }
            }  // for each current indirect tax rate
            
            //For any rates that are left in our rate array, create a new post for the indirect tax rate.
            if (!empty($state_rates)){
               foreach ($state_rates as $state=>$rate) {

                 //2.1.0 - Add to Database File
                 $update = $wpdb->insert( 
                             $table_name, 
                             array( 
                                  'indirect_tax_type' => $indirect_tax_type_id, 
                                  'indirect_tax_state' => $state, 
                                  'indirect_tax_rate' => $rate, 
                                  'indirect_tax_calc_method' => $state_method[$state],
                                  'min_max_calc_method' => $state_min_max_method[$state],
                                  'max_amount_to_tax' => $state_max_amount[$state],
                                  'min_amount_to_tax' => $state_min_amount[$state]
                                  ) 
                              );

               }  // for each rate that needs to be added
               $updated = true;
            	
            }

           //Clear cache
           if ($updated){
           	  MM_Object_Cache::delete_by_scope('indirect_tax_type', $indirect_tax_type_id, 'indirect_tax');
           }


        	
        }

        /**
         * Create/Update/Delete Indirect Taxes (rates by Jurisdiction).
         *
         * @param long $indirect_tax_type_id
         * @param array() $j1_rates
         * @param array() $j1_method
         * @param array() $j2_rates
         * @param array() $j2_method
         *
         * @since 1.0.0
         * @access private
         * @return none
         */
        private function update_jurisdictions($indirect_tax_type_id,$j1_rates,$j1_method,$j1_min_max_method,$j1_max_amount,$j1_min_amount,$j2_rates,$j2_method,$j2_min_max_method,$j2_max_amount,$j2_min_amount){


            $this->debug_log('Updating Jurisdictions for '. $indirect_tax_type_id . 'J1 Rates -> ' . print_r($j1_rates,true) . ' J1 Method ->' . print_r($j1_method,true) .' J1 Min/Max Method '. print_r($j1_min_max_method,true) . ' J1 Max Amount ' . print_r($j1_max_amount,true) . ' J1 Min Amount ' . print_r($j1_min_amount,true) .
                                                                                   'J2 Rates -> ' . print_r($j2_rates,true) . ' J2 Method ->' . print_r($j2_method,true) .' J2 Min/Max Method '. print_r($j2_min_max_method,true) . ' J2 Max Amount ' . print_r($j2_max_amount,true) . ' J2 Min Amount ' . print_r($j2_min_amount,true));

            //get indirect taxes to update
            global $wpdb; //2.1.0 - Need database access
            
            //2.1.0 - Get Indirect Taxes from database file
            $table_name =  $wpdb->prefix . 'mm_indirect_tax_jurisdiction';
            $query = 'SELECT * FROM '.$table_name.' WHERE indirect_tax_type='.$indirect_tax_type_id;
            $results = $wpdb->get_results($query);

            $updated = false; //flag to indicate an update was made

            foreach ($results as $indirect_tax_result) {
              
               // loop through states for this indirect tax
               //2.1.0 - Changed logic to get values from database result 

               $indirect_tax_code = $indirect_tax_result->indirect_tax_code;
               
               if (array_key_exists($indirect_tax_code,$j1_rates)){
                  //The current taxode was passed.  Update the rate, if necessary (note: we check j1, if there, it will be in both j1 and j2)
                  $update_record = false;
                  $current_rate_j1 = $indirect_tax_result->indirect_tax_rate_j1;
                  $new_rate_j1 = $j1_rates[$indirect_tax_code];
                  $current_tax_calc_method_j1 = $indirect_tax_result->indirect_tax_calc_method_j1;
                  $new_tax_calc_method_j1 = $j1_method[$indirect_tax_code];
                  $current_min_max_method_j1 = $indirect_tax_result->min_max_calc_method_j1;
                  $new_min_max_method_j1 = $j1_min_max_method[$indirect_tax_code];
                  $current_max_amount_j1 =  $indirect_tax_result->max_amount_to_tax_j1;
                  $new_max_amount_j1 = $j1_max_amount[$indirect_tax_code];
                  $current_min_amount_j1 =  $indirect_tax_result->min_amount_to_tax_j1;
                  $new_min_amount_j1 = $j1_min_amount[$indirect_tax_code];
                  $current_rate_j2 = $indirect_tax_result->indirect_tax_rate_j2;
                  $new_rate_j2 = $j2_rates[$indirect_tax_code];
                  $current_tax_calc_method_j2 = $indirect_tax_result->indirect_tax_calc_method_j2;
                  $new_tax_calc_method_j2 = $j2_method[$indirect_tax_code];
                  $new_min_max_method_j2 = $j2_min_max_method[$indirect_tax_code];
                  $current_max_amount_j2 =  $indirect_tax_result->max_amount_to_tax_j2;
                  $new_max_amount_j2 = $j2_max_amount[$indirect_tax_code];
                  $current_min_amount_j2 =  $indirect_tax_result->min_amount_to_tax_j2;
                  $new_min_amount_j2 = $j2_min_amount[$indirect_tax_code];

                  if (($new_rate_j1 != $current_rate_j1 ) ||
                     ($new_tax_calc_method_j1 != $current_tax_calc_method_j1) ||
                     ($new_min_max_method_j1 != $current_min_max_method_j1) ||
                     ($new_max_amount_j1 != $current_max_amount_j1) ||
                     ($new_min_amount_j1 != $current_min_amount_j1) ||
                     ($new_rate_j2 != $current_rate_j2 ) ||
                     ($new_tax_calc_method_j2 != $current_tax_calc_method_j2) ||
                     ($new_min_max_method_j2 != $current_min_max_method_j2) ||
                     ($new_max_amount_j2 != $current_max_amount_j2) ||
                     ($new_min_amount_j2 != $current_min_amount_j2)) {
                     

                     $update_record = true;
                  }
                  
                  if ($update_record == true){
                  	//Update record in database
                     $update = $wpdb->update($table_name,
                                             array(
                                                  'indirect_tax_rate_j1'=>$new_rate_j1,
                                                  'indirect_tax_calc_method_j1'=>$new_tax_calc_method_j1,
                                                  'min_max_calc_method_j1'=>$new_min_max_method_j1,
                                                  'max_amount_to_tax_j1'=>$new_max_amount_j1,
                                                  'min_amount_to_tax_j1'=>$new_min_amount_j1,
                                                  'indirect_tax_rate_j2'=>$new_rate_j2,
                                                  'indirect_tax_calc_method_j2'=>$new_tax_calc_method_j2,
                                                  'min_max_calc_method_j2'=>$new_min_max_method_j2,
                                                  'max_amount_to_tax_j2'=>$new_max_amount_j2,
                                                  'min_amount_to_tax_j2'=>$new_min_amount_j2
                                                  ),
                                             array(
                                                  'id'=>$indirect_tax_result->id
                                                  ),
                                             array( '%f', '%d', '%f', '%d' ), array( '%d' )
                                             );
                  }
                  //Remove tax code from our rate arrays, so all that will be left are tax codes to add.
                  unset($j1_rates[$indirect_tax_code]);
                  unset($j2_rates[$indirect_tax_code]);
                  $updated = true;
               }
               else {
                  //Taxcode was not passed.  Delete it.
                  
                  //2.1.0 - Remove from database file
                  $update = $wpdb->delete( $table_name, array('id' => $indirect_tax_result->id ));
                  $updated = true;
                  
               }
            }  // for each current indirect tax rate
            
            //For any rates that are left in our rate array, create a new post for the indirect tax rate.
            if (!empty($j1_rates)){
               foreach ($j1_rates as $tax_code=>$rate) {

                 //2.1.0 - Add to Database File
                 $update = $wpdb->insert( 
                             $table_name, 
                             array( 
                                  'indirect_tax_type' => $indirect_tax_type_id, 
                                  'indirect_tax_code' => $tax_code, 
                                  'indirect_tax_rate_j1' => $rate, 
                                  'indirect_tax_calc_method_j1' => $j1_method[$tax_code],
                                  'min_max_calc_method_j1'=>$j1_min_max_method[$tax_code],
                                  'max_amount_to_tax_j1'=>$j1_max_amount[$tax_code],
                                  'min_amount_to_tax_j1'=>$j1_min_amount[$tax_code],
                                  'indirect_tax_rate_j2' => $j2_rates[$tax_code],  //we are looping for each j1_rates, so must reference j2 by index
                                  'indirect_tax_calc_method_j2' => $j2_method[$tax_code],
                                  'min_max_calc_method_j2'=>$j2_min_max_method[$tax_code],
                                  'max_amount_to_tax_j2'=>$j2_max_amount[$tax_code],
                                  'min_amount_to_tax_j2'=>$j2_min_amount[$tax_code]
                                  ) 
                              );

               }  // for each rate that needs to be added
            	 $updated = true;
            }

           //Clear cache
           if ($updated){
           	  MM_Object_Cache::delete_by_scope('indirect_tax_type', $indirect_tax_type_id, 'indirect_tax_jurisdiction');
           }

        	
        }


        /**
         * Remove all Indirect Taxes for a tax type (rates by State and Jurisdictions).
         *
         * @param long $indirect_tax_type_id
         *
         * @since 1.0.0
         * @access private
         * @return none
         */
        private function remove_rates($indirect_tax_type_id){

           global $wpdb;
           
           //2.1.0 - Delete records from indirect tax database table
           $table_name =  $wpdb->prefix . 'mm_indirect_tax';
           $update = $wpdb->delete($table_name,array('indirect_tax_type' => $indirect_tax_type_id));
            
           //4.8.0 - Delete records from jurisdictions database table
           $table_name =  $wpdb->prefix . 'mm_indirect_tax_jurisdictions';
           $update = $wpdb->delete($table_name,array('indirect_tax_type' => $indirect_tax_type_id));
           
           //5.1.0 - Remove Cache
           MM_Object_Cache::delete_by_scope('indirect_tax_type', $indirect_tax_type_id, 'indirect_tax');
           MM_Object_Cache::delete_by_scope('indirect_tax_type', $indirect_tax_type_id, 'indirect_tax_jurisdiction');
        	
        }
        

        protected function debug_log($message) {
        	  if ($this->debug_active === null){
        	  	$this->debug_active = MM_Debug_Mode::is_active(MM_Debug_Mode::REST_API);
        	  }
            if ($this->debug_active) {
               MM_Connector_Functions::debug_log($message, MM_Debug_Mode::REST_API);
            }
        }
        


    }

}
