<?php if (!defined('ABSPATH')) {
    exit;
}
// Exit if accessed directly

if (!class_exists('MM_REST_Media_V1_Controller')) {

    /**
     * Model that houses the logic of REST Interface with WordPress Media Library.
     *
     * @since 1.0.0
     */
    class MM_REST_Media_V1_Controller extends WC_REST_Controller {

        /*
        |--------------------------------------------------------------------------
        | Class Properties
        |--------------------------------------------------------------------------
         */

        /**
         * Property that holds the single main instance of MM_REST_Media_V1_Controller.
         *
         * @var MM_REST_Media_V1_Controller
         */
        private static $_instance;


        /**
         * Endpoint namespace.
         *
         * @var string
         */
        protected $namespace = 'mm/v1';

        /**
         * Route base.
         *
         * @var string
         */
        protected $rest_base = 'media';


        /*
        |--------------------------------------------------------------------------
        | Class Methods
        |--------------------------------------------------------------------------
         */

        /**
         * MM_REST_Media_V1_Controller constructor.
         *
         * @since 1.0.0
         * @access public
         */
        public function __construct() {

            
            // Fires when preparing to serve an API request.
            add_action("rest_api_init", array($this, "register_routes"));
            
           
        }

        /**
         * Register routes for ManageMore roles API.
         *
         * @since 1.0.0
         * @access public
         */
        public function register_routes() {


            register_rest_route($this->namespace, '/' . $this->rest_base , array(
                array(
                    'methods' => WP_REST_Server::READABLE,
                    'callback' => array($this, 'get_media'),
                    'permission_callback' => array($this, 'get_media_permissions_check'),
                    'args' => array(
                          'search' => array(
                                            'type' => 'string',
                                            'description' => 'Search for a specific value',
                                            'required' => false,
                                            'sanitize_callback' =>'sanitize_text_field',
                                            'validate_callback' => array($this,'validate_search_parameter')
                                           )
                     ),
            )));



        }

 
        /**
         * Check if a given request has access to read media. We may add a real check later, for now we always allow
         *
         * @param  WP_REST_Request $request Full details about the request.
         * @return WP_Error|boolean
         */
        public function get_media_permissions_check($request) {
        	

            return true;
        }

        /**
         * Validate Search Parameter
         *
         * @param  WP_REST_Request $request Full details about the request.
         * @return WP_Error|boolean
         */
        public function validate_search_parameter($request) {
        	

            return true;
        }



        /**
         * Get one user.
         *
         * @param WP_REST_Request         $request
         *
         * @since 1.0.0
         * @access public
         * @return WP_Error|WP_REST_Request
         */
        public function get_media($request) {
        	
        	   $search = $request->get_param( 'search' );
            $search = urldecode( $search ); // Decode the search parameter
            

            global $wpdb;
            $query = $wpdb->prepare("SELECT ID, guid FROM {$wpdb->posts} WHERE post_type = 'attachment' AND guid LIKE %s LIMIT 1", '%/'.$search);

            $results = $wpdb->get_results($query);
            
            if ($results){
            	$id = $results[0]->ID;
            	$filename = $results[0]->guid;
            }
            else {
            	$id = 0;
            	$filename = '';
            }
            
            
           $result = array(
                  'id'=> $id,
                  'filename' => $filename,
                  'search'=> $search,
            );
            

            return new WP_REST_Response($result, 200);



        }




    }

}
