<?php if (!defined('ABSPATH')) {
    exit;
}
// Exit if accessed directly

if (!class_exists('MM_REST_Settings_V1_Controller')) {

    /**
     * Model that houses the logic of REST Interface with WordPress Settings Library.
     *
     * @since 1.0.0
     */
    class MM_REST_Settings_V1_Controller extends WC_REST_Controller {

        /*
        |--------------------------------------------------------------------------
        | Class Properties
        |--------------------------------------------------------------------------
         */

        /**
         * Property that holds the single main instance of MM_REST_Settings_V1_Controller.
         *
         * @var MM_REST_Settings_V1_Controller
         */
        private static $_instance;


        /**
         * Endpoint namespace.
         *
         * @var string
         */
        protected $namespace = 'mm/v1';

        /**
         * Route base.
         *
         * @var string
         */
        protected $rest_base = 'settings';


        //Other properties
        private $debug_active = null;


        /*
        |--------------------------------------------------------------------------
        | Class Methods
        |--------------------------------------------------------------------------
         */

        /**
         * MM_REST_Settings_V1_Controller constructor.
         *
         * @since 1.0.0
         * @access public
         */
        public function __construct() {

            
             
            // Fires when preparing to serve an API request.
            add_action("rest_api_init", array($this, "register_routes"));
            
           
        }

        /**
         * Register routes for ManageMore roles API.
         *
         * @since 1.0.0
         * @access public
         */
        public function register_routes() {


            register_rest_route($this->namespace, '/' . $this->rest_base , array(
                array(
                    'methods' => WP_REST_Server::READABLE,
                    'callback' => array($this, 'get_settings'),
                    'permission_callback' => array($this, 'get_settings_permissions_check'),
                ),
                array(
                    'methods' => WP_REST_Server::EDITABLE,
                    'callback' => array($this, 'get_settings'),
                    'permission_callback' => array($this, 'get_settings_permissions_check'),
                ),


            ));



        }

 
        /**
         * Check if a given request has access to read settings. We may add a real check later, for now we always allow
         *
         * @param  WP_REST_Request $request Full details about the request.
         * @return WP_Error|boolean
         */
        public function get_settings_permissions_check($request) {
        	

            return true;
        }




        /**
         * Get Wordpress/WooCommerce settings. Also Sets ManageMore Info 
         *
         * @param WP_REST_Request         $request
         *
         * @since 1.0.0
         * @access public
         * @return WP_Error|WP_REST_Request
         */
        public function get_settings($request) {
        	
        	 global $wpdb,$wp_version;
        	 
        	 $mm_build = $request['mm_build'];
        	 $mm_serial_no = $request['mm_serial_no'];
        	 $mm_last_communication = current_time('mysql');
           // Get IP Address from multiple possible headers
           $mm_last_ip_address = $request->get_header('X-Forwarded-For');
           if (!$mm_last_ip_address) {
               $mm_last_ip_address = $request->get_header('HTTP_X_FORWARDED_FOR');
           }
           if (!$mm_last_ip_address) {
               $mm_last_ip_address = $request->get_header('REMOTE_ADDR');
           }
           if (!$mm_last_ip_address) {
               $mm_last_ip_address = $_SERVER['HTTP_X_FORWARDED_FOR'] ?? $_SERVER['REMOTE_ADDR'] ?? 'UNKNOWN';
           }

           $this->debug_log('Processing REST Request from ManageMore - Serial No.: ' . $mm_serial_no . ' Build: ' . $mm_build . ' IP: ' . $mm_last_ip_address . ' Timestamp: ' . $mm_last_communication );

           add_option('mm_build',$mm_build);
           add_option('mm_serial_no',$mm_serial_no);
           add_option('mm_last_communication',$mm_last_communication);
           add_option('mm_last_ip_address',$mm_last_ip_address);
        	 
        	
           //Get settings from WordPress/WooCommerce to Pass Back to MM
           //Wordpress Settings
           $permalink_structure = get_option('permalink_structure');
           //Our Plug-In Settings
           $mm_connector_version = MM_CURRENT_VERSION;
           $mm_indirect_tax_enable = get_option('mm_indirect_tax_enable');
           $mm_customer_pricing_enable = get_option('mm_customer_pricing_enable');
           //$mm_allow_price_lower_than_minimum = get_option('mm_allow_price_lower_than_minimum');  //deprecated 4.6.0
           $mm_allow_sale_price_if_lower = get_option('mm_allow_sale_price_if_lower');
           //WooCommerce Settings
           $wpp_active = MM_Connector_Functions::is_plugin_active('woocommerce-wholesale-prices-premium/woocommerce-wholesale-prices-premium.bootstrap.php');
           $wp_active = MM_Connector_Functions::is_plugin_active('woocommerce-wholesale-prices/woocommerce-wholesale-prices.bootstrap.php');
           $woo_active = MM_Connector_Functions::is_plugin_active('woocommerce/woocommerce.php');
           $woo_version = WC_VERSION;
           $woo_brands_active = MM_Connector_Functions::is_plugin_active('woocommerce-brands/woocommerce-brands.php');
           $woo_allow_auto_password = get_option('woocommerce_registration_generate_password'); //This setting is about sending the user a message on new password, but must be set to allow auto generated passwords
           $woo_last_order_id = $wpdb->get_var( "SELECT MAX(ID)
                                      FROM {$wpdb->prefix}posts
                                      WHERE post_type = 'shop_order'
                                      AND post_status IN ( 'wc-processing', 'wc-completed' )" );
           $woo_weight_unit = get_option('woocommerce_weight_unit');

          //Ultimate Member Settings
          $um_active = MM_Connector_Functions::is_plugin_active('ultimate-member/ultimate-member.php');
          $um_woo_active = MM_Connector_Functions::is_plugin_active('um-woocommerce/um-woocommerce.php');
          //Check whether wholesale roles allow profile edit. If not, user will not be able to update license image, or any other fields we allow updating
          if ($wpp_active && $um_active){
             $role_data = UM()->roles()->role_data( 'wholesale_customer' );
             $price_1_can_edit_profile = (isset($role_data['can_edit_profile']) && $role_data['can_edit_profile']) ? 1 : 0;
             $role_data = UM()->roles()->role_data( 'mm_price_2' );
             $price_2_can_edit_profile = (isset($role_data['can_edit_profile']) && $role_data['can_edit_profile']) ? 1 : 0;
             $role_data = UM()->roles()->role_data( 'mm_price_3' );
             $price_3_can_edit_profile = (isset($role_data['can_edit_profile']) && $role_data['can_edit_profile']) ? 1 : 0;
             $role_data = UM()->roles()->role_data( 'mm_price_4' );
             $price_4_can_edit_profile = (isset($role_data['can_edit_profile']) && $role_data['can_edit_profile']) ? 1 : 0;
             $role_data = UM()->roles()->role_data( 'mm_price_5' );
             $price_5_can_edit_profile = (isset($role_data['can_edit_profile']) && $role_data['can_edit_profile']) ? 1 : 0;
             $role_data = UM()->roles()->role_data( 'mm_price_6' );
             $price_6_can_edit_profile = (isset($role_data['can_edit_profile']) && $role_data['can_edit_profile']) ? 1 : 0;
             $role_data = UM()->roles()->role_data( 'mm_price_7' );
             $price_7_can_edit_profile = (isset($role_data['can_edit_profile']) && $role_data['can_edit_profile']) ? 1 : 0;
             $role_data = UM()->roles()->role_data( 'mm_price_8' );
             $price_8_can_edit_profile = (isset($role_data['can_edit_profile']) && $role_data['can_edit_profile']) ? 1 : 0;
          }
          else {
          	 $price_1_can_edit_profile = 0;
          	 $price_2_can_edit_profile = 0;
          	 $price_3_can_edit_profile = 0;
          	 $price_4_can_edit_profile = 0;
          	 $price_5_can_edit_profile = 0;
          	 $price_6_can_edit_profile = 0;
          	 $price_7_can_edit_profile = 0;
          	 $price_8_can_edit_profile = 0;
          }  
          
            
           $result = array(
                  'wp_version'=> $wp_version,
                  'woo_version' => $woo_version,
                  'mm_connector_version' => $mm_connector_version,
                  'php_version' => phpversion(),
                  'permalink_structure' => $permalink_structure,
                  'mm_indirect_tax_enable'=> $mm_indirect_tax_enable,
                  'mm_customer_pricing_enable' => $mm_customer_pricing_enable,
                  //'mm_allow_price_lower_than_minimum'=> $mm_allow_price_lower_than_minimum,  //deprecated 4.6.0
                  //'mm_allow_sale_price_if_lower'=> $mm_allow_sale_price_if_lower,    //depricated 4.6.0
                  'woo_active' => $woo_active,
                  'woo_brands_active' => $woo_brands_active,
                  'wpp_active' => $wpp_active,
                  'wp_active' => $wp_active,
                  'woo_allow_auto_password' => $woo_allow_auto_password,
                  'woo_last_order_id' => $woo_last_order_id,
                  'woo_weight_unit' => $woo_weight_unit,
                  'um_active' => $um_active,
                  'um_woo_active' => $um_woo_active,
                  'price_1_can_edit_profile' => $price_1_can_edit_profile,
                  'price_2_can_edit_profile' => $price_2_can_edit_profile,
                  'price_3_can_edit_profile' => $price_3_can_edit_profile,
                  'price_4_can_edit_profile' => $price_4_can_edit_profile,
                  'price_5_can_edit_profile' => $price_5_can_edit_profile,
                  'price_6_can_edit_profile' => $price_6_can_edit_profile,
                  'price_7_can_edit_profile' => $price_7_can_edit_profile,
                  'price_8_can_edit_profile' => $price_8_can_edit_profile,
            );
            

            return new WP_REST_Response($result, 200);



        }

        protected function debug_log($message) {
        	  if ($this->debug_active === null){
        	  	$this->debug_active = MM_Debug_Mode::is_active(MM_Debug_Mode::REST_API);
        	  }
            if ($this->debug_active) {
               MM_Connector_Functions::debug_log($message, MM_Debug_Mode::REST_API);
            }
        }



    }

}
