<?php if (!defined('ABSPATH')) {
    exit;
}
// Exit if accessed directly

if (!class_exists('MM_REST_Trade_Class_V1_Controller')) {

    /**
     * Model that houses the logic of REST Interface with Trade Classes, and Restrictions.
     *
     * @since 1.0.0
     */
     class MM_REST_Trade_Class_V1_Controller extends WC_REST_Controller {

             /*
             |--------------------------------------------------------------------------
             | Class Properties
             |--------------------------------------------------------------------------
              */

             /**
              * Property that holds the single main instance of MM_REST_Media_V1_Controller.
              *
              * @var MM_REST_Trade_Class_V1_Controller
              */
             private static $_instance;


             /**
              * Endpoint namespace.
              *
              * @var string
              */
             protected $namespace = 'mm/v1';

             /**
              * Route base.
              *
              * @var string
              */
             protected $rest_base = 'trade-class';


             //Other properties
             private $debug_active = null;


             /*
             |--------------------------------------------------------------------------
             | Class Methods
             |--------------------------------------------------------------------------
              */

             /**
              * MM_REST_Media_V1_Controller constructor.
              *
              * @since 1.0.0
              * @access public
              */
             public function __construct() {



                 
                 // Fires when preparing to serve an API request.
                 add_action("rest_api_init", array($this, "register_routes"));
                 
                
             }

             /**
              * Register routes for ManageMore roles API.
              *
              * @since 1.0.0
              * @access public
              */
              public function register_routes() {
                  register_rest_route($this->namespace, '/' . $this->rest_base, array(
                      'methods'  => 'POST',
                      'callback' => array($this, 'update_trade_classes'),
                      'permission_callback' => function () {
                          return current_user_can('manage_options'); // Restrict to admins
                      },
                  ));
              }

              public function update_trade_classes(WP_REST_Request $request) {
                  global $wpdb;
                  $data = $request->get_json_params();

                  if (!isset($data['trade_classes']) || !isset($data['trade_class_restrictions']) || !is_array($data['trade_classes']) || !is_array($data['trade_class_restrictions'])) {
                      return new WP_REST_Response([
                          'success' => false,
                          'message' => 'Invalid request format.'
                      ], 400);
                  }

                  $delete_all = isset($data['all']) && $data['all'] === true;
                  $table_trade_class = $wpdb->prefix . 'mm_trade_class';
                  $table_restrict_names = $wpdb->prefix . 'mm_trade_class_restrict_names';
                  $table_restrictions = $wpdb->prefix . 'mm_trade_class_restrictions';

                  $updated = false;  //flag to indicate restrictions were updated

                  if ($delete_all) {
                      // Delete all trade classes and restrictions
                      $wpdb->query("DELETE FROM $table_trade_class");
                      $wpdb->query("DELETE FROM $table_restrict_names");
                      $wpdb->query("DELETE FROM $table_restrictions");
                  }

                  // Insert or update trade classes
                  foreach ($data['trade_classes'] as $trade_class) {
                      $wpdb->replace(
                          $table_trade_class,
                          array(
                              'trade_class_id' => intval($trade_class['trade_class_id']),
                              'trade_class_name' => sanitize_text_field($trade_class['trade_class_name']),
                          ),
                          array('%d', '%s')
                      );
                  }

                  // Insert or update trade class restrictions
                  foreach ($data['trade_class_restrictions'] as $restriction) {
                      $trade_class_restrict_id = $restriction['trade_class_restrict_id'];
                      $result = $wpdb->replace(
                          $table_restrict_names,
                          array(
                              'trade_class_restrict_id' => $trade_class_restrict_id,
                              'restriction_name' => sanitize_text_field($restriction['restriction_name']),
                          ),
                          array('%d', '%s')
                      );

                      //if we did not delete all, remove restrictions first, before re-adding
                      if (!$delete_all) {
                         $wpdb->query("DELETE FROM $table_restrictions WHERE trade_class_restrict_id=".$restriction['trade_class_restrict_id']);
                      }

                      foreach ($restriction['restrictions'] as $restriction_entry) {
                          $wpdb->replace(
                              $table_restrictions,
                              array(
                                  'trade_class_restrict_id' => $trade_class_restrict_id,
                                  'state' => sanitize_text_field($restriction_entry['state']),
                                  'trade_class_id' => intval($restriction_entry['id']),
                              ),
                              array('%d', '%s', '%d')
                          );
                      }

                      //Remove Cache
                      $this->debug_log('Removing Cache for Trade Class Restrict Id: '.$trade_class_restrict_id);
                      MM_Object_Cache::delete_by_scope('trade_class_restrict_id', $trade_class_restrict_id, 'tc_restriction');

                  }


                  

                  return new WP_REST_Response([
                      'success' => true,
                      'message' => $delete_all ? 'All trade classes and restrictions deleted and replaced successfully.' : 'Trade classes and restrictions updated without deletion.'
                  ], 200);
              }



        protected function debug_log($message) {
        	  if ($this->debug_active === null){
        	  	$this->debug_active = MM_Debug_Mode::is_active(MM_Debug_Mode::REST_API);
        	  }
            if ($this->debug_active) {
               MM_Connector_Functions::debug_log($message, MM_Debug_Mode::REST_API);
            }
        }


     }

}
