<?php

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly.
}

class MM_Camera_Snapshot {

    public function __construct() {
        add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
        add_action( 'wp_ajax_mm_upload_snapshot', array( $this, 'handle_snapshot_upload' ) );
    }

    /**
     * Enqueue the external JavaScript file.
     */
    public function enqueue_scripts() {
        wp_enqueue_script(
            'mm-connector-camera-snapshot',
            MM_JS_URL . 'mm-connector-camera-snapshot.js',
            array( 'jquery' ),
            null,
            true
        );

        // Pass the AJAX URL to the JavaScript file.
        wp_localize_script(
            'mm-connector-camera-snapshot',
            'mm_snapshot_ajax',
            array( 'ajax_url' => admin_url( 'admin-ajax.php' ) )
        );
    }

    /**
     * AJAX handler for uploading the snapshot image.
     */
    public function handle_snapshot_upload() {
        if ( ! current_user_can( 'upload_files' ) ) {
            wp_send_json_error( array( 'message' => 'Permission denied' ) );
        }

        if ( isset( $_POST['image'] ) ) {
            $image_data = $_POST['image'];
            $image_data = str_replace( 'data:image/png;base64,', '', $image_data );
            $image_data = base64_decode( $image_data );

            $upload_dir = wp_upload_dir();
            $filename   = 'snapshot_' . time() . '.png';
            $file_path  = trailingslashit( $upload_dir['path'] ) . $filename;

            if ( ! file_put_contents( $file_path, $image_data ) ) {
                wp_send_json_error( array( 'message' => 'Failed to save image.' ) );
            }

            $attachment = array(
                'post_mime_type' => 'image/png',
                'post_title'     => sanitize_file_name( $filename ),
                'post_content'   => '',
                'post_status'    => 'inherit',
            );

            $attach_id = wp_insert_attachment( $attachment, $file_path );
            require_once ABSPATH . 'wp-admin/includes/image.php';
            $attach_data = wp_generate_attachment_metadata( $attach_id, $file_path );
            wp_update_attachment_metadata( $attach_id, $attach_data );

            wp_send_json_success( array( 'attachment_id' => $attach_id ) );
        }

        wp_send_json_error( array( 'message' => 'No image data received' ) );
    }
}

new MM_Camera_Snapshot();
?>
