<?php 

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

//Require List Table Class we are extending
if ( ! class_exists( 'WP_List_Table' ) ) {
   require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

class MM_Customer_Price_List extends WP_List_Table {

   /** Class constructor */
   public function __construct() {

   parent::__construct( [
      'singular' => __( 'Customer Price', 'managemore-connector' ), //singular name of the listed records
      'plural' => __( 'Customer Prices', 'managemore-connector' ), //plural name of the listed records
      'ajax' => false //should this table support ajax?

      ] );

   }

   /**
   * Retrieve customer price data from the database
   *
   * @param int $per_page
   * @param int $page_number
   *
   * @return mixed
   */
   public static function get_customer_prices( ) {

      global $wpdb;


      $sql = "SELECT * FROM {$wpdb->prefix}mm_customer_price AS ccp";
      
      if (!empty($_SESSION['mm_pricing_custid']) && !empty($_SESSION['mm_pricing_sku'])) {
         $custid = $_SESSION['mm_pricing_custid'];
         $sku = $_SESSION['mm_pricing_sku'];
         $sql .= $wpdb->prepare(" JOIN {$wpdb->prefix}usermeta AS um ON ccp.user_id = um.user_id
                   JOIN {$wpdb->prefix}postmeta AS pm ON ccp.product_id = pm.post_id
                   WHERE um.meta_key = 'mm_field_CID'
                   AND LOWER(um.meta_value) = LOWER('$custid')
                   AND pm.meta_key = '_sku'
                   AND LOWER(pm.meta_value) = LOWER('$sku')");
      } 
      elseif ( ! empty( $_SESSION['mm_pricing_custid'] ) ) {
      	
      	$custid = $_SESSION['mm_pricing_custid'];
         $sql .= $wpdb->prepare(" JOIN {$wpdb->prefix}usermeta AS um ON ccp.user_id = um.user_id
                   WHERE um.meta_key = 'mm_field_CID'
                   AND LOWER(um.meta_value) = LOWER('$custid')");
      	
      }
      elseif (!empty($_SESSION['mm_pricing_sku'])) {
          $sku = $_SESSION['mm_pricing_sku'];
          $sql .= $wpdb->prepare(" JOIN {$wpdb->prefix}postmeta AS pm ON ccp.product_id = pm.post_id
                   WHERE pm.meta_key = '_sku'
                   AND LOWER(pm.meta_value) = LOWER('$sku')");
      }
      

      if ( ! empty( $_REQUEST['orderby'] ) ) {
         $sql .= ' ORDER BY ccp.' . esc_sql( $_REQUEST['orderby'] );
         $sql .= ! empty( $_REQUEST['order'] ) ? ' ' . esc_sql( $_REQUEST['order'] ) : ' ASC';
      }

      

      $result = $wpdb->get_results( $sql, 'ARRAY_A' );

      return $result;

   }

   /**
   * Delete a customer price record.
   *
   * @param int $id customer price ID
   */
   public static function delete_customer_price( $id ) {
      global $wpdb;

      $wpdb->delete(
         "{$wpdb->prefix}mm_customer_price",
         [ 'id' => $id ],
         [ '%d' ]
         );
      }

   /**
   * Delete all customer price records.
   *
   * @param int $id customer price ID
   */
   public static function delete_all_customer_prices() {
      global $wpdb;
      $delete = $wpdb->query("TRUNCATE TABLE {$wpdb->prefix}mm_customer_price");
   }


   /**
   * Returns the count of records in the database.
   *
   * @return null|string
   */
   public static function record_count() {
      global $wpdb;

      $sql = "SELECT COUNT(*) FROM {$wpdb->prefix}mm_customer_price";

      return $wpdb->get_var( $sql );
   }

   /** Text displayed when no customer price data is available */
   public function no_items() {
      echo esc_html( 'No customer prices available.' );
   }

   /**
   * Render a column when no column specific method exists.
   *
   * @param array $item
   * @param string $column_name
   *
   * @return mixed
   */
   public function column_default( $item, $column_name ) {

   	global $woocommerce;
   	
      switch ( $column_name ) {
      case 'user_id':
         $user_id = $item[ $column_name ];
         $cust_id = get_user_meta($user_id,'mm_field_CID',true) ;
         $first_name = get_user_meta($user_id,'billing_first_name',true) ;
         $last_name = get_user_meta($user_id,'billing_last_name',true) ;
         $customer_info = $cust_id . ' - ' . $first_name. ' '. $last_name;
         return $customer_info;
      case 'product_id':
         $product_id = $item[ $column_name ];      
         $woo_product = wc_get_product( $product_id );
         if($woo_product != FALSE){
            $product_info =  $woo_product->get_sku() ;
         }
         else {
            $product_info = '[ '.$product_id.' ]';
         }
         return $product_info;
      case 'product_name':
         $product_id = $item[ 'product_id' ];      
         $woo_product = wc_get_product( $product_id );
         if($woo_product != FALSE){
            $product_name = $woo_product->get_name() ;      
         }
         else {
          	$product_name = 'Unknown Product';
         }
         return $product_name;
      case 'method':
         $method_description = array('','Fixed Price','Discount Percent','Markup over Last Cost','Markup over Average Cost','Fixed amount over Last Cost',
                                        'Fixed amount over Average Cost','Price Level 1','Price Level 2','Price Level 3','Price Level 4','Price Level 5',
                                        'Price Level 6','Price Level 7','Price Level 8'); //note mm values are 1-based
         $method = $item[ $column_name ];
         return $method_description[$method];
      case 'amount':
         $amount = number_format($item[ $column_name ],2);     
         return $amount;
      case 'date_range':
         $start_date = $item[ 'start_date' ];
         $end_date = $item[ 'end_date' ];
         $formatted_start_date = ($start_date == '') ? '' : date_format(date_create_from_format('Ymd',$start_date),'m/d/Y') ;
         $formatted_end_date = ($end_date == '') ? '' : date_format(date_create_from_format('Ymd',$end_date),'m/d/Y') ;
         $date_info = '';
         if ($start_date != '' && $end_date != ''){
            $date_info = $formatted_start_date . ' - ' . $formatted_end_date ;
         }
         elseif ($start_date != '') {
         	$date_info = 'Starting ' . $formatted_start_date ; 
         }
         elseif ($end_date != ''){
         	$date_info = 'Ending ' . $formatted_end_date ; 
         }
         return $date_info;
      
      default:
         return print_r( $item, true ); //Show the whole array for troubleshooting purposes
      }
   }

   /**
   * Render the bulk edit checkbox
   *
   * @param array $item
   *
   * @return string
   */
   function column_cb( $item ) {
      return sprintf(
      '<input type="checkbox" name="bulk-delete[]" value="%s" />', $item['id']
      );
   }

   /**
   * Associative array of columns
   *
   * @return array
   */
   function get_columns() {
      $columns = [
      'cb' => '<input type="checkbox" />',
      'user_id' => __( 'Customer', 'managemore-connector' ),
      'product_id' => __( 'SKU Item', 'managemore-connector' ),
      'product_name' => __( 'Product Name', 'managemore-connector' ),
      'method' => __( 'Method', 'managemore-connector' ),
      'amount' => __( 'Amount', 'managemore-connector' ),
      'date_range' => __( 'Date Range', 'managemore-connector' )
      ];

      return $columns;
   }

   /**
   * Columns to make sortable.
   *
   * @return array
   */
   public function get_sortable_columns() {
      $sortable_columns = array(
      'user_id' => array( 'user_id', true ),
      'product_id' => array( 'product_id', false )
      );

      return $sortable_columns;
   }

   /**
   * Returns an associative array containing the bulk action
   *
   * @return array
   */
   public function get_bulk_actions() {
      $actions = [
      'bulk-delete' => 'Delete',
      'delete-all' => 'Delete All'
      ];

      return $actions;
   }   

   /**
   * Handles data query and filter, sorting, and pagination.
   */
   public function prepare_items() {

 
      $this->_column_headers = $this->get_column_info();

       /** Process bulk action */
      $this->process_bulk_action();

      // Save the filter parameters in the session.
      if (isset($_REQUEST['mm_pricing_custid'])) {
         $_SESSION['mm_pricing_custid'] = sanitize_key($_REQUEST['mm_pricing_custid']);
      }

      if (isset($_REQUEST['mm_pricing_sku'])) {
         $_SESSION['mm_pricing_sku'] = sanitize_key($_REQUEST['mm_pricing_sku']);
      }


      $data =  self::get_customer_prices();

      $per_page = $this->get_items_per_page( 'customer_prices_per_page', 50 );
      $current_page = $this->get_pagenum();

      // Calculate the total number of pages.
      $total_pages = ceil(count($data) / $per_page);

      // Slice the data based on the current page number and per page.
      $data_slice = array_slice($data, (($current_page-1)*$per_page), $per_page);

      $this->set_pagination_args( [
        'total_items' => count($data), // Total number of items
        'per_page' => $per_page, //Items Per Page
        'total_pages' => $total_pages // Total number of pages

      ] );

      $this->items = $data_slice;
   }


   /**
   * Add Filters to Customer Pricing page
   */
   public function extra_tablenav($which) {

   if ($which === 'top') {
     echo '<div class="alignleft actions bulkactions">';
     echo '<label for="mm_pricing_custid">Account No:</label>&nbsp;';
     echo '<input type="text" id="mm_pricing_custid" name="mm_pricing_custid" value="' . esc_attr($_SESSION['mm_pricing_custid']) . '" />&nbsp;&nbsp;';
     echo '<label for="mm_pricing_sku">SKU:</label>&nbsp;';
     echo '<input type="text" id="mm_pricing_sku" name="mm_pricing_sku" value="' . esc_attr($_SESSION['mm_pricing_sku']) . '" />&nbsp;&nbsp;';
     echo '<input type="hidden" name="page" value="' . esc_attr($_REQUEST['page']) . '" />';
     submit_button('Apply Filter', 'primary', 'filter_action', false, array('id' => 'post-query-submit'));
     echo '</div>';
   }



   }



   /**
   * Process the Bulk Action
   */
   public function process_bulk_action() {

      //Detect when a single delete action is being triggered... (not currently implemented, would require custom column function)
/*      if ( 'delete' === $this->current_action() ) {

         // In our file that handles the request, verify the nonce.
         $nonce = esc_attr( $_REQUEST['_wpnonce'] );

         if ( ! wp_verify_nonce( $nonce, 'mm_delete_customer_price' ) ) {
            die( 'Invalid Request' );
         }
         else {
            self::delete_customer_price( absint( $_GET['customer'] ) );

            wp_redirect( esc_url( add_query_arg() ) );
            exit;
         }

      }
*/

      // If the delete bulk action is triggered (delete selected items)
      if ( ( isset( $_POST['action'] ) && $_POST['action'] == 'bulk-delete' )
      || ( isset( $_POST['action2'] ) && $_POST['action2'] == 'bulk-delete' )
      ) {

         $delete_ids = esc_sql( $_POST['bulk-delete'] );

         // loop over the array of record IDs and delete them
         foreach ( $delete_ids as $id ) {
         self::delete_customer_price( $id );

         }

         wp_redirect( esc_url( add_query_arg() ) );
         exit;
      }
      
      // If the delete all bulk action is triggered (delete all items)
      if ( ( isset( $_POST['action'] ) && $_POST['action'] == 'delete-all' )
      || ( isset( $_POST['action2'] ) && $_POST['action2'] == 'delete-all' )
      ) {

         self::delete_all_customer_prices( );


         wp_redirect( esc_url( add_query_arg() ) );
         exit;
      }
      
   }

} 