<?php
if (!defined('ABSPATH')) exit; // Exit if accessed directly

if (!class_exists('WP_List_Table')) {
    require_once(ABSPATH . 'wp-admin/includes/class-wp-list-table.php');
}

class MM_Trade_Class_Restriction_List extends WP_List_Table {
    private $table_name;
    private $trade_class_table;
    private $restrict_names_table;

    public function __construct() {
        global $wpdb;
        $this->table_name = $wpdb->prefix . 'mm_trade_class_restrictions';
        $this->trade_class_table = $wpdb->prefix . 'mm_trade_class';
        $this->restrict_names_table = $wpdb->prefix . 'mm_trade_class_restrict_names';

        parent::__construct([
            'singular' => __('Trade Class Restriction', 'managemore-connector'),
            'plural'   => __('Trade Class Restrictions', 'managemore-connector'),
            'ajax'     => false
        ]);
    }

    public function get_columns() {
        return [
            'cb'                     => '<input type="checkbox" />',
            'state'                  => __('State', 'managemore-connector'),
            'trade_class_name'        => __('Trade Class Name (ID)', 'managemore-connector'),
            'restriction_name'        => __('Trade Class Restriction Name (ID)', 'managemore-connector')
        ];
    }

    public function get_sortable_columns() {
        return [
            'state' => ['state', true]
        ];
    }

    public function column_cb($item) {
        return sprintf('<input type="checkbox" name="bulk-delete[]" value="%s" />', $item['trade_class_restrict_id']);
    }

    public function column_state($item) {
        return esc_html($item['state']);
    }

    public function column_trade_class_name($item) {
        return esc_html($item['trade_class_name']) . ' (' . esc_html($item['trade_class_id']) . ')';
    }

    public function column_restriction_name($item) {
        return esc_html($item['restriction_name']) . ' (' . esc_html($item['trade_class_restrict_id']) . ')';
    }

    public function prepare_items() {
        global $wpdb;
        $per_page = $this->get_items_per_page( 'restrictions_per_page', 50 );
        $current_page = $this->get_pagenum();
        $orderby = isset($_REQUEST['orderby']) && in_array($_REQUEST['orderby'], ['state']) ? $_REQUEST['orderby'] : 'state';
        $order = isset($_REQUEST['order']) && in_array($_REQUEST['order'], ['asc', 'desc']) ? $_REQUEST['order'] : 'asc';
        
        $sql = "SELECT r.state, t.trade_class_name, rn.restriction_name, r.trade_class_id, r.trade_class_restrict_id
                FROM {$this->table_name} r
                LEFT JOIN {$this->trade_class_table} t ON r.trade_class_id = t.trade_class_id
                LEFT JOIN {$this->restrict_names_table} rn ON r.trade_class_restrict_id = rn.trade_class_restrict_id
                ORDER BY $orderby $order
                LIMIT $per_page OFFSET " . (($current_page - 1) * $per_page);
        
        $this->items = $wpdb->get_results($sql, 'ARRAY_A');
        $this->set_pagination_args([
            'total_items' => $wpdb->get_var("SELECT COUNT(*) FROM {$this->table_name}"),
            'per_page'    => $per_page
        ]);
    }

    public function no_items() {
        esc_html_e('No trade class restrictions found.', 'managemore-connector');
    }
}
