<?php 

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if (!class_exists('MM_WOO_EXTENDER')) {

    /**
     * Model that houses the logic of MM WooCommerce Related Functions (Custom Woo Fields, Indirect Taxes, Customer Pricing, etc.)
     *
     * @since 1.0.0
     */
    class MM_WOO_EXTENDER {
    	
        /*
        |--------------------------------------------------------------------------
        | Class Properties
        |--------------------------------------------------------------------------
         */
        public $show_account_status_section = 0 ;
        public $indirect_tax_enabled = 0 ;
        public $mm_indirect_tax_allow_cart_detail = 0 ;
        public $mm_indirect_tax_allow_cart_warning = 0 ;
        public $mm_allow_pdf_upload = 0 ;
        public $mm_field_meta_keys = array(); //array of custom field meta names
        private $mm_pay_balance_sku = 'WEBPMT';      // Variable for product SKU used for Customer Payment
        private $mm_cust_bal_meta_field = 'mm_field_B';  // Variable for user meta field used to store Customer Balance
        private $custom_cart_data = array();
        private $gateway_surcharge_amount = 0.00;
        private $gateway_surcharge_name = '';
        

        /*
        |--------------------------------------------------------------------------
        | Class Methods
        |--------------------------------------------------------------------------
         */

        /**
        * This method is used to add additional fields to the WooCommerce product page for 'simple' type products.
        * These fields include UPC/Barcode, Average Cost, Last Cost, Lowest Price, Taxable Basis, Product Net Weight, etc.
        * It also checks if the indirect tax is enabled. If so, it adds an additional field for the Indirect Tax Type.
        * 
        * Deprecated 4.1.1 - These fields are now on a custom tab (mm_custom_tab_product_data_fields)
        *
        * @global WP_Post $post The post object of the current post.
        * @global WC_Product $product_object The WooCommerce product object.
        */
/*        public function simple_product_woo_fields() {
            
            global $post,$product_object;



            if( $product_object->get_type() == 'simple' ) {


                ?>
                <div class="options-group">
                <h3 style="padding: 8px 12px 0 12px; font-size: 14px">Additional Information</h3>
                <p><i>This information comes from ManageMore Business Software</i></p>

                <?php 


                  woocommerce_wp_text_input( 
                     	array( 
                    		'id'                => 'mm_product_upc', 
                    		'label'             => __( 'UPC / Barcode', 'managemore-connector' ), 
                    		'description'       => __( 'The UPC or other code assigned to the product.  May be used for searches.', 'managemore-connector' ),
                        'desc_tip'          => true,
                     	'type'              => 'text',
                    		'maxlength'         => 25,
                        )
                    );


                  woocommerce_wp_text_input( 
                     	array( 
                    		'id'                => 'mm_product_cost', 
                    		'label'             => __( 'Average Cost', 'managemore-connector' ), 
                    		'description'       => __( 'The product average cost. May be used to calculate Indirect Tax or Customer Price', 'managemore-connector' ),
                        'desc_tip'          => true,
                    		'type'              => 'number', 
          		          'custom_attributes' => array(
          				              'step' 	=> '.00001',
          				              'min'	=> '0'
                    	           )
                        )
                    );


                  woocommerce_wp_text_input( 
                     	array( 
                    		'id'                => 'mm_product_last_cost', 
                    		'label'             => __( 'Last Cost', 'managemore-connector' ), 
                    		'description'       => __( 'The product last cost. May be used to calculate Customer Price', 'managemore-connector' ),
                        'desc_tip'          => true,
                    		'type'              => 'number', 
          		          'custom_attributes' => array(
          				              'step' 	=> '.00001',
          				              'min'	=> '0'
                    	           )
                        )
                    );

                  woocommerce_wp_text_input( 
                     	array( 
                    		'id'                => 'mm_product_lowest_price', 
                    		'label'             => __( 'Lowest Price', 'managemore-connector' ), 
                    		'description'       => __( 'The product lowest selling price. May be used to calculate Customer Price', 'managemore-connector' ),
                        'desc_tip'          => true,
                    		'type'              => 'number', 
          		          'custom_attributes' => array(
          				              'step' 	=> '.01',
          				              'min'	=> '0'
                    	           )
                        )
                    );


                  woocommerce_wp_text_input( 
                     	array( 
                    		'id'                => 'mm_product_basis_1', 
                    		'label'             => __( 'Taxable Basis 1', 'managemore-connector' ), 
                    		'description'       => __( 'The product basis. May be used to calculate Indirect Tax', 'managemore-connector' ),
                        'desc_tip'          => true,
                    		'type'              => 'number', 
          		          'custom_attributes' => array(
          				              'step' 	=> '.0001',
          				              'min'	=> '0'
                    	           )
                        )
                    );

                  woocommerce_wp_text_input( 
                     	array( 
                    		'id'                => 'mm_product_basis_2', 
                    		'label'             => __( 'Taxable Basis 2', 'managemore-connector' ), 
                    		'description'       => __( 'The product basis. May be used to calculate Indirect Tax', 'managemore-connector' ),
                        'desc_tip'          => true,
                    		'type'              => 'number', 
          		          'custom_attributes' => array(
          				              'step' 	=> '.0001',
          				              'min'	=> '0'
                    	           )
                        )
                    );

                  woocommerce_wp_text_input( 
                     	array( 
                    		'id'                => 'mm_product_basis_3', 
                    		'label'             => __( 'Taxable Basis 3', 'managemore-connector' ), 
                    		'description'       => __( 'The product basis. May be used to calculate Indirect Tax', 'managemore-connector' ),
                        'desc_tip'          => true,
                    		'type'              => 'number', 
          		          'custom_attributes' => array(
          				              'step' 	=> '.0001',
          				              'min'	=> '0'
                    	           )
                        )
                    );

                  woocommerce_wp_text_input( 
                     	array( 
                    		'id'                => 'mm_product_net_weight', 
                    		'label'             => __( 'Product Net Weight', 'managemore-connector' ), 
                    		'description'       => __( 'The product net weight. May be used to calculate Indirect Tax', 'managemore-connector' ),
                        'desc_tip'          => true,
                    		'type'              => 'number', 
          		          'custom_attributes' => array(
          				              'step' 	=> '.0001',
          				              'min'	=> '0'
                    	           )
                        )
                    );
                    
                  woocommerce_wp_select( 
                         array( 
                               'id' => 'mm_required_license_type', 
                               'name' => 'mm_required_license_type', 
                               'label' => __( 'Required License Type', 'managemore-connector' ), 
                               'description' => __( 'Select the required license type for this product.', 'managemore-connector' ),
                               'desc_tip' => true,
                               'options' => array(
                                   '0' => __( 'No License Required', 'managemore-connector' ),
                                   '1' => MM_Connector_Functions::mm_get_option('mm_customer_license_1_name'),
                                   '2' => MM_Connector_Functions::mm_get_option('mm_customer_license_2_name'),
                                   '3' => MM_Connector_Functions::mm_get_option('mm_customer_license_3_name'),
                               )
                           )
                       );
                       
                   
                 if ($this->indirect_tax_enabled){
               	 $indirect_tax_type_field  = 'mm_indirect_tax_type';
             	    $indirect_tax_type_value = get_post_meta($post->ID,'mm_indirect_tax_type',true);
                 	?>
                    <p class="form-field">
                    <label for="<?php echo $indirect_tax_type_field; ?>">Indirect Tax Type:</label>
                    <select name="<?php echo $indirect_tax_type_field; ?>"> 
                    <?php  echo MM_Connector_Functions::post_type_selection_options('mm-indirect-tax-type',$indirect_tax_type_value, true);  ?>
                    </select>
                    </p>
                <?php
                 }
                 
                 ?>
               </div>
               <?php   
                    
            }
            
        }
*/


        /**
        * This method is responsible for saving the additional fields added to the WooCommerce product page.
        * It creates a string of region restrictions for a product and saves it.
        * It also loops through all the 'mm_field_meta_keys' to save their corresponding values.
        *
        * @param int $post_id The ID of the current post.
        */
        public function simple_product_woo_fields_save($post_id) {
            // This is our restrictions string.
            $restriction_string = '';
            for ($i = 0; $i < 53; $i++) {
                $checkbox_value = isset($_POST['mm_sale_restrict_region_'.$i]) ? '1' : '0';
                $restriction_string .= $checkbox_value;
            }
            update_post_meta( $post_id, 'mm_sale_restrict_region', $restriction_string );

            foreach ($this->mm_field_meta_keys as $mm_field) {
                if($mm_field != 'mm_sale_restrict_region') {
                    $woocommerce_field = $_POST[$mm_field];
                    if(!empty($woocommerce_field)) {
                        update_post_meta($post_id, $mm_field, esc_attr($woocommerce_field));
                    } else {
                        delete_post_meta($post_id, $mm_field);
                    }
                }
            }
        }

        /**
        * This method is used to add new tab called 'ManageMore Data' and 'Region Restrictions' to the product data meta box in WooCommerce.
        * The new tabs will appear in the product data section when editing a product.
        *
        * @param array $tabs The existing array of tabs.
        * @return array The modified array of tabs including the new tabs.
        */
        public function mm_add_product_tabs( $tabs ) {
            $tabs['managemore_data'] = array(
                'label'    => __( 'ManageMore Data', 'managemore-connector' ),
                'target'   => 'managemore_product_data',
                'class'    => array(),
                'priority' => 80,
            );
            $tabs['state_restrictions'] = array(
                'label'    => __( 'Region Restrictions', 'managemore-connector' ),
                'target'   => 'state_restrictions_product_data',
                'class'    => array(),
                'priority' => 80,
            );
            return $tabs;
        }

        /**
        * This method is responsible for adding additional fields for our custom tabs
        *
        * For the ManageMore data tab, we only display fields for simple products only
        *
        * For the state restrictions tab, It retrieves an array of states, 
        * checks which of these states have been marked as restricted, and displays them.
        *
        * @global WP_Post $post The post object of the current post.
        */
        public function mm_custom_tab_product_data_fields() {

            global $post,$product_object;
            
            
            if( $product_object->get_type() == 'simple' ) {


                ?>
                <div id="managemore_product_data" class="panel woocommerce_options_panel">
                <h3 style="padding: 8px 12px 0 12px; font-size: 14px">ManageMore Data</h3>
                <p><i>This information comes from ManageMore Business Software</i></p>

                <?php 


                  woocommerce_wp_text_input( 
                     	array( 
                    		'id'                => 'mm_product_upc', 
                    		'label'             => __( 'UPC / Barcode', 'managemore-connector' ), 
                    		'description'       => __( 'The UPC or other code assigned to the product.  May be used for searches.', 'managemore-connector' ),
                        'desc_tip'          => true,
                     	'type'              => 'text',
                    		'maxlength'         => 25,
                        )
                    );


                  woocommerce_wp_text_input( 
                     	array( 
                    		'id'                => 'mm_product_cost', 
                    		'label'             => __( 'Average Cost', 'managemore-connector' ), 
                    		'description'       => __( 'The product average cost. May be used to calculate Indirect Tax or Customer Price', 'managemore-connector' ),
                        'desc_tip'          => true,
                    		'type'              => 'number', 
          		          'custom_attributes' => array(
          				              'step' 	=> '.00001',
          				              'min'	=> '0'
                    	           )
                        )
                    );


                  woocommerce_wp_text_input( 
                     	array( 
                    		'id'                => 'mm_product_last_cost', 
                    		'label'             => __( 'Last Cost', 'managemore-connector' ), 
                    		'description'       => __( 'The product last cost. May be used to calculate Customer Price', 'managemore-connector' ),
                        'desc_tip'          => true,
                    		'type'              => 'number', 
          		          'custom_attributes' => array(
          				              'step' 	=> '.00001',
          				              'min'	=> '0'
                    	           )
                        )
                    );

                  woocommerce_wp_text_input( 
                     	array( 
                    		'id'                => 'mm_product_lowest_price', 
                    		'label'             => __( 'Lowest Price', 'managemore-connector' ), 
                    		'description'       => __( 'The product lowest selling price. May be used to calculate Customer Price', 'managemore-connector' ),
                        'desc_tip'          => true,
                    		'type'              => 'number', 
          		          'custom_attributes' => array(
          				              'step' 	=> '.01',
          				              'min'	=> '0'
                    	           )
                        )
                    );


                  woocommerce_wp_text_input( 
                     	array( 
                    		'id'                => 'mm_product_basis_1', 
                    		'label'             => __( 'Taxable Basis 1', 'managemore-connector' ), 
                    		'description'       => __( 'The product basis. May be used to calculate Indirect Tax', 'managemore-connector' ),
                        'desc_tip'          => true,
                    		'type'              => 'number', 
          		          'custom_attributes' => array(
          				              'step' 	=> '.0001',
          				              'min'	=> '0'
                    	           )
                        )
                    );

                  woocommerce_wp_text_input( 
                     	array( 
                    		'id'                => 'mm_product_basis_2', 
                    		'label'             => __( 'Taxable Basis 2', 'managemore-connector' ), 
                    		'description'       => __( 'The product basis. May be used to calculate Indirect Tax', 'managemore-connector' ),
                        'desc_tip'          => true,
                    		'type'              => 'number', 
          		          'custom_attributes' => array(
          				              'step' 	=> '.0001',
          				              'min'	=> '0'
                    	           )
                        )
                    );

                  woocommerce_wp_text_input( 
                     	array( 
                    		'id'                => 'mm_product_basis_3', 
                    		'label'             => __( 'Taxable Basis 3', 'managemore-connector' ), 
                    		'description'       => __( 'The product basis. May be used to calculate Indirect Tax', 'managemore-connector' ),
                        'desc_tip'          => true,
                    		'type'              => 'number', 
          		          'custom_attributes' => array(
          				              'step' 	=> '.0001',
          				              'min'	=> '0'
                    	           )
                        )
                    );

                  woocommerce_wp_text_input( 
                     	array( 
                    		'id'                => 'mm_product_net_weight', 
                    		'label'             => __( 'Product Net Weight', 'managemore-connector' ), 
                    		'description'       => __( 'The product net weight. May be used to calculate Indirect Tax', 'managemore-connector' ),
                        'desc_tip'          => true,
                    		'type'              => 'number', 
          		          'custom_attributes' => array(
          				              'step' 	=> '.0001',
          				              'min'	=> '0'
                    	           )
                        )
                    );
                    
                  woocommerce_wp_select( 
                         array( 
                               'id' => 'mm_required_license_type', 
                               'name' => 'mm_required_license_type', 
                               'label' => __( 'Required License Type', 'managemore-connector' ), 
                               'description' => __( 'Select the required license type for this product.', 'managemore-connector' ),
                               'desc_tip' => true,
                               'options' => array(
                                   '0' => __( 'No License Required', 'managemore-connector' ),
                                   '1' => MM_Connector_Functions::mm_get_option('mm_customer_license_1_name'),
                                   '2' => MM_Connector_Functions::mm_get_option('mm_customer_license_2_name'),
                                   '3' => MM_Connector_Functions::mm_get_option('mm_customer_license_3_name'),
                               )
                           )
                       );
                       
                   
                 if ($this->indirect_tax_enabled){
               	 $indirect_tax_type_field  = 'mm_indirect_tax_type';
             	    $indirect_tax_type_value = get_post_meta($post->ID,'mm_indirect_tax_type',true);
                 	?>
                    <p class="form-field">
                    <label for="<?php echo $indirect_tax_type_field; ?>">Indirect Tax Type:</label>
                    <select name="<?php echo $indirect_tax_type_field; ?>"> 
                    <?php  echo MM_Connector_Functions::post_type_selection_options('mm-indirect-tax-type',$indirect_tax_type_value, true);  ?>
                    </select>
                    </p>
                <?php
                 }
                 
                 ?>
               </div>
               <?php   
                    
            }
            else if( $product_object->get_type() == 'variable' ) {


                ?>
                <div id="managemore_product_data" class="panel woocommerce_options_panel">
                <h3 style="padding: 8px 12px 0 12px; font-size: 14px">ManageMore Data</h3>
                <p><i>This is a variable product. These fields are found on the individual variations.</i></p>
                </div>

                <?php 
            	
            }
            
            
            
            
            //State Restrictions
            $state_array = $this->mm_get_state_array();
            $restriction_string = get_post_meta($post->ID, 'mm_sale_restrict_region', true);

            echo '<div id="state_restrictions_product_data" class="panel woocommerce_options_panel">
                  <h3 style="padding: 8px 12px 0 12px; font-size: 14px">Region Restrictions</h3>
                  <p><i>This information comes from from ManageMore Business Software</i></p>
                  <p>This product is not available in the following States / Countries</p>';
                  
            foreach($state_array as $state_code => $state_name) {
                $index = array_search($state_code, array_keys($state_array));
                $is_checked = (isset($restriction_string[$index]) && $restriction_string[$index] === '1') ? true : false;
                woocommerce_wp_checkbox(array(
                    'id'            => 'mm_sale_restrict_region_'.$index,
                    'label'         => __($state_name, 'managemore-connector'),
                    'value'         => '1',
                    'cbvalue'       => $is_checked,
                    'desc_tip'      => false,
                    'description'   => '',
                ));
            }
            echo '</div>';
        }


        /**
        * This method adds CSS for the icon of the 'ManageMore Data' and 'Region Restrictions' tabs in the WooCommerce product data section.
        * f164 is an excerpt view, f123 is a Location Pin.
        */
        public function mm_add_state_restrictions_icon_css() {
        	   //note: \f230 is a Location Pin
            echo '<style>
                #woocommerce-product-data ul.wc-tabs li.managemore_data_options a::before {
                    font-family: "dashicons";
                    content: "\f164";
                } 
                #woocommerce-product-data ul.wc-tabs li.state_restrictions_options a::before {
                    font-family: "dashicons";
                    content: "\f230";
                } 
            </style>';
        }


       /**
        * Adds extra custom fields to each product variation on the WooCommerce edit product page.
        * The custom fields include UPC/Barcode, Average Cost, Last Cost, Lowest Price, three different 
        * taxable bases, product net weight, and required license type. These can be used for various calculations,
        * such as Indirect Tax or Customer Price. 
        * For indirect tax, it has a conditional check to see if it's enabled and if so, a dropdown selection
        * for the Indirect Tax Type is displayed. The function also checks whether a minimum order price or 
        * quantity is set (which are features of the Wholesale Prices Premium plugin). If these settings are active,
        * a warning is displayed, as they could prevent the Indirect Taxes from being applied.
        *
        * @param int $loop
        * @param array $variation_data
        * @param WP_Post $variation
        */
        public function variable_product_woo_fields($loop, $variation_data, $variation){

        	   global $woocommerce, $post;
        	   
        	   //Added 2.1.3 - Get Wholesales Prices Premium settings for minimum order for Wholesale Price to apply 
        	   //              (these settings can cause Wholesale Price not to override, thus causing indirect tax not to calculate
        	   //
        	   $wwpp_min_order_price =  get_option('wwpp_settings_minimum_order_price',0);
        	   $wwpp_min_order_qty = get_option('wwpp_settings_minimum_order_quantity',0);
        	   $wwpp_min_order_warning = '';
        	   if ($wwpp_min_order_price != 0 || $wwpp_min_order_qty !=0){
        	   	$wwpp_min_order_warning = '<p style="color:red;">WARNING! You have enabled the minimum order settings for Wholesale Prices Premium.  If the minimum order quantity/amount is not met, Indirect Taxes will not be added.</p><br>';
        	   }
        	   

            ?>
               <div class="options-group" style="border-top: 1px solid #DDDDDD;">
                  <h4 style="padding: 10px 0; font-size: 14px">ManageMore Data</h4>
                  <p><i>This information comes from ManageMore Business Software</i></p>
               
            <?php


             $field_value = get_post_meta($variation->ID,'mm_product_upc', true);
             woocommerce_wp_text_input(
                       array(
                        'id'                => 'mm_product_upc['.$loop.']',
                    		'label'             => __( 'UPC / Barcode', 'managemore-connector' ), 
                    		'description'       => __( 'The UPC or other code assigned to the product.  May be used for searches.', 'managemore-connector' ),
                        'desc_tip'          => true,
                    		'wrapper_class'     => 'form-row form-row-full',
                        'value'             => $field_value,
                    		'type'              => 'text',
                    		'maxlength'         => 25,
                        )
                    );


             $field_value = get_post_meta($variation->ID,'mm_product_cost', true);
             woocommerce_wp_text_input(
                       array(
                        'id'                => 'mm_product_cost['.$loop.']',
                    		'label'             => __( 'Average Cost', 'managemore-connector' ), 
                    		'description'       => __( 'The product average cost. May used to calculate Indirect Tax or Customer Price', 'managemore-connector' ),
                        'desc_tip'          => true,
                    		'wrapper_class'     => 'form-row form-row-full',
                        'value'             => $field_value,
                    		'type'              => 'number',
          		          'custom_attributes' => array(
          				              'step' 	=> '.00001',
          				              'min'	=> '0'
                    	           )
                        )
                    );


             $field_value = get_post_meta($variation->ID,'mm_product_last_cost', true);
             woocommerce_wp_text_input(
                       array(
                        'id'                => 'mm_product_last_cost['.$loop.']',
                    		'label'             => __( 'Last Cost', 'managemore-connector' ), 
                    		'description'       => __( 'The product last cost. May used to calculate Customer Price', 'managemore-connector' ),
                        'desc_tip'          => true,
                    		'wrapper_class'     => 'form-row form-row-full',
                        'value'             => $field_value,
                    		'type'              => 'number',
          		          'custom_attributes' => array(
          				              'step' 	=> '.00001',
          				              'min'	=> '0'
                    	           )
                        )
                    );



             $field_value = get_post_meta($variation->ID,'mm_product_lowest_price', true);
             woocommerce_wp_text_input(
                       array(
                        'id'                => 'mm_product_lowest_price['.$loop.']',
                    		'label'             => __( 'Lowest Price', 'managemore-connector' ), 
                    		'description'       => __( 'The product lowest selling price. May used to calculate Customer Price', 'managemore-connector' ),
                        'desc_tip'          => true,
                    		'wrapper_class'     => 'form-row form-row-full',
                        'value'             => $field_value,
                    		'type'              => 'number',
          		          'custom_attributes' => array(
          				              'step' 	=> '.00001',
          				              'min'	=> '0'
                    	           )
                        )
                    );



             $field_value = get_post_meta($variation->ID,'mm_product_basis_1', true);
             woocommerce_wp_text_input(
                       array(
                        'id'                => 'mm_product_basis_1['.$loop.']',
                    		'label'             => __( 'Taxable Basis 1', 'managemore-connector' ), 
                    		'description'       => __( 'The product basis. May used to calculate Indirect Tax', 'managemore-connector' ),
                        'desc_tip'          => true,
                    		'wrapper_class'     => 'form-row form-row-full',
                        'value'             => $field_value,
                    		'type'              => 'number',
          		          'custom_attributes' => array(
          				              'step' 	=> '.0001',
          				              'min'	=> '0'
                    	           )
                        )
                    );


             $field_value = get_post_meta($variation->ID,'mm_product_basis_2', true);
             woocommerce_wp_text_input(
                       array(
                        'id'                => 'mm_product_basis_2['.$loop.']',
                    		'label'             => __( 'Taxable Basis 2', 'managemore-connector' ), 
                    		'description'       => __( 'The product basis. May used to calculate Indirect Tax', 'managemore-connector' ),
                        'desc_tip'          => true,
                    		'wrapper_class'     => 'form-row form-row-full',
                        'value'             => $field_value,
                    		'type'              => 'number',
          		          'custom_attributes' => array(
          				              'step' 	=> '.0001',
          				              'min'	=> '0'
                    	           )
                        )
                    );


             $field_value = get_post_meta($variation->ID,'mm_product_basis_3', true);
             woocommerce_wp_text_input(
                       array(
                        'id'                => 'mm_product_basis_3['.$loop.']',
                    		'label'             => __( 'Taxable Basis 3', 'managemore-connector' ), 
                    		'description'       => __( 'The product basis. May used to calculate Indirect Tax', 'managemore-connector' ),
                        'desc_tip'          => true,
                    		'wrapper_class'     => 'form-row form-row-full',
                        'value'             => $field_value,
                    		'type'              => 'number',
          		          'custom_attributes' => array(
          				              'step' 	=> '.0001',
          				              'min'	=> '0'
                    	           )
                        )
                    );


             $field_value = get_post_meta($variation->ID,'mm_product_net_weight', true);
             woocommerce_wp_text_input(
                       array(
                        'id'                => 'mm_product_net_weight['.$loop.']',
                    		'label'             => __( 'Product Net Weight', 'managemore-connector' ), 
                    		'description'       => __( 'The product net weight. May used to calculate Indirect Tax', 'managemore-connector' ),
                        'desc_tip'          => true,
                    		'wrapper_class'     => 'form-row form-row-full',
                        'value'             => $field_value,
                    		'type'              => 'number',
          		          'custom_attributes' => array(
          				              'step' 	=> '.0001',
          				              'min'	=> '0'
                    	           )
                        )
                    );

             $field_value = get_post_meta($variation->ID,'mm_required_license_type', true);
             $field_value = (!isset($field_value)) ? 0 : $field_value;
             woocommerce_wp_select( 
                         array( 
                               'id' => 'mm_required_license_type['.$loop.']', 
                               'label' => __( 'Required License Type', 'managemore-connector' ), 
                               'description' => __( 'Select the required license type for this product.', 'managemore-connector' ),
                               'desc_tip' => true,
                        		 'wrapper_class'     => 'form-row form-row-full',
                               'value'             => $field_value,
                               'options' => array(
                                   '0' => __( 'No License Required', 'managemore-connector' ),
                                   '1' => MM_Connector_Functions::mm_get_option('mm_customer_license_1_name'),
                                   '2' => MM_Connector_Functions::mm_get_option('mm_customer_license_2_name'),
                                   '3' => MM_Connector_Functions::mm_get_option('mm_customer_license_3_name'),
                               )
                           )
                       );


               if ($this->indirect_tax_enabled)	{
            	   $indirect_tax_type_field  = 'mm_indirect_tax_type['.$loop.']';
           	      $indirect_tax_type_value = get_post_meta($variation->ID,'mm_indirect_tax_type', true);

            ?>
                 <label>Indirect Tax Type:</label><br>
                 <select name="<?php echo $indirect_tax_type_field; ?>"> 
                 <?php  echo MM_Connector_Functions::post_type_selection_options('mm-indirect-tax-type',$indirect_tax_type_value, true);  ?>
                 </select>
                 <br><?php echo $wwpp_min_order_warning; ?><br>

                 
            <?php 
               }


            ?>
            </div>
            <?php   
        	
        }

       /**
        * Saves the custom fields created in the `variable_product_woo_fields` function for each product variation
        * when the WooCommerce edit product page is saved.
        *
        * @param int $postid
        */
        public function variable_product_woo_fields_save($postid) {

               
              $variable_post_id = $_POST['variable_post_id'];
              $max_loop = max(array_keys($variable_post_id));

              $mm_field_meta_keys = $this->mm_field_meta_keys;
              
              foreach ($mm_field_meta_keys as $mm_field){

                 for ($i = 0; $i <= $max_loop; $i++) {

                     if (!isset($variable_post_id[$i])) {
                        continue;
                     }
                     $variation_id = (int) $variable_post_id[$i];
                     
           	         $woocommerce_field = $_POST[$mm_field][$i];
            	      if( !empty( $woocommerce_field ) ) {
        		             update_post_meta( $variation_id, $mm_field, esc_attr( $woocommerce_field ) );
                     }
                     else {
                        delete_post_meta( $variation_id, $mm_field );
                     }

                 }

              	
              }
              
              
         }



       /**
        * Updates order meta data with extra customer fields when a WooCommerce order is placed.
        * These fields include the Customer ID from the User record (which is added to the Order to help when parsing 
        * downloaded order), the user login id (for auto-creating a CustOnline record), the user's tax code and their 
        * indirect tax exemption status.
        *
        * 4.2.1 - Changed logic to use WooCommerce functions to since Woo will soon use their own tables to store order data.
        *
        * @param int $order_id
        */
        public function mm_checkout_field_update_order_meta( $order_id ) {
        	
              //Add Extra Customer Fields to Order       	

              //Get Account Number on User Record, and add to Order (helps when parsing downloaded order)
              $order = wc_get_order( $order_id );

              
              /* Old Logic - Deprecated 4.2.1
              *
              *
              // Get the Customer ID (User ID)
              $user_id = $order->get_user_id();
              $mm_field_CID = get_user_meta( $user_id, 'mm_field_CID' , true);
              update_post_meta( $order_id, 'mm_field_CID', $mm_field_CID);
              
              // Get the login id (for auto-creating CustOnline record in mm)
              $user = get_userdata($user_id);
              $login_id = $user->user_login;
              update_post_meta( $order_id, 'mm_login_id', $login_id);

              //Tax Code
              $mm_field_TXC = get_user_meta( $user_id, 'mm_field_TXC' , true);
              update_post_meta( $order_id, 'mm_field_TXC', $mm_field_TXC);
              
              //Indirect Tax Exempt
              $mm_field_ITX = get_user_meta( $user_id, 'mm_field_ITX' , true);
              update_post_meta( $order_id, 'mm_field_ITX', $mm_field_ITX);
              */
              
              // Get the Customer ID (User ID)
              $user_id = $order->get_user_id();
              $mm_field_CID = get_user_meta( $user_id, 'mm_field_CID' , true);
              $order->update_meta_data( 'mm_field_CID', $mm_field_CID);
              
              // Get the login id (for auto-creating CustOnline record in mm)
              $user = get_userdata($user_id);
              $login_id = $user->user_login;
              $order->update_meta_data( 'mm_login_id', $login_id);

              //Tax Code
              $mm_field_TXC = get_user_meta( $user_id, 'mm_field_TXC' , true);
              $order->update_meta_data( 'mm_field_TXC', $mm_field_TXC);
              
              //Indirect Tax Exempt
              $mm_field_ITX = get_user_meta( $user_id, 'mm_field_ITX' , true);
              $order->update_meta_data( 'mm_field_ITX', $mm_field_ITX);

              $order->update_meta_data( '_econnect_surcharge_name', $this->gateway_surcharge_name);
              $order->update_meta_data( '_econnect_surcharge_amount', $this->gateway_surcharge_amount);
          

              // Finally save all the meta data
              $order->save();              

        }



        /**
         * Add the indirect tax amount from the cart item to the order item meta data.
         *
         * @since 4.2.1
         * @param WC_Order_Item_Product $item The order item object that is being created.
         * @param string $cart_item_key The cart item key, which allows access to the original cart item's data.
         * @param array $values The cart item's data.
         */
        public function add_indirect_tax_to_order_item_meta($item, $cart_item_key, $values) {

            $quantity = $item->get_quantity();
            
            // Check if the field exists in the cart item data
            // If it does, add this value as meta data to the corresponding order item
            // Note: The cart has the unit amount.  we store the extended amount
            if (isset($values['indirect_tax_amount'])) {
                $item->add_meta_data('_indirect_tax_amount', $values['indirect_tax_amount'] * $quantity, true);
            }
            if (isset($values['indirect_tax_basis'])) {
                $item->add_meta_data('_indirect_tax_basis', $values['indirect_tax_basis'] * $quantity, true);
            }
            if (isset($values['indirect_tax_amount_j1'])) {
                $item->add_meta_data('_indirect_tax_amount_j1', $values['indirect_tax_amount_j1'] * $quantity, true);
            }
            if (isset($values['indirect_tax_basis_j1'])) {
                $item->add_meta_data('_indirect_tax_basis_j1', $values['indirect_tax_basis_j1'] * $quantity, true);
            }
            if (isset($values['indirect_tax_amount_j2'])) {
                $item->add_meta_data('_indirect_tax_amount_j2', $values['indirect_tax_amount_j2'] * $quantity, true);
            }
            if (isset($values['indirect_tax_basis_j2'])) {
                $item->add_meta_data('_indirect_tax_basis_j2', $values['indirect_tax_basis_j2'] * $quantity, true);
            }


        }



       /**
        * Displays the Account No. in the order details page in the WooCommerce admin dashboard.
        *
        * @param WC_Order $order
        */
        public function mm_checkout_field_display_admin_order_meta($order){

             //$mm_field_CID = get_post_meta( $order->id, 'mm_field_CID', true );
             
             $mm_field_CID = $order->get_meta('mm_field_CID',true);
             echo '<p><strong>'.__('Account No.').':</strong> ' . $mm_field_CID . '</p>';
        	
        	
        }


        /**
         * Modifies the columns in the WooCommerce Orders page, adding a new column for 'Account No.'.
         * The column is added after the second column in the existing array.
         *
         * @param array $columns Existing columns on the Orders page.
         * @return array $new_columns Modified columns with the 'Account No.' column added.
         */
        public function mm_field_CID_order_column( $columns ) {
        	
        	 $new_columns = array();
        	 $colno=0;

          foreach ( $columns as $key => $name ) {
            $colno++;
            $new_columns[ $key ] = $name;
            if ( $colno == 2 ) {
                 $new_columns['mm_field_CID'] = __( 'Account No.', 'managemore-connector' );
            }
          }
          return $new_columns;

        }
        
        /**
         * Populates the 'Account No.' column in the WooCommerce Orders page with data from the 
         * 'mm_field_CID' meta field for each order.
         *
         * @param string $column The slug of the current column being populated.
         */
        public function mm_add_order_mm_field_CID_column_content ($column){

        	     global $post;
        	     $order_id = $post->ID;
        	     //$mm_field_CID = get_post_meta( $order_id, 'mm_field_CID' , true);
      	     // Use a WC_Order instance to retrieve the order metadata
              $order = wc_get_order( $order_id );
              $mm_field_CID = $order->get_meta('mm_field_CID', true);

        	     if ( $column === 'mm_field_CID' ) {
        	     	  echo $mm_field_CID;
        	     }
        	     
        	    
        }
        
        /**
         * Makes the 'Account No.' column in the WooCommerce Orders page sortable by the 'mm_field_CID' 
         * meta field value.
         *
         * @param array $columns Existing columns on the Orders page.
         * @return array $columns Modified columns with 'mm_field_CID' marked as sortable.
         */
        public function mm_shop_order_column_mm_field_CID_sortable( $columns ) {    
              $meta_key = 'mm_field_CID';
              return wp_parse_args( array('mm_field_CID' => $meta_key), $columns );
        }
        
        /**
         * Handles sorting of the WooCommerce Orders page when sorted by the 'Account No.' column.
         * Orders are sorted by the value of their 'mm_field_CID' meta field.
         *
         * @param WP_Query $query The current query object.
         */
        public function mm_shop_order_column_mm_field_CID_sortable_orderby( $query ) {
 
              global $pagenow;

              if ( 'edit.php' === $pagenow && isset($_GET['post_type']) && 'shop_order' === $_GET['post_type'] ){

                 $orderby  = $query->get( 'orderby');
                 $meta_key = 'mm_field_CID';

                 if ('mm_field_CID' === $orderby){
                    $query->set('meta_key', $meta_key);
                    $query->set('orderby', 'meta_value');
                  }
              }
        }
        
        /**
         * Adds 'mm_field_CID' to the list of meta fields that are searchable from the WooCommerce Orders page.
         *
         * @param array $meta_keys Current list of meta keys that can be searched.
         * @return array $meta_keys Modified list of meta keys with 'mm_field_CID' added.
         */
        public function mm_field_CID_searchable_field( $meta_keys ){
            $meta_keys[] = 'mm_field_CID';
            return $meta_keys;
        }

        
        /**
         * Adds a section to the user profile page in the admin interface that displays the user's 
         * 'Account No.', which is fetched from the 'mm_field_CID' user meta field.
         * If the 'mm_field_CID' field is empty, displays 'Not Linked in ManageMore'.
         *
         * @param WP_User $user The user object currently being edited.
         */
        public function mm_print_user_admin_fields($user) {
 
            $mm_field_CID = get_user_meta($user->ID, 'mm_field_CID', true );
            $mm_field_CID = ($mm_field_CID == '') ? 'Not Linked in ManageMore' : $mm_field_CID;
            $mm_field_TXC = get_user_meta($user->ID, 'mm_field_TXC', true );

        	 ?>
	         <h2>ManageMore Information</h2>
	         <table class="form-table" id="mm-additional-information">
		         <tbody>
         			<tr>
			          	<th>
					      <label for="mm_field_CID">Account No.</label>
          				</th>
			          	<td>
			       		<?php echo $mm_field_CID;?>
          				</td>
			           </tr>
         			<tr>
			          	<th>
					      <label for="mm_field_TXC">Tax Code</label>
          				</th>
			          	<td>
			       		<?php echo $mm_field_TXC;?>
          				</td>
			           </tr>
           		</tbody>
         	</table>
	      <?php
        }
        
        /**
         * Modifies the columns in the WordPress Users page, adding a new column for 'Account No.'.
         * The column is added after the second column in the existing array.
         *
         * @param array $columns Existing columns on the Users page.
         * @return array $new_columns Modified columns with the 'Account No.' column added.
         */
        public function mm_field_CID_user_column( $columns ) {
        	$new_columns = array();
        	$colno=0;

        foreach ( $columns as $key => $name ) {
            $colno++;
            $new_columns[ $key ] = $name;
            if ( $colno == 2 ) {
                 $new_columns['mm_field_CID'] = __( 'Account No.', 'managemore-connector' );
            }
          }
          return $new_columns;

        }
        
        /**
         * Populates the 'Account No.' column in the WordPress Users page with data from the 
         * 'mm_field_CID' meta field for each user.
         *
         * @param string $val Empty string for column value.
         * @param string $column_name Name of the column.
         * @param int $user_id ID of the current user.
         * @return string Value of the column.
         */
        public function mm_add_user_mm_field_CID_column_content($val, $column_name, $user_id ){
        	    switch ($column_name) {
               case 'mm_field_CID' :
                  return get_user_meta($user_id, 'mm_field_CID', true );
               default:
             }    
             return $val;
        }

        /**
         * Modifies the WooCommerce Account menu links, adding a new link for 'Account Status'.
         * The link is added after the fifth link in the existing array.
         *
         * @param array $menu_links Existing menu links on the Account page.
         * @return array $menu_links Modified menu links with 'Account Status' link added.
         */
        public function mm_account_status_link( $menu_links ){
         
        	$menu_links = array_slice( $menu_links, 0, 5, true ) 
        	+ array( 'account-status' => 'Account Status' )
        	+ array_slice( $menu_links, 5, NULL, true );
         
        	return $menu_links;
         
        }

        /**
         * Adds a rewrite endpoint for 'account-status' to the WordPress rewrite rules.
         * This allows for URLs like 'myaccount/account-status' to be valid and handled by the template hierarchy.
         */
        public function mm_account_status_add_endpoint() {
             add_rewrite_endpoint( 'account-status', EP_PAGES );
        }

     	 /**
     	  * Return the HTML to display on the WooCommerce Account Status page
     	  * This is used if Ultimate Member is not installed. If it is, we add this info to their Account section
     	  *
     	  */
        public function mm_account_status_endpoint_content(){

        	  $user_id = get_current_user_id();
    	      $mm_last_sync = get_user_meta($user_id,'mm_last_sync',true); //Stores timestamp of last sync
    	      $mm_payment_pending = get_user_meta($user_id,'mm_payment_pending',true); //Stores timestamp pending payment
        	  $mm_field_CID = get_user_meta($user_id, 'mm_field_CID', true );
        	  $mm_field_B = get_user_meta($user_id, 'mm_field_B', true );
        	  $mm_field_CLMT = get_user_meta($user_id, 'mm_field_CLMT', true );
        	  $mm_field_LAR = get_user_meta($user_id, 'mm_field_LAR', true );
        	  $mm_field_LPD = get_user_meta($user_id, 'mm_field_LPD', true );
        	  $mm_field_L1N = get_user_meta($user_id, 'mm_field_L1N', true );
        	  $mm_field_L2N = get_user_meta($user_id, 'mm_field_L2N', true );
        	  $mm_field_L3N = get_user_meta($user_id, 'mm_field_L3N', true );
        	  $mm_field_L1X = get_user_meta($user_id, 'mm_field_L1X', true );
        	  $mm_field_L2X = get_user_meta($user_id, 'mm_field_L2X', true );
        	  $mm_field_L3X = get_user_meta($user_id, 'mm_field_L3X', true );
     	   	  $mm_allow_balance_payment = get_option('mm_allow_balance_payment',0);
     	   	  $mm_show_customer_balance = get_option('mm_show_customer_balance',0);
     	   	  $mm_show_credit_limit = get_option('mm_show_customer_balance',0);
     	   	  $mm_show_customer_last_payment = get_option('mm_show_customer_last_payment',0);
     	   	  $mm_show_customer_license_1 = get_option('mm_show_customer_license_1',0);
     	   	  $mm_show_customer_license_2 = get_option('mm_show_customer_license_2',0);
     	   	  $mm_show_customer_license_3 = get_option('mm_show_customer_license_3',0);
         	  $mm_customer_license_1_name = MM_Connector_Functions::mm_get_option('mm_customer_license_1_name');
    	      $mm_customer_license_2_name = MM_Connector_Functions::mm_get_option('mm_customer_license_2_name');
    	      $mm_customer_license_3_name = MM_Connector_Functions::mm_get_option('mm_customer_license_3_name');
    	      //Reformat dates according to website format.  We store dates as yyyy/mm/dd to correspond with Ultimate Member
    	      $store_date_format =  get_option( 'date_format' );
            $date = date_create_from_format('Y/m/d', $mm_field_L1X);
            if ($date) {
              $mm_license_1_expiration_date = $date->format($store_date_format);
            }
            $date = date_create_from_format('Y/m/d', $mm_field_L2X);
            if ($date) {
              $mm_license_2_expiration_date = $date->format($store_date_format);
            }
            $date = date_create_from_format('Y/m/d', $mm_field_L3X);
            if ($date) {
              $mm_license_3_expiration_date = $date->format($store_date_format);
            }
            $date = date_create_from_format('Y/m/d', $mm_field_LPD);
            if ($date) {
              $mm_last_payment_date = $date->format($store_date_format);
            }
    	      $mm_customer_license_1_expires_string = ($mm_field_L1X != '') ? 'Expires: ' .$mm_license_1_expiration_date : '';
    	      $mm_customer_license_2_expires_string = ($mm_field_L2X != '') ? 'Expires: ' .$mm_license_2_expiration_date : '';
    	      $mm_customer_license_3_expires_string = ($mm_field_L3X != '') ? 'Expires: ' .$mm_license_3_expiration_date : '';
            $mm_product_id = wc_get_product_id_by_sku($this->mm_pay_balance_sku);
            
            $on_account_purchases = get_user_meta($user_id, 'mm_field_OAP', true);
            if(isset($mm_field_CLMT) && $mm_field_CLMT > 0 && $on_account_purchases == '1'){
            	$credit_limit_display = wc_price($mm_field_CLMT);
            }
            else{
             	$credit_limit_display = 'N/A';
            }

            //Add Advanced Woo Customer Search (Id would be a setting)
            //if ( function_exists( 'aws_get_search_form') && ( current_user_can( 'switch_users' ) || current_user_can('edit_users')  ) ) { 
            //	aws_get_search_form( true, array( 'id' => 4 ) ); 
            // 
            //}



    	      if ($mm_last_sync == '' || $mm_field_CID == ''){
    	      	echo "<b>Account status information not available.</b>";
    	         return;  //Do not show rest of information on account not synchronized
    	      }
    	      else {
               //$mm_last_sync is stored as Shop-Based Timestamp
               $date_format = get_option( 'date_format' ).' h:i A';
    	         $as_of_date = date($date_format,$mm_last_sync); 
            }

            //Generate and display the Account Status Table        
            $table_html = "<table>";
            $table_html .= "<tr><td width='25%'><strong>Account Number</strong></td><td width='75%' colspan='3'>".$mm_field_CID."</td></tr>";
            //Balance
            if($mm_show_customer_balance) {
                $table_html .= "<tr><td width='25%'><strong>Account Balance</strong></td><td width='25%'>" . wc_price($mm_field_B) . "</td><td width='50%' colspan='2'>As of: ".$as_of_date."</td></tr>";
            }
            //Last Payment Amount and Date
            if($mm_show_customer_balance && $mm_field_LPD != '') {
                $table_html .= "<tr><td width='25%'><strong>Last Payment</strong></td><td width='25%'>" . wc_price($mm_field_LAR) . "</td><td width='50%' colspan='2'>Received on: ".$mm_last_payment_date."</td></tr>";
            }
            //Credit Limit
            if($mm_show_credit_limit) {
                $table_html .= "<tr><td width='25%'><strong>Credit Limit</strong></td><td width='25%'>" . $credit_limit_display . "</td><td width='50%' colspan='2'></td></tr>";
            }
            // Create an array of licenses
            $licenses = array(
                array("show" => $mm_show_customer_license_1, "name" => $mm_customer_license_1_name, "number" => $mm_field_L1N, "expires" => $mm_customer_license_1_expires_string),
                array("show" => $mm_show_customer_license_2, "name" => $mm_customer_license_2_name, "number" => $mm_field_L2N, "expires" => $mm_customer_license_2_expires_string),
                array("show" => $mm_show_customer_license_3, "name" => $mm_customer_license_3_name, "number" => $mm_field_L3N, "expires" => $mm_customer_license_3_expires_string),
            );
            // Loop over the licenses
            foreach ($licenses as $license) {
                if ($license["show"]) {
                    $table_html .= "<tr><td width='25%'><strong>".$license["name"]."</strong></td><td width='25%'>".$license["number"]."</td><td width='50%' colspan='2'>".$license["expires"]."</td></tr>";
                }
            }
            $table_html .= "</table>";
            echo $table_html;

            
            if ($mm_product_id && $mm_field_B && (float)$mm_field_B > 0 && $mm_allow_balance_payment == 1) {
            	if (empty($mm_payment_pending)){
                  $mm_button_html = sprintf(
                      '<a href="/cart?add-to-cart=%s" class="button add_to_cart_button %s">%s</a>',
                      $mm_product_id,
                      '',            // Insert custom class (we can add these values as settings later if needed)
                      'Pay Balance'  // Insert custom text
                  );
            		
            	}
            	else{
                  $pending_since = date('m/d/Y h:i A', $mm_payment_pending); // format the timestamp
                  $mm_button_html =  "<i>Payment pending since ". $pending_since."</i>";
            	}
               echo $mm_button_html;
            }
            
        }


       	/**
       	 * Add the column UPC inside the product list table.
       	 *
       	 * @param $columns
       	 *
       	 * @return array
       	 */
        public function manage_product_columns( $columns ) {
       		$product_items = array( 'mm_product_upc' => 'UPC' );
       		$ref_pos       = array_search ( 'sku', array_keys ( $columns ) );
       		$columns = array_slice ( $columns, 0, $ref_pos + 1, true ) + $product_items + array_slice ( $columns, $ref_pos + 1, count ( $columns ) - 1, true );

       		return $columns;
       	}

       	/**
       	 * Show the UPC code inside the product list.
       	 *
       	 * @param $column
       	 * @return void
       	 *
       	 */
        public function show_upc_code_in_product_list( $column ) {

       		if ( 'mm_product_upc' == $column ) {
       			global $post, $the_product;

       			if ( empty( $the_product ) || $the_product->get_id() !== $post->ID ) {
       				$the_product = wc_get_product( $post->ID );
       			}

       			if ($the_product->is_type('variable')){
         			  echo '-';
       			}
                else {
       			  $mm_product_upc = $the_product->get_meta( 'mm_product_upc' );
         			  echo esc_html( $mm_product_upc );
                }

       		}
  	     }



        /**
         * Add UPC column styling.
         * 
         * @since 1.0.1
         * @access public
         */
        public function add_upc_column_styling() {
            ?>

            <style> 
                th#mm_product_upc { 
                    width: 100px; 
                }

            </style>

            <?php
        }



       	/**
       	 * Add UPC Search to Wholesale Order Form Plug In.
       	 *
       	 * @param $listing_args  arguments passed to query for posts, must include proper parent product ids in post__in 
       	 *
       	 */
        public function mm_filter_product_listing_query_arg($listing_args){
        	
        	if($listing_args['searched_keyword'] == '' ){
        		return $listing_args;
        	}


         //Get UPC
	      $args = array(
            'post_type' => array('product','product_variation'),
            'post_status' => 'publish',
            'meta_key' => 'mm_product_upc',
            'meta_value' => $listing_args['searched_keyword']
             );

          $posts = get_posts( $args );
          $post_in = array();
          foreach($posts as $post){
          	 if ($post->post_type == 'product_variation'){
            	 $variation = wc_get_product($post->ID);
                $post_in[] = $post->ID; //Include the child in case listed separately
                $post_in[] = $variation->get_parent_id(); //also include the parent
          	 }
          	 else {
            	 $post_in[] = $post->ID;
          	 }
          }
            
          if (!empty($posts)){
            $listing_args['post__in'] = $post_in;
          }
 			 return $listing_args;
			
			
        }

       /**
         * Perform Action on Product Deletion. (Remove Customer Pricing)
         * 
         * @since 2.1.0
         * @access public
         */
        public function mm_product_deleted($post_id) {
 
            if( get_post_type( $post_id ) != 'product' ) return;
            $customer_pricing  = ''; //Sending blank string will cause all pricing to be removed
    	      MM_Connector_Functions::update_customer_pricing($customer_pricing,$post_id,2);         	
        	
        }
      

         /**
         * Shortcode function to create a Pay Balance button.
         * 
         * @since 4.0.0
         * @access public
         */
         public function mm_pay_balance_button($atts = array()) {

              $mm_product_id = wc_get_product_id_by_sku($this->mm_pay_balance_sku);

              // Parse shortcode attributes
              $atts = shortcode_atts(
                  array(
                      'text' => __('Pay Balance', 'woocommerce'),  // Default button text
                      'class' => '',  // Default class
                  ), 
                  $atts,
                  'mm_pay_balance_button'
              );
              

              if (is_user_logged_in() ) {
                  // Get the ID of the current user
                  $mm_user_id = get_current_user_id();

                  // Get the customer balance from the user meta data
                  $mm_customer_balance = get_user_meta($mm_user_id, $this->mm_cust_bal_meta_field, true);
                  $mm_payment_pending = get_user_meta($mm_user_id,'mm_payment_pending',true);

                  if ($mm_product_id && $mm_customer_balance && (float)$mm_customer_balance > 0 && empty($mm_payment_pending)) {
                      $mm_button_html = sprintf(
                          '<a href="/cart?add-to-cart=%s" class="button add_to_cart_button %s">%s</a>',
                          $mm_product_id,
                          esc_attr($atts['class']),  // Insert custom class
                          esc_html($atts['text'])  // Insert custom text
                      );

                      return $mm_button_html;
                  }
                  if (!empty($mm_payment_pending)) {
                    $pending_since = date('m/d/Y h:i A', $mm_payment_pending); // format the timestamp
                    return "<i>Payment pending since {$pending_since}</i>";
                  }
                  
              }
          }


         /* 
         * Hook to adjust the Customer Payment product price when added to cart
         * 
         * @since 4.0.0
         * @access public
         */
         public function mm_adjust_paybal_item_price($cart_object) {
             // Get the ID of the current user
             $mm_user_id = get_current_user_id();

             // Get the customer balance from the user meta data
             $mm_customer_balance = get_user_meta($mm_user_id, $this->mm_cust_bal_meta_field, true);

             // Check each cart item and adjust the price if it matches your SKU
             foreach ($cart_object->cart_contents as $key => $value) {
                 if ($value['data']->get_sku() == $this->mm_pay_balance_sku) {
                     $value['data']->set_price((float)$mm_customer_balance);
                 }
             }
         }

         /**
         * Shortcode function to display user meta field (mm_field_$$$)
         *
         * pass field = "$$$"
         * 
         * 
         * @since 4.0.0
         * @access public
         */
         public function mm_display_field_shortcode($atts) {
             // Parse shortcode attributes
             $atts = shortcode_atts(
                 array(
                     'field' => '',  // Default field
                 ), 
                 $atts,
                 'mm_display_field'
             );

             if (is_user_logged_in() && $atts['field'] != '') {
                 // Get the ID of the current user
                 $mm_user_id = get_current_user_id();

                 // Prepare the full meta key
                 $mm_meta_key = 'mm_field_' . $atts['field'];

                 // Get the meta data from the user meta data
                 $mm_meta_value = get_user_meta($mm_user_id, $mm_meta_key, true);

                    // Check the field and format the value accordingly
                    switch ($atts['field']) {
                        case 'B':
                        case 'LAR':
                            $mm_meta_value = number_format((float)$mm_meta_value, 2);
                            break;
                        case 'CLMT':
                            $on_account_purchases = get_user_meta($mm_user_id, 'mm_field_OAP', true);
                            if (isset($mm_meta_value) && $mm_meta_value > 0  && $on_account_purchases == '1'){
                               $mm_meta_value = number_format((float)$mm_meta_value, 2);
                            }
                            else {
                            	 $mm_meta_value ='N/A';
                            }
                            break;
                        // Add other cases here
                    }
                    return esc_html($mm_meta_value);
             }

             // In case of failure return an empty string
             return '';
         }

         /*
          * Checks for the existence of the balance payment product in WooCommerce
          * 
          * @since 4.0.0
          * @access public
          */
         public function check_for_pay_balance_product() {
             if ($this->get_pay_balance_product() === null) {
                 $this->create_pay_balance_product();
             }
         }

         /*
          * Retrieves the balance payment product from WooCommerce
          * 
          * @since 4.0.0
          * @access private
          */
         private function get_pay_balance_product() {
             $product_id = wc_get_product_id_by_sku($this->mm_pay_balance_sku);
             if ($product_id === 0) {
                 return null;
             } else {
                 return wc_get_product($product_id);
             }
         }

          /*
          * Creates a balance payment product in WooCommerce
          * 
          * @since 4.0.0
          * @access private
          */ 
          private function create_pay_balance_product() {
             $product = new WC_Product();
             $product->set_name('Customer Balance Payment');
             $product->set_status('publish');
             $product->set_catalog_visibility('hidden');
             $product->set_sku($this->mm_pay_balance_sku);
             $product->set_regular_price(0.01);              // Note: we found that adding the item with no price prevented it from being added to cart.
             $product->set_sold_individually(true); // Set the product as sold individually
             $product->set_virtual(true);     
             $product_id = $product->save();

             if ($product_id === 0) {
                 error_log('Failed to create Pay Balance product.');
             }
         }

         /*
          * Adds  custom sections to the Ultimate Member account page
          * 
          * @since 4.0.0
          * @access public
          */
         public function add_custom_sections_to_um( $tabs ) {

            $mm_show_customer_license_1 = get_option('mm_show_customer_license_1',0);
            $mm_show_customer_license_2 = get_option('mm_show_customer_license_2',0);
            $mm_show_customer_license_3 = get_option('mm_show_customer_license_3',0);
            $mm_show_customer_tax_info = get_option('mm_show_customer_tax_info',0);
            $mm_allow_pdf_upload = $this->mm_allow_pdf_upload; 
            
            if ($mm_show_customer_license_1 || $mm_show_customer_license_2 || $mm_show_customer_license_3 ){
       	      $tabs[800]['license']['icon'] = 'um-icon-ribbon-b';
	            $tabs[800]['license']['title'] = 'License';
	            $tabs[800]['license']['submit_title'] = 'Save License Information';
	            $tabs[800]['license']['custom'] = true;
	            $tabs[800]['license']['show_button'] = ($mm_allow_pdf_upload) ? false : true;
            }
            
            if ($mm_show_customer_tax_info){
         	   $tabs[805]['tax_info']['icon'] = 'um-faicon-institution';
	            $tabs[805]['tax_info']['title'] = 'Tax Information';
	            $tabs[805]['tax_info']['custom'] = true;
	            $tabs[805]['tax_info']['show_button'] = false;
            }
             
         	$tabs[810]['account_status']['icon'] = 'um-faicon-briefcase';
	         $tabs[810]['account_status']['title'] = 'Account Status';
	         $tabs[810]['account_status']['custom'] = true;
	         $tabs[810]['account_status']['show_button'] = false;

	         return $tabs;

         }
         


         /*
          * Hooks into the content of the Ultimate Member license tab
          * 
          * @since 4.0.0
          * @access public
          */
         public function mm_content_hook_license( $output = '') {

           //Note: We do not currently allow changing these options, since:
           //a) We think it is best that they are set this way
           //b) We want to assume that the license data always comes from MM 
           //   and is to be updated on the website from MM data, not vice versa.
           //   The exception is the image, where it is most practical that user uploads a new image for license renewal
           $mm_edit_license = false;
           $mm_edit_license_image = true;
           $mm_allow_pdf_upload = $this->mm_allow_pdf_upload;
           
           ob_start(); 

           // Loop for licenses 1 to 3
           for ($i = 1; $i <= 3; $i++) {
               if (get_option('mm_show_customer_license_' . $i, 0)) {
                   // Edit license details
                   if ($mm_edit_license) {
                       $data = UM()->builtin()->get_a_field('mm_field_L' . $i . 'N');    //Get Number
                       echo UM()->fields()->edit_field('mm_field_L' . $i . 'N', $data);  //Edit Number
                       $data = UM()->builtin()->get_a_field('mm_field_L' . $i . 'X');    //Get Expiration
                       echo UM()->fields()->edit_field('mm_field_L' . $i . 'X', $data);  //Edit Expiration
                   }
                   // View license details
                   else {
                       echo $this->mm_view_um_field('mm_field_L' . $i . 'N');            //View number
                       echo $this->mm_view_um_field('mm_field_L' . $i . 'X');            //View Expiration
                   }
                   // Always show preview of saved image, if present, and not using file (allow PDF)
                   if ($mm_allow_pdf_upload == 0){
                      $image_name = UM()->fields()->field_value('mm_field_L' . $i . 'I');
                      if (!empty($image_name)) {
                         echo $this->mm_view_um_image('mm_field_L' . $i . 'I');
                      }
                   }
                   // Edit license image/Display Link to Profile
                   if ($mm_edit_license_image) {
                   	  if ($mm_allow_pdf_upload){
                   	  	  if ($i == 1 ) {
                             $license_name = MM_Connector_Functions::mm_get_option("mm_customer_license_{$i}_name");
                             //Get user profile URL
                             $profile_url = um_user_profile_url();
                             //Add the query argument to the URL
                             $edit_profile_url = add_query_arg( 'um_action', 'edit', $profile_url );
                             echo '<div class="um-field"><div class="um-field-label"><a href="' . esc_url( $edit_profile_url ) . '"> Upload a New ' . esc_html( $license_name ) . '</a></div></div>';
                   	  	  }
                  	  	  
                   	  }
                   	  else{
                          $data = UM()->builtin()->get_a_field('mm_field_L' . $i . 'I');
                          $data ['label'] .= ' (New)';
                          echo UM()->fields()->edit_field('mm_field_L' . $i . 'I', $data);
                   	  }
                   }
               }
           }

           echo '<br><br>';
    
           $output = ob_get_clean();
           return $output;
         }


         /*
          * Hooks into the content of the Ultimate Member Tax Info tab
          * 
          * @since 4.0.0
          * @access public
          */
         public function mm_content_hook_tax_info( $output = '') {
         	
             // List of fields to display.
             $fields_display = array(
                  'mm_field_TXID',
                  'mm_field_FXID',
             );
         	
             // Check if no Account (CID is empty)
             $CID_value = UM()->fields()->field_value( 'mm_field_CID' );
             if(empty($CID_value)) {
                 $noAccountInfoMsg = "Tax information not available.";
                 return '<br><br><div class="um-field-value">' . $noAccountInfoMsg . '</div>';
             }

             ob_start(); 

             //Display Fields from array set above
             foreach( $fields_display as $key ) {
                 echo $this->mm_view_um_field( $key );
             }

             $output = ob_get_clean();
             return $output;

         }


         /*
          * Hooks into the content of the Ultimate Member account status tab
          * 
          * @since 4.0.0
          * @access public
          */
         public function mm_content_hook_account_status( $output = '') {

             // List of fields to display.
             // Always display Account Number
             $fields_display = array(
                  'mm_field_CID',
             );

             // Display Balance, Last Payment Info if plug-in setting enabled
             if(get_option('mm_show_customer_balance',0)){
             	 $fields_display = array_merge($fields_display,array(
                  'mm_field_B',
                  'mm_field_LAR',
                  'mm_field_LPD',
             	 ));
             }
             
             //Display Credit Limit if plug-in setting enabled
             if(get_option('mm_show_credit_limit',0)){
             	 $fields_display = array_merge($fields_display,array(
                  'mm_field_CLMT',
             	 ));
             }
             

             // Check if no Account (CID is empty)
             $CID_value = UM()->fields()->field_value( 'mm_field_CID' );
             if(empty($CID_value)) {
                 $noAccountInfoMsg = "Account status information not available.";
                 return '<br><br><div class="um-field-value">' . $noAccountInfoMsg . '</div>';
             }

             ob_start(); 

             //Display Fields from array set above
             foreach( $fields_display as $key ) {
                 echo $this->mm_view_um_field( $key );
             }

             // Display Pay Balance Button if plug-in setting enabled
             if (get_option('mm_allow_balance_payment',0)){
                echo '<br><br>';
                echo $this->mm_pay_balance_button();  //Will only display the button if there is a balance, and no pending payment already
             }

             $output = ob_get_clean();
             return $output;
         }

         /**
          * Save custom fields from the License Tab.
          *
          * @since 4.0.0
          * @param int $user_id User ID.
          */
         public function um_account_update_license( $user_id, $changes ) {

            $mm_allow_pdf_upload = $this->mm_allow_pdf_upload;
         	if ($mm_allow_pdf_upload) {
         		return;
         	}

         	// List of fields to update.
         	//Note: We currently do not allow updating the license number and expiration on the website
         	//      But, the admin can add to the profile and update there. (that is not handled here)
         	//      This only applies to the account page.  For now, these fields are display-only there.
         	$fields = array(
         		//'mm_field_L1N',
         		//'mm_field_L1X',
         		'mm_field_L1I',
         		//'mm_field_L2N',
         		//'mm_field_L2X',
         		'mm_field_L2I',
         		//'mm_field_L3N',
         		//'mm_field_L3X',
         		'mm_field_L3I',
         	);

         	foreach( $fields as $key ) {

         		//$_POST[ $key ] is the new value
         		
         		if( isset( $_POST[ $key ] ) && ! UM()->form()->has_error( $key ) ) {

         			$field = UM()->builtin()->get_a_field($key);
 
            		   update_user_meta( $user_id, $key, wp_unslash( $_POST[ $key ] ) ); //Update Value


                     if ( isset( $field['type'] ) && in_array( $field['type'], array( 'image', 'file' ) ) ) {

                         if ( um_is_temp_file($_POST[ $key ] ) || $_POST[ $key ] == 'empty_file' ) {
                             $files[ $key ] = $_POST[ $key ];
                         //} elseif( um_is_file_owner( UM()->uploader()->get_upload_base_url() . '/'. $user_id . '/' . $_POST[ $key ], $user_id ) ) {
                         //    $files[ $key ] = $_POST[ $key ];
                         //} else {
                         //    $files[ $key ] = 'empty_file';
                         }
                         
                         
                     }

         	   }
            }
            
            
            if ( ! empty( $files ) && is_array( $files ) ) {

               UM()->uploader()->replace_upload_dir = true;
               UM()->uploader()->move_temporary_files( $user_id, $files, true );
               UM()->uploader()->replace_upload_dir = false;

               $this->send_license_change_email($user_id, $files); //Possibly send email regarding changed files
 
            }
            
            
         }


          
         /**
          * Possibly send an email regarding license change to Admin/designate
          *
          * @since 4.3.0
          * @param int $user_id  The current user
          * @param array $files  The array of files updated
          *
          * @return null
          */
         private function send_license_change_email($user_id, $files){

               //Send email about image updates
               $mm_license_change_email = MM_Connector_Functions::mm_get_option('mm_license_change_email');
               if (!empty($mm_license_change_email)){
                  // Fetch user information
                  $user_data = get_userdata($user_id);
                  $email = $user_data->user_email;

                  $first_name = get_user_meta($user_id, 'billing_first_name', true);
                  $last_name = get_user_meta($user_id, 'billing_last_name', true);
                  $company_name = get_user_meta($user_id, 'billing_company', true); 
                  $mm_field_CID = get_user_meta($user_id, 'mm_field_CID', true);
                  $upload_url = UM()->uploader()->get_upload_base_url() . $user_id . '/';

                  ob_start();
                  ?>
                  <table>
                      <tbody>
                          <tr>
                              <td><strong>Name:</strong></td>
                              <td colspan="2"><?php echo $first_name . ' ' . $last_name; ?></td>
                          </tr>
                          <tr>
                              <td><strong>Company:</strong></td>
                              <td colspan="2"><?php echo $company_name; ?></td>
                          </tr>
                          <tr>
                              <td><strong>Email:</strong></td>
                              <td colspan="2"><?php echo $email; ?></td>
                          </tr>
                          <tr>
                              <td><strong>Account Number:</strong></td>
                              <td colspan="2"><?php echo $mm_field_CID; ?></td>
                          </tr>
                          <tr>
                              <td colspan="3"><strong>Updated License Image(s):</strong></td>
                          </tr>
                          <?php foreach($files as $key => $file): ?>
                              <?php 
                                 //Extract the digit from the key to use in constructing the option key
                                 preg_match("/\d/", $key, $matches);
                                 $license_number = $matches[0];
                              ?>
                              <tr>
                                  <td><strong><?php echo MM_Connector_Functions::mm_get_option("mm_customer_license_{$license_number}_name"); ?></strong></td>
                                  <td colspan="2"><img src="<?php echo $upload_url . $file; ?>" style="width: 100%; height: auto;"></td>
                              </tr>
                          <?php endforeach; ?>
                      </tbody>
                  </table>
                  <?php
                  $contents = ob_get_contents();
                  ob_end_clean();

                   // Headers to send HTML email
                  $headers = array('Content-Type: text/html; charset=UTF-8');
                  wp_mail($mm_license_change_email, 'Profile updated', $contents, $headers);
               	
               }
         	
         }


         /**
          * View an Ultimate member field.
          *
          * @since 4.0.0
          * @param string $key field meta key.
          */
         private function mm_view_um_field($key) {
             $data = UM()->builtin()->get_a_field( $key );
             $value = UM()->fields()->field_value( $key );

             // Note: We should just be able to echo UM()->fields()->view_field( $key, $data );
             //       but we cannot since it ignores empty fields and does not format properly 
             //       because it does not call our hook for formatting the value

             // Format values for specific fields
             switch ( $key ) {
                 case 'mm_field_B':
                 case 'mm_field_LAR':
                     $value =  number_format( (float)$value, 2, '.', ',' );
                     break;
                 case 'mm_field_CLMT':
                     //for some reason, $value does not have the value of this field.
                     //not sure why, but we have to reget the user meta ($limit) to display properly here
                     $user_id = get_current_user_id();
                     $on_account_purchases = get_user_meta($user_id, 'mm_field_OAP', true);
                     $limit = get_user_meta($user_id, 'mm_field_CLMT', true);
                     if(isset($limit) && $limit > 0 && $on_account_purchases == '1'  ){
                        $value =  number_format( (float)$limit, 2, '.', ',' );
                     }
                     else {
                     	$value = 'N/A';
                     }
                     break;
                 case 'mm_field_LPD':
                 case 'mm_field_L1X':
                 case 'mm_field_L2X':
                 case 'mm_field_L3X':
                     $store_date_format =  get_option( 'date_format' );
                     $date = date_create_from_format('Y/m/d', $value);
                     if ($date) {
                         $value = $date->format($store_date_format);
                     } 
             }

             // Generate the custom HTML
             $output = '<div class="um-field">';
             $output .= '<div class="um-field-label">' . $data['label'] . '</div>';
             $output .= '<div class="um-field-area">';
             $output .= '<div class="um-field-value">' . $value . '</div>';
             $output .= '</div>';
             $output .= '</div>';

             return $output;
         }

         /**
          * View an Ultimate member image field.
          *
          * @since 4.0.0
          * @param string $key image field meta key.
          */
         private function mm_view_um_image($key) {
             $data = UM()->builtin()->get_a_field( $key );
             $value = UM()->fields()->field_value( $key );

             // Get the user's uploads folder
             $image_folder = UM()->uploader()->get_upload_base_url() . um_user( 'ID' ) . DIRECTORY_SEPARATOR;

             // Get the image URL
             $image_url = $image_folder . $value;

             // Generate the custom HTML
             $output = '<div class="um-field">';
             $output .= '<div class="um-field-label">' . $data['label'] .' (Saved)</div>';
             $output .= '<div class="um-field-area">';
             $output .= '<div class="um-field-value">';
             $output .= '<div class="um-photo">';
             $output .= '<a href="#" class="um-photo-modal" data-src="' . $image_url . '">';
             $output .= '<img src="' . $image_url . '" alt="' . $data['label'] . '" title="' . $data['label'] . '">';
             $output .= '</a>';
             $output .= '</div>';
             $output .= '</div>';
             $output .= '</div>';
             $output .= '</div>';

             return $output;
         }



         /*
         * Filters the available payment gateways based on various conditions
         * 
         * @since 4.0.0
         * @access public
         */
         public function mm_filter_gateways($gateways) {
             global $woocommerce;

             // Check if the balance payment product is in the cart
             $balance_payment_in_cart = false;
             if (isset($woocommerce->cart) && is_array($woocommerce->cart->cart_contents)) {
                foreach ($woocommerce->cart->cart_contents as $key => $values) {
                    if ($values['data']->get_sku() == $this->mm_pay_balance_sku) {
                        $balance_payment_in_cart = true;
                        break;
                    }
                }
             }

             // Get current user and check for 'mm_field_OAP' meta value
             $current_user = wp_get_current_user();
             $user_allowed_on_account = get_user_meta($current_user->ID, 'mm_field_OAP', true);
             //Check if On Account gateway is restricted
             if (isset($gateways['managemore_onaccount'])) {
                 $restrict_onaccount = $gateways['managemore_onaccount']->get_restrict_use() === 'yes' ? true : false;
             }
             else {
             	  $restrict_onaccount = false;
             }

             // Get the preferred gateway for balance payments
             $preferred_gateway = get_option('mm_balance_payment_gateway');

             // If the balance payment product is not in the cart and the exclusive gateway option is enabled
             if (!$balance_payment_in_cart && get_option('mm_use_payment_gateway_exclusive') == '1') {
                 // Remove the preferred gateway
                 unset($gateways[$preferred_gateway]);
             }

             // If customer is not allowed on account purchases, remove 'managemore_onaccount' gateway
             if ($restrict_onaccount && $user_allowed_on_account != '1') {
                 unset($gateways['managemore_onaccount']);
             }

             
             // If the balance payment product is in the cart
             if ($balance_payment_in_cart) {
                 // Get a list of all available gateways
                 $available_gateways = array_keys($gateways);

                 // Loop through all available gateways
                 foreach($available_gateways as $gateway_id) {
                     // If the gateway is not the preferred one, or is 'managemore_onaccount', remove it
                     if($gateway_id != $preferred_gateway || $gateway_id == 'managemore_onaccount') {
                         unset($gateways[$gateway_id]);
                     }
                 }
             }

             return $gateways;
         }
  
         

         /*
          * Perform actions after saving the order:
          * Displays a custom thank you message after checkout, and sets the Pending Payment flag (timestamp)
          * 
          * @since 4.0.0
          * @access public
          */
         public function mm_after_saving_payment_order($order_id) {
             // Get the order based on order_id
             $order = wc_get_order($order_id);
             
             // Check if the balance payment product is in the order
             $balance_payment_in_order = false;
             foreach ($order->get_items() as $item_id => $item) {
                 if ($item->get_product()->get_sku() == $this->mm_pay_balance_sku) {
                     $balance_payment_in_order = true;
                     break;
                 }
             }

             // If the balance payment product is in the order
             if ($balance_payment_in_order) {
                 // Get the customer user id
                 $user_id = $order->get_user_id();

                 // Save the current timestamp as user meta
                 update_user_meta($user_id, 'mm_payment_pending', current_time('timestamp', 0));

                 // Get the custom message from the setting
                 $custom_message = MM_Connector_Functions::mm_get_option('mm_payment_checkout_message');
                 echo '<p>' . esc_html($custom_message) . '</p>';
             }
         }
         
         /*
          * Changes the order status to processing if the order contains the balance payment product
          * 
          * @since 4.0.0
          * @access public
          */
         public function mm_mark_balance_payment_order_processing($order_id){
             // Create an instance of WC_Order object
             $order = wc_get_order($order_id);

             // Loop through all order line items
             foreach ($order->get_items() as $item_id => $item) {
                 // Get product id
                 $product_id = $item->get_product_id();
                 // Get product
                 $product = wc_get_product($product_id);
                 // If product SKU matches the special SKU
                 if ($product->get_sku() == $this->mm_pay_balance_sku) {
                     // Change order status to 'processing'
                     $order->update_status('processing');
                     // We found our product, no need to loop further
                     break;
                 }
             }
         }

         /*
          * Adds a custom message to the customer order emails for customer balance payments
          * 
          * @since 4.0.0
          * @access public
          */
          public function mm_add_payment_message_to_emails( $order, $sent_to_admin, $plain_text, $email ) {
             // We only want to add the message to customer emails, not admin emails
             if ( $sent_to_admin ) {
                 return;
             }
             
             // We only want to add the message if the balance payment product is in the order
             $items = $order->get_items();
             foreach ( $items as $item ) {
                 $product = $item->get_product();
                 // Check if get_product() returned a valid product object
                 if ( $product && is_a($product, 'WC_Product') ) {
                     if ( $product->get_sku() == $this->mm_pay_balance_sku ) {
                         $custom_message = MM_Connector_Functions::mm_get_option('mm_payment_checkout_message');
                         echo wpautop( wptexturize( $custom_message ) );
                         break;
                     }
                 }
             }
         }

         /*
          * Adds predefined fields to Ultimate Member.  These fields should be added to the Registration form to allow MM to capture this info
          * May also be added to other forms to allow update of this information online.  Some fields (e.g. Balance are display only)
          * 
          * @since 4.0.0
          * @access public
          */
         public function mm_um_add_fields( $fields ) {

            $mm_allow_pdf_upload = $this->mm_allow_pdf_upload;
          	$upload_type = ($mm_allow_pdf_upload) ? 'file' : 'image';

         	$fields['mm_field_CID'] = array(
         		'title'             => __( 'MM Account Number', 'managemore-connector' ),
         		'metakey'           => 'mm_field_CID',
         		'type'              => 'text',
         		'label'             => __( 'Account Number', 'managemore-connector' ),
               'public'            => -1,
               'editable'          => 0,
         		'edit_forbidden'    => 1,    
         		'show_anyway'       => true,
         		'custom'            => true,
         	);

         	$fields['mm_field_B'] = array(
         		'title'             => __( 'MM Balance', 'managemore-connector' ),
         		'metakey'           => 'mm_field_B',
         		'type'              => 'text',
         		'label'             => __( 'Balance', 'managemore-connector' ),
               'public'            => -1,
               'editable'          => 0,
         		'edit_forbidden'    => 1,    
         		'show_anyway'       => true,
         		'custom'            => true,
         	);

         	$fields['mm_field_CLMT'] = array(
         		'title'             => __( 'MM Credit Limit', 'managemore-connector' ),
         		'metakey'           => 'mm_field_CLMT',
         		'type'              => 'text',
         		'label'             => __( 'Credit Limit', 'managemore-connector' ),
               'public'            => -1,
               'editable'          => 0,
         		'edit_forbidden'    => 1,    
         		'show_anyway'       => true,
         		'custom'            => true,
         	);


         	$fields['mm_pay_balance_button'] = array(
         		'title'             => __( 'MM Pay Balance Button', 'managemore-connector' ),
         		'type'              => 'shortcode',
               'public'            => 2,
               'content'           => '[mm_pay_balance_button]',
         		'edit_forbidden'    => 1,    
         		'show_anyway'       => true,
         		'custom'            => true,
         	);

         	$fields['mm_field_LAR'] = array(
         		'title'             => __( 'MM Last Amount Received', 'managemore-connector' ),
         		'metakey'           => 'mm_field_LAR',
         		'type'              => 'text',
         		'label'             => __( 'Last Amount Received', 'managemore-connector' ),
               'public'            => -1,
               'editable'          => 0,
         		'edit_forbidden'    => 1,    
         		'show_anyway'       => true,
         		'custom'            => true,
         	);

         	$fields['mm_field_LPD'] = array(
         		'title'             => __( 'MM Last Payment Date', 'managemore-connector' ),
         		'metakey'           => 'mm_field_LPD',
         		'type'              => 'text',
         		'label'             => __( 'Last Payment Date', 'managemore-connector' ),
               'public'            => -1,
               'editable'          => 0,
         		'edit_forbidden'    => 1,    
         		'show_anyway'       => true,
         		'custom'            => true,
         	);

          	$fields['mm_field_TXID'] = array(
               'title'    => __( 'MM State Tax Id', 'managemore-connector' ),
               'metakey'  => 'mm_field_TXID',
               'type'     => 'text',
               'label'    => 'State Tax Id',
               'required' => 1,
               'public'   => -1,
               'editable' => 0,
         		'edit_forbidden'    => 1,    
         		'show_anyway'       => true,
               'max_chars' => 15,
	         );

          	$fields['mm_field_FXID'] = array(
               'title'    => __( 'MM Federal Tax Id', 'managemore-connector' ),
               'metakey'  => 'mm_field_FXID',
               'type'     => 'text',
               'label'    => 'Federal Tax Id',
               'required' => 1,
               'public'   => -1,
               'editable' => 0,
         		'edit_forbidden'    => 1,    
         		'show_anyway'       => true,
               'max_chars' => 15,
	         );

          	$fields['mm_field_L1N'] = array(
               'title'    => __( 'MM License 1', 'managemore-connector' ),
               'metakey'  => 'mm_field_L1N',
               'type'     => 'text',
               'label'    => MM_Connector_Functions::mm_get_option('mm_customer_license_1_name'),
               'required' => 1,
               'public'   => -1,
               'editable' => 0,
         		'edit_forbidden'    => 1,    //See note in mm_content_hook_license
               'max_chars' => 30,
	         );

          	$fields['mm_field_L1X'] = array(
               'title'    => __( 'MM License 1 Expiration', 'managemore-connector' ),
					'metakey' => 'mm_field_L1X',
					'type' => 'date',
               'label'    => MM_Connector_Functions::mm_get_option('mm_customer_license_1_name').' Expiration',
					'required' => 1,
					'public' => -1,
					'editable' => 0,
         		'edit_forbidden'    => 1,    //See note in mm_content_hook_license
					'format' => 'M j Y',
					'pretty_format' => 0,
					'format_custom' => get_option( 'date_format' ),
					'years' => 20,
					'years_x' => 'future',
	         );


          	$fields['mm_field_L1I'] = array(
               'title'    => __( 'MM License 1 Image', 'managemore-connector' ),
					'metakey' => 'mm_field_L1I',
					'type' => $upload_type,
					'label' => MM_Connector_Functions::mm_get_option('mm_customer_license_1_name').' Image',
					'upload_text' => 'Upload a copy of your '.MM_Connector_Functions::mm_get_option('mm_customer_license_1_name'),
					'allowed_types' => array('pdf','png','jpg','jpeg','gif'),  //this is valid for type 'file' but then it will not show a preview
					'max_size' => 9999999,
               'public'   => -1,
               'editable' => 1,
	         );


          	$fields['mm_field_L2N'] = array(
              'title'    => __( 'MM License 2', 'managemore-connector' ),
              'metakey'  => 'mm_field_L2N',
              'type'     => 'text',
              'label'    => MM_Connector_Functions::mm_get_option('mm_customer_license_2_name'),
              'required' => 1,
              'public'   => -1,
              'editable' => 0,
        		  'edit_forbidden'    => 1,    //See note in mm_content_hook_license
              'max_chars' => 30,
	         );


          	$fields['mm_field_L2X'] = array(
               'title'    => __( 'MM License 2 Expiration', 'managemore-connector' ),
					'metakey' => 'mm_field_L2X',
					'type' => 'date',
               'label'    => MM_Connector_Functions::mm_get_option('mm_customer_license_2_name').' Expiration',
					'required' => 1,
					'public' => -1,
					'editable' => 0,
         		'edit_forbidden'    => 1,    //See note in mm_content_hook_license
					'format' => 'M j Y',
					'pretty_format' => 0,
					'format_custom' => get_option( 'date_format' ),
					'years' => 20,
					'years_x' => 'future',
	         );
            if (!$this->mm_allow_pdf_upload){
             	$fields['mm_field_L2I'] = array(
                  'title'    => __( 'MM License 2 Image', 'managemore-connector' ),
   					'metakey' => 'mm_field_L2I',
   					'type' => $upload_type,
   					'label' => MM_Connector_Functions::mm_get_option('mm_customer_license_2_name').' Image',
   					'upload_text' => 'Upload a copy of your '.MM_Connector_Functions::mm_get_option('mm_customer_license_2_name'),
   					'allowed_types' => array('pdf','png','jpg','jpeg','gif'),  //this is valid for type 'file' but then it will not show a preview
   					'max_size' => 9999999,
                  'public'   => -1,
                  'editable' => 1,
   	         );
            }

          	$fields['mm_field_L3N'] = array(
              'title'    => __( 'MM License 3', 'managemore-connector' ),
              'metakey'  => 'mm_field_L3N',
              'type'     => 'text',
              'label'    => MM_Connector_Functions::mm_get_option('mm_customer_license_3_name'),
              'required' => 1,
              'public'   => -1,
              'editable' => 0,
        		  'edit_forbidden'    => 1,    //See note in mm_content_hook_license
              'max_chars' => 30,
	         );



          	$fields['mm_field_L3X'] = array(
               'title'    => __( 'MM License 3 Expiration', 'managemore-connector' ),
					'metakey' => 'mm_field_L3X',
					'type' => 'date',
               'label'    => MM_Connector_Functions::mm_get_option('mm_customer_license_3_name').' Expiration',
					'required' => 1,
					'public' => -1,
					'editable' => 0,
         		'edit_forbidden'    => 1,    //See note in mm_content_hook_license
					'format' => 'M j Y',
					'pretty_format' => 0,
					'format_custom' => get_option( 'date_format' ),
					'years' => 20,
					'years_x' => 'future',
	         );

            if (!$this->mm_allow_pdf_upload){
             	$fields['mm_field_L3I'] = array(
                  'title'    => __( 'MM License 3 Image', 'managemore-connector' ),
   					'metakey' => 'mm_field_L3I',
   					'type' => $upload_type,
   					'label' => MM_Connector_Functions::mm_get_option('mm_customer_license_3_name').' Image',
   					'upload_text' => 'Upload a copy of your '.MM_Connector_Functions::mm_get_option('mm_customer_license_3_name'),
   					'allowed_types' => array('pdf','png','jpg','jpeg','gif'),  //this is valid for type 'file' but then it will not show a preview
   					'max_size' => 9999999,
                  'public'   => -1,
                  'editable' => 1,
   	         );
            }

         	return $fields;

         }

        /**
         * Show Account No. as UM Field
         *
         * @param $value
         * @param $data
         *
         * @return string
         */
        public function um_profile_field_filter_hook__mm_field_CID( $value, $data ) {

         $user_id = get_current_user_id();
         // Get the customer balance from the user meta data
         $mm_field_CID = get_user_meta($user_id, 'mm_field_CID', true);
         $output = '';
         if (!empty($mm_field_CID)){
         	$output = $mm_field_CID;
         }
         else {
         	$output = ''; //Return nothing so field does not show
         }
        	return $output;

        }

        /**
         * Show Balance as UM Field
         *
         * @param $value
         * @param $data
         *
         * @return string
         */
        public function um_profile_field_filter_hook__mm_field_B( $value, $data ) {

         $user_id = get_current_user_id();
         $mm_field_CID =  get_user_meta($user_id,'mm_field_CID',true);

         // Get the customer balance from the user meta data
         $mm_customer_balance = get_user_meta($user_id, 'mm_field_B', true);
       	$output = ''  ;
         if (!empty($mm_field_CID)){
          	$output = number_format((float)$mm_customer_balance, 2, '.', ',' )  ;
         }
         else {
         	//Do not return formatted zero for non customers.
          	$output = ''  ;
         }
        	return $output;

        }


        /**
         * Show Credit Limit as UM Field
         *
         * @param $value
         * @param $data
         *
         * @return string
         */
        public function um_profile_field_filter_hook__mm_field_CLMT( $value, $data ) {

         $user_id = get_current_user_id();
         $mm_field_CID =  get_user_meta($user_id,'mm_field_CID',true);

         // Get the fields from the user meta data
         $mm_credit_limit = get_user_meta($user_id, 'mm_field_CLMT', true);
         $on_account_purchases = get_user_meta($user_id, 'mm_field_OAP', true);
       	$output = ''  ;
         if (!empty($mm_field_CID)){
         	if(isset($mm_credit_limit) && $mm_credit_limit > 0 && $on_account_purchases == '1' ){
             	$output = number_format((float)$mm_credit_limit, 2, '.', ',' )  ;
         	}
         	else {
         	 	$output = 'N/A';
         	}
         }
         else {
         	//Do not return formatted zero for non customers.
          	$output = ''  ;
         }
        	return $output;

        }
        
        /**
         * Show Last Amount Received as UM Field
         *
         * @param $value
         * @param $data
         *
         * @return string
         */
        public function um_profile_field_filter_hook__mm_field_LAR( $value, $data ) {

          $user_id = get_current_user_id();
          $mm_cust_id =  get_user_meta($user_id,'mm_field_CID',true);

          // Get the customer balance from the user meta data
          $mm_last_amount_received = get_user_meta($mm_user_id, 'mm_field_LAR', true);
          if (empty($mm_cust_id)){
         	//Do not return formatted zero for non customers.
          	$output = ''  ;
          }
          else {
          	$output = number_format((float)$mm_last_amount_received, 2, '.', ',' )  ;
          }
        	 return $output;

        }

        /**
         * Show Last Payment Date as UM Field
         *
         * @param $value
         * @param $data
         *
         * @return string
         */
        public function um_profile_field_filter_hook__mm_field_LPD( $value, $data ) {

          $user_id = get_current_user_id();
          $mm_cust_id =  get_user_meta($user_id,'mm_field_CID',true);

          // Get the customer balance from the user meta data
          $mm_last_amount_received = get_user_meta($mm_user_id, 'mm_field_LAR', true);
          if (empty($mm_cust_id)){
         	//Do not return formatted zero for non customers.
          	$output = ''  ;
          }
          else {
            $store_date_format =  get_option( 'date_format' );
            $date = date_create_from_format('Y/m/d', $mm_last_amount_received);
            if ($date) {
               $output = $date->format($store_date_format);
            } 
          }
        	 return $output;

        }


        /**
         * Restrict Checkout with Error if necessary
         *
         * @return void
         */
        public function mm_maybe_restrict_checkout() {


            $user_id = is_user_logged_in() ? get_current_user_id() : 0;
            
            //This logic was moved from authorization (login) to here 4.0.0
            //We now prevent checkout if disabled. We prevent login if inactive.
            //Check if user is disabled.
            if(is_user_logged_in()){
           	   $mm_disable_user =  get_user_meta($user_id,'mm_disable_user',true);
    	         if ($mm_disable_user == '1'){
                  wc_add_notice(MM_Connector_Functions::mm_get_option('mm_disabled_account_message'), 'error');
                  return;  //No other notices are relevant
               }
            }
            
            $restrict_checkout_license_expired = get_option('mm_restrict_checkout_license_expired', 0);
            if ($restrict_checkout_license_expired){
               $license_expiration_dates = $this->mm_get_user_license_dates($user_id);
            }

            $problems_found = array();
            // 4.6.1 - Intialize array to avoid notice on strict sites
            $problems_found['region'] = false;
            $problems_found['login'] = false;
            $problems_found['license1'] = false;
            $problems_found['license2'] = false;
            $problems_found['license3'] = false;
            $pay_balance=false; // 4.6.1 - Intialize var to avoid notice on strict sites

            $cart_items = WC()->cart->get_cart();
            foreach ($cart_items as $cart_item_key => $cart_item) {
                $product = $cart_item['data'];
                //Check if Balance Payment is in the cart
                if ( $product->get_sku() == $this->mm_pay_balance_sku ) {
                   $pay_balance = true;
                }

                //Check if License Expired
                if ($restrict_checkout_license_expired){
                   $product_id = $product->get_id();
                   $result = $this->mm_check_product_license($product_id, $user_id, $license_expiration_dates);
                   if ($result !== true){
                   	 if ($user_id == 0){
                   	 	 $problems_found['login'] = true;  //this should not really happen here, since product could not have been added to the cart.
                   	 	 $this->custom_cart_data[$cart_item_key]['mm_problem_login'] = true;
                   	 }
                   	 elseif($result < 0){
                   	 	 $restricted_license_type = -$result ;
                   	 	 $problems_found['license'.$restricted_license_type] = true;  //required license expired
                   	 	 $this->custom_cart_data[$cart_item_key]['mm_problem_license'.$restricted_license_type] = true;
                   	 }
                   } 
                }

                //Check Region Restrictions
                // If product is a variation, switch to the parent product ID. (fixed 4.2.3)
                if ( $product->is_type( 'variation' ) ) {
                    $region_product = wc_get_product($product->get_parent_id());
                } else {
                    $region_product = $product;
                }

                //changed to use product object 4.2.4
                $state_restrictions = $region_product->get_meta('mm_sale_restrict_region', true);
                //$state_restrictions = get_post_meta($region_product_id, 'mm_sale_restrict_region', true);

                $restrictions_array = str_split($state_restrictions);

                // Check shipping country first
                $ship_country = WC()->customer->get_shipping_country();
                if ($ship_country !== 'US') {
                    if ($ship_country === 'CA' && $restrictions_array[51] === '1') {
                       $problems_found['region'] = true;  //region restriction
                       $this->custom_cart_data[$cart_item_key]['mm_problem_region'] = true;
                    }
                    if ($ship_country === 'MX' && $restrictions_array[52] === '1') {
                       $problems_found['region'] = true;  //region restriction
                       $this->custom_cart_data[$cart_item_key]['mm_problem_region'] = true;
                    }
                    // Assuming we do not restrict shipping to other countries
                }
                else {
                   // Check if shipping state is restricted for this product
                   $ship_state = WC()->customer->get_shipping_state();
                   $state_array = $this->mm_get_state_array();
                   $state_index = array_search($ship_state, array_keys($state_array));
                   if (isset($restrictions_array[$state_index]) && $restrictions_array[$state_index] === '1') {
                      $problems_found['region'] = true;  //region restriction 
                      $this->custom_cart_data[$cart_item_key]['mm_problem_region'] = true;
                   }
                }

            }

            //Add Notices to Cart
            // If special SKU is in cart and more than one item is in the cart, show error
            if ( $pay_balance && count( $cart_items ) > 1 ) {
               wc_add_notice( 'You cannot checkout with other items in your cart when making a customer payment.', 'error' );
               return; //no other notices are relevant
            }
            if ($problems_found['login'] == true ){
               wc_add_notice( 'You must login to make this purchase, due to items restricted in your cart.', 'error' );
               return; //no other notices are relevant
            }
            if ($problems_found['region'] == true ){
               $this->add_region_notice();
            }
            for ($i = 1; $i <= 3; $i++) {
               if ($problems_found['license'.$i] == true ) {
                  $this->add_license_notice($i);
               }
            }
        }


        /**
         * Hooks into WooCommerce's add-to-cart process and restricts adding a product to the cart based on user license conditions.
         *
         * @param bool $passed     The original validation status of adding the product to the cart.
         * @param int  $product_id The ID of the product being added to the cart.
         * @param int  $quantity   The quantity of the product being added to the cart.
         * @return bool Returns the potentially modified validation status. If false, the product won't be added to the cart.
         */
        public function mm_maybe_restrict_add_to_cart($passed, $product_id, $quantity) {

            if (get_option('mm_restrict_checkout_license_expired', 0)) {
               $user_id = is_user_logged_in() ? get_current_user_id() : 0;
               $license_expiration_dates = $this->mm_get_user_license_dates($user_id);
               $result = $this->mm_check_product_license($product_id, $user_id, $license_expiration_dates);
               if ( $result != true ) {
               	//add a notice to the cart
               	if($user_id == 0){
 		               wc_add_notice(__('You must be logged in to purchase this product.', 'managemore-connector'), 'error');
               	}
               	elseif ($result < 0 ) {
               		//if negative, result represents the license type that is bad
               		//Add Notice to Cart
               		$required_license_type = - $result;
               		$this->add_license_notice($required_license_type);
               	}
                  return false; 
               }
            }
            
            // Add more conditions here, if necessary ...
            
            
            return $passed;
        }

        /**
         * Add notice to cart for License Restriction
         *
         * @param int $required_license_type The License 1-3 that is being restricted
         *
         * @return void
         */
        private function add_license_notice($required_license_type){
           $license_name_option = 'mm_customer_license_' . $required_license_type . '_name';
           $license_name = MM_Connector_Functions::mm_get_option($license_name_option);
           $expired_message = MM_Connector_Functions::mm_get_option('mm_license_expired_message');
           $expired_notation = MM_Connector_Functions::mm_get_option('mm_license_expired_notation_'.$required_license_type);
           $expired_message = str_replace('{license_name}', $license_name, $expired_message);
           $expired_message = str_replace('{notation}', $expired_notation, $expired_message);
           $linebreak = (wc_notice_count('error') > 0 ) ? '<br>' : '' ;
           wc_add_notice($linebreak.$expired_message, 'error');
        }

        /**
         * Adds notice to cart for Region Restriction
         *
         * @return void
         */
        private function add_region_notice(){
           $region_message = MM_Connector_Functions::mm_get_option('mm_region_restricted_message');
           $linebreak = (wc_notice_count('error') > 0 ) ? '<br>' : '' ;
           wc_add_notice($linebreak.$region_message, 'error');
        }

    
        /** 
         * Define a State Array to match ManageMore
         *
         * @ return array Returns the state array
         *
         */
        private function mm_get_state_array() {
            return array(
                'AL' => 'Alabama',        
                'AK' => 'Alaska',         
                'AZ' => 'Arizona',        
                'AR' => 'Arkansas',       
                'CA' => 'California',     
                'CO' => 'Colorado',       
                'CT' => 'Connecticut',    
                'DE' => 'Delaware',       
                'FL' => 'Florida',        
                'GA' => 'Georgia',        
                'HI' => 'Hawaii',         
                'ID' => 'Idaho',          
                'IL' => 'Illinois',       
                'IN' => 'Indiana',        
                'IA' => 'Iowa',           
                'KS' => 'Kansas',         
                'KY' => 'Kentucky',       
                'LA' => 'Louisiana',      
                'ME' => 'Maine',          
                'MD' => 'Maryland',       
                'MA' => 'Massachusettes', 
                'MI' => 'Michigan',       
                'MN' => 'Minnesota',      
                'MS' => 'Mississippi',    
                'MO' => 'Missouri',       
                'MT' => 'Montana',        
                'NE' => 'Nebraska',       
                'NV' => 'Nevada',         
                'NH' => 'New Hampshire',  
                'NJ' => 'New Jersey',     
                'NM' => 'New Mexico',     
                'NY' => 'New York',       
                'NC' => 'North Carolina', 
                'ND' => 'North Dakota',   
                'OH' => 'Ohio',           
                'OK' => 'Oklahoma',       
                'OR' => 'Oregon',         
                'PA' => 'Pennsylvania',   
                'RI' => 'Rhode Island',   
                'SC' => 'South Carolina', 
                'SD' => 'South Dakota',   
                'TN' => 'Tennessee',      
                'TX' => 'Texas',          
                'UT' => 'Utah',           
                'VT' => 'Vermont',        
                'VA' => 'Virginia',       
                'WA' => 'Washington',     
                'WV' => 'West Virginia',  
                'WI' => 'Wisconsin',      
                'WY' => 'Wyoming',        
                'PR' => 'Puerto Rico',    
                'CAN' => 'Canada',         
                'MEX' => 'Mexico'          
            );
        }


        /**
         * Checks user license expiration dates for a particular product.
         * Adds the license expired notice to the cart if they cannot.
         *
         * @param int $user_id The ID of the user.
         * @return int Returns true if the product can be purchased by the user, and false, or negative 1, 2, 3 otherwise.
         *             Negative value indicates the license that is restricted
         *
         */
        private function mm_check_product_license($product_id, $user_id, $license_expiration_dates) {

            //changed to use product object 4.2.4
            $product = wc_get_product($product_id);
            $required_license_type = $product->get_meta('mm_required_license_type', true);
            //$required_license_type = get_post_meta($product_id, 'mm_required_license_type', true);
            if ($required_license_type == '' || $required_license_type == '0') return true;
            
            if ($user_id === 0) {
                return false;
            }

            $license_expiration_meta = 'mm_field_L' . $required_license_type . 'X';
            $license_expiration_date = $license_expiration_dates[$license_expiration_meta] ?? null;

            // If license is not set or expired
            if (!$license_expiration_date || strtotime($license_expiration_date) < strtotime(date('Y-m-d'))) {
            	 $bad_license = - $required_license_type;  //return a negative 1, 2, or 3, representing the bad license.
                return $bad_license;
            }
          
            
            return true;
        }

        /**
         * Retrieve user's license expiration dates.
         *
         * This function retrieves the license expiration dates for a specific user from user meta.
         * If the user is not logged in, the function will return null values for each license type.
         *
         * @param int $user_id User's ID.
         * @return array An associative array containing keys as license types and values as the respective license expiration dates.
         *               If the user is not logged in, the values will be null.
         */
        private function mm_get_user_license_dates($user_id) {
            $license_expiration_dates = array();
            if ($user_id > 0) {
                for ($i = 1; $i <= 3; $i++) {
                    $license_expiration_meta = 'mm_field_L' . $i . 'X';
                    $license_expiration_dates[$license_expiration_meta] = get_user_meta($user_id, $license_expiration_meta, true);
                }
            }
            return $license_expiration_dates;
        }


        /**
         * Display the Indirect Tax notation/amount in the cart, at the end of the price
         *
         * @param string $price The formatted price string
         * @param array $cart_item The item array
         * @param int $cart_item_key The key to the cart item we are on.
         *
         * @return string The possibly updated price string
         */
        public function display_indirect_tax_in_cart($price, $cart_item, $cart_item_key) {
            if (!empty($cart_item['indirect_tax_amount'])) {
                // Get the indirect tax detail message from the options
                $indirect_tax_message = MM_Connector_Functions::mm_get_option('mm_indirect_tax_detail_message');

                // If the indirect tax message is not empty 
                if (!empty($indirect_tax_message)) {
                    // If it contains the {indirect_tax_amount} tag Replace the tag with the formatted indirect tax amount
                    $indirect_tax_message = str_replace('{indirect_tax_amount}', wc_price($cart_item['indirect_tax_amount']), $indirect_tax_message);

                    // Append the indirect tax message to the price
                    $price .= $indirect_tax_message;
                }
            }
            return $price;
        }

        /**
         * Adds notice to cart that Indirect Tax Is Present
         *
         * @return void
         */
        public function add_indirect_tax_notice() {
            $has_indirect_tax = false;
            $total_indirect_tax = 0.00;
            $cart = WC()->cart->get_cart();
            foreach ($cart as $cart_item) {
                if (!empty($cart_item['indirect_tax_amount'])) {
                    $has_indirect_tax = true;
                    $total_indirect_tax += $cart_item['indirect_tax_amount'] * $cart_item['quantity'];
                }
            }
            if ($has_indirect_tax) {
                $mm_indirect_tax_warning_message = MM_Connector_Functions::mm_get_option('mm_indirect_tax_warning_message');
                // Replace the tag with the formatted indirect tax total
                $mm_indirect_tax_warning_message = str_replace('{total_indirect_tax}', wc_price($total_indirect_tax), $mm_indirect_tax_warning_message);
                wc_add_notice($mm_indirect_tax_warning_message, 'notice');
            }
        }


        /**
         * Display the notations for restricted products at the beginning of the Item Name
         *
         * @param string $product_name is the existing product name
         * @param array $cart_item is the array containing product info
         * @param int $cart_item_key is the unique cart item ID
         *
         * @return string The possibly updated price string
         */
        public function maybe_modify_cart_item_name($product_name, $cart_item, $cart_item_key) {
            
            $spacer = '&nbsp;&nbsp;';

            // Check if there's custom data for this cart item
            if (isset($this->custom_cart_data[$cart_item_key])) {
                $custom_data = $this->custom_cart_data[$cart_item_key];

                if (isset($custom_data['mm_problem_region']) && $custom_data['mm_problem_region']) {
                	  $notation = MM_Connector_Functions::mm_get_option('mm_region_restricted_notation');
                	  if (!empty($notation)){
                       $product_name =  $notation . $spacer . $product_name ;
                	  }
                }

                for ($i = 1; $i <= 3; $i++) {
                    if (isset($custom_data['mm_problem_license'.$i]) && $custom_data['mm_problem_license'.$i]) {
                   	   $notation = MM_Connector_Functions::mm_get_option('mm_license_expired_notation_'.$i);
                    	   if (!empty($notation)){
                   	      $product_name =  $notation . $spacer . $product_name ;
                   	   }
                    }
                }

            }


            return $product_name;
        }

  /**
   * Apply service charge to cart.
  */
  public function apply_gateway_surcharge() {

      $chosen_payment_method = WC()->session->get('chosen_payment_method');
      $surcharge_enabled = MM_Connector_Functions::get_payment_gateway_setting($chosen_payment_method,'surcharge_enabled');
      $surcharge_percent = MM_Connector_Functions::get_payment_gateway_setting($chosen_payment_method,'surcharge_percent');
      $surcharge_name = MM_Connector_Functions::get_payment_gateway_setting($chosen_payment_method,'surcharge_name');
      
     
      if ($surcharge_enabled === 'yes' && isset(WC()->session)) {
          $percentage = floatval($surcharge_percent);
          $cart_total = WC()->cart->cart_contents_total;
          $surcharge_amount = ($percentage / 100) * $cart_total;
          $surcharge_amount = round($surcharge_amount, 2);
          WC()->cart->add_fee(__($surcharge_name, 'woocommerce'), $surcharge_amount, false);
          $this->gateway_surcharge_name = $surcharge_name;
          $this->gateway_surcharge_amount = $surcharge_amount;
      }
      else {
          $this->gateway_surcharge_name = '';
          $this->gateway_surcharge_amount = 0.00;
      }
  }

  public function enqueue_payment_scripts() {

    wc_enqueue_js("
        $('body').on('change', '.checkout .input-radio', function() {
            $('body').trigger('update_checkout');
        });
    ");

    
  }


        /**
         * Execute model.
         *
         * @since 1.0.0
         * @access public
         */
        public function run() {

              //Set Class Properties based on Options
              $this->show_account_status_section = get_option('mm_show_account_status_section',1);
              $this->indirect_tax_enabled = get_option('mm_indirect_tax_enable',0);
              $this->mm_indirect_tax_allow_cart_warning = get_option('mm_indirect_tax_allow_cart_warning',0);
              $this->mm_indirect_tax_allow_cart_detail = get_option('mm_indirect_tax_allow_cart_detail',0);
              $this->mm_allow_pdf_upload = get_option('mm_allow_pdf_upload',1);
              
              //Build Array of Custom Field Meta Keys
              $this->mm_field_meta_keys = array('mm_product_upc',
                                                'mm_product_cost',
                                                'mm_product_last_cost',
                                                'mm_product_lowest_price',
                                                'mm_product_basis_1',
                                                'mm_product_basis_2',
                                                'mm_product_basis_3',
                                                'mm_product_net_weight',
                                                'mm_required_license_type',
                                                'mm_sale_restrict_region');

              if ($this->indirect_tax_enabled){
              	  $this->mm_field_meta_keys[] = 'mm_indirect_tax_type';
              }

              //Add Action to Update Order with Custom Data (e.g. Account Number)
              add_action( 'woocommerce_checkout_update_order_meta', array($this,'mm_checkout_field_update_order_meta'),99 );
              // Add Action to Update Order Item Meta with indirect tax amount
              if ($this->indirect_tax_enabled){
                  add_action('woocommerce_checkout_create_order_line_item', array($this, 'add_indirect_tax_to_order_item_meta'), 10, 3);
              }
              //Display Field on Order Edit Page
              add_action( 'woocommerce_admin_order_data_after_billing_address', array($this,'mm_checkout_field_display_admin_order_meta'), 10, 1);
              //Add column to Woo Orders Table
              add_filter( 'manage_edit-shop_order_columns',  array($this,'mm_field_CID_order_column' ) , 10, 1);
              add_action( 'manage_shop_order_posts_custom_column', array($this,'mm_add_order_mm_field_CID_column_content' ));
              // Make custom column sortable
              add_filter( "manage_edit-shop_order_sortable_columns", array($this,'mm_shop_order_column_mm_field_CID_sortable' ));
              // Make sorting work properly
              add_action('pre_get_posts', array($this,'mm_shop_order_column_mm_field_CID_sortable_orderby' ));
              // Make account no. searchable in the shop orders list
              add_filter( 'woocommerce_shop_order_search_fields', array($this,'mm_field_CID_searchable_field' ));


              //Add Custom Fields to Ultimate Member
              if (MM_Connector_Functions::is_plugin_active('ultimate-member/ultimate-member.php')) {
              	  	  //If using Ultimate Member, add fields from MM to be available on UM Forms
                   add_filter( 'um_predefined_fields_hook', array($this, 'mm_um_add_fields'), 10, 1 );
                   //Add filter to display balance, other fields, formatted properly
                   add_filter( 'um_profile_field_filter_hook__mm_field_B', array($this, 'um_profile_field_filter_hook__mm_field_B'), 99, 2 );
                   add_filter( 'um_profile_field_filter_hook__mm_field_CID', array($this, 'um_profile_field_filter_hook__mm_field_CID'), 99, 2 );
                   add_filter( 'um_profile_field_filter_hook__mm_field_LAR', array($this, 'um_profile_field_filter_hook__mm_field_LAR'), 99, 2 );
                   add_filter( 'um_profile_field_filter_hook__mm_field_CLMT', array($this, 'um_profile_field_filter_hook__mm_field_CLMT'), 99, 2 );
              }

              //Show extra tabs in the Account Section of Ultimate Member/Woo
              //Make New Tab
              if ($this->show_account_status_section == 1){
              	  if (MM_Connector_Functions::is_plugin_active('ultimate-member/ultimate-member.php')) {
              	  	  //If using Ultimate Member, add tabs to their account page
              	  	  add_filter('um_account_page_default_tabs_hook', array($this,'add_custom_sections_to_um'), 100 );
              	  	  add_filter('um_account_content_hook_license', array($this,'mm_content_hook_license'));
              	  	  add_filter('um_account_content_hook_tax_info', array($this,'mm_content_hook_tax_info'));
              	  	  add_filter('um_account_content_hook_account_status', array($this,'mm_content_hook_account_status'));
              	  	  //Add Action to save updates to licenses
              	  	  add_action( 'um_after_user_account_updated', array($this,'um_account_update_license'), 8 , 2 );
                 }
              	  else {
                    add_filter ( 'woocommerce_account_menu_items', array($this,'mm_account_status_link'), 40 );
                    add_action( 'init', array($this,'mm_account_status_add_endpoint' ));
                    add_action( 'woocommerce_account_account-status_endpoint', array($this,'mm_account_status_endpoint_content') );
                	}
               
              }
           
              
              //Show Account Number in the Admin User area
              add_action( 'show_user_profile', array($this,'mm_print_user_admin_fields'), 30 ); // admin: edit profile
              add_action( 'edit_user_profile', array($this,'mm_print_user_admin_fields'), 30 ); // admin: edit other users
              
              //Show Account Number in the Admin Users table
              add_filter( 'manage_users_columns', array($this,'mm_field_CID_user_column' ));
              add_filter( 'manage_users_custom_column', array($this,'mm_add_user_mm_field_CID_column_content'), 10, 3 );

              //Add Custom Tabs fpr ManageMore Data and Region Restrictions
              add_filter( 'woocommerce_product_data_tabs', array($this, 'mm_add_product_tabs' ));
              add_action( 'admin_head', array($this, 'mm_add_state_restrictions_icon_css') );
              
              //Add Data to Custom Tabs for Simple products and Region Restrictions
              add_action( 'woocommerce_product_data_panels', array($this, 'mm_custom_tab_product_data_fields' ));
 
         	  //Add Custom Fields to Simple Products
              //add_action( 'woocommerce_product_options_general_product_data', array($this,'simple_product_woo_fields'),10 );
              add_action( 'woocommerce_process_product_meta', array($this,'simple_product_woo_fields_save' ), 10, 1 );

         	  //Add Custom Fields to Variable Products
              add_action('woocommerce_product_after_variable_attributes', array($this, 'variable_product_woo_fields'), 10, 3);
              add_action('woocommerce_save_product_variation', array($this, 'variable_product_woo_fields_save'), 10, 1);
              add_action('woocommerce_ajax_save_product_variations', array($this, 'variable_product_woo_fields_save'), 10, 1); // Via Ajax ( Introduced on WooCommerce 2.4 series )


              

              //Add Search By UPC
              //add_action( 'pre_get_posts', array( $this, 'extend_admin_search' ),10 );
              //add_action( 'pre_get_posts', array( $this, 'extend_product_search' ),11 );
 
     		      //add the column UPC on product list
		          add_filter( 'manage_product_posts_columns', array( $this, 'manage_product_columns' ), 20 );
		          add_action( 'manage_product_posts_custom_column', array( $this, 'show_upc_code_in_product_list' ) );
              add_action( 'admin_head', array( $this , 'add_upc_column_styling' ) ); 
 		        
		          add_filter('wwof_filter_product_listing_query_arg',array($this,'mm_filter_product_listing_query_arg'));

		          //Add Action for when product is deleted 
		          add_action( 'delete_post', array($this, 'mm_product_deleted'), 99 );


              //Add shortcode for Pay Balance button, action to adjust special sku amount when added to cart
              add_shortcode('mm_pay_balance_button', array($this, 'mm_pay_balance_button'));
              add_action('woocommerce_before_calculate_totals', array($this, 'mm_adjust_paybal_item_price'), 10, 1);
              
              //Shortcode to display user meta field (such as balance)
              add_shortcode('mm_display_field', array($this, 'mm_display_field_shortcode'));
              
              //Restrict Payment Gateway(s) if Customer Payment sku is in cart or not
              add_filter('woocommerce_available_payment_gateways', array($this,'mm_filter_gateways'));
              
              //Add Custom message for Customer payment on thank you page and customer email; also sets flag that a payment is processing
              add_action( 'woocommerce_thankyou', array($this, 'mm_after_saving_payment_order') );
              add_action( 'woocommerce_email_after_order_table', array($this, 'mm_add_payment_message_to_emails'), 10, 4 );
              
              //Add action to mark customer balance payment orders as "Processing" - otherwise, since it is a virtual product, it would be marked "Completed"
              //We want to mark it completed when the order is pulled into ManageMore and the payment is applied to the balance
              add_action('woocommerce_new_order', array($this, 'mm_mark_balance_payment_order_processing'));

              //Maybe Restrict Checkout (e.g. payment sku with other items, or temp disabled)
              add_action('woocommerce_check_cart_items', array($this, 'mm_maybe_restrict_checkout'));

              //Maybe Restrict Add to Cart (e.g., license expired)
              add_filter('woocommerce_add_to_cart_validation', array($this, 'mm_maybe_restrict_add_to_cart'), 10, 3);
              
              //Maybe Modify Product Name to denote restriction
              add_filter('woocommerce_cart_item_name', array($this, 'maybe_modify_cart_item_name'), 10, 3);

              
              //Add Notices for Indirect Taxes in Cart
              if ($this->indirect_tax_enabled){
              	  if($this->mm_indirect_tax_allow_cart_detail){
                	  add_filter('woocommerce_cart_item_price',  array($this,'display_indirect_tax_in_cart'), 10, 3);
              	  }
              	  if($this->mm_indirect_tax_allow_cart_warning){
              	     add_action('woocommerce_before_cart', array($this,'add_indirect_tax_notice'));
              	  }
              }

              //process script to recalc when payment is selected
              add_action('woocommerce_after_checkout_form', array($this, 'enqueue_payment_scripts'));

              //Add action for adding gateway based service charge to cart, if necessary
              add_action('woocommerce_cart_calculate_fees', array($this, 'apply_gateway_surcharge'));

    
        }
    	
    	
    	
    	
    }   
}