<?php if (!defined('ABSPATH')) {
    exit;
}
// Exit if accessed directly

if (!class_exists('MM_WOO_WHOLESALE')) {

    /**
     * Model that houses the logic of MM WooCommerce Wholesale Related Functions (Custom Woo Fields, Indirect Taxes, Customer Pricing, etc.)
     * Note: This functionality requires the Wholesale Prices Premium Plug In.  The code uses filters/logic provided by that plug-in
     *
     * @since 1.0.0
     */
    class MM_WOO_WHOLESALE {

        /*
        |--------------------------------------------------------------------------
        | Class Properties
        |--------------------------------------------------------------------------
         */
        public $indirect_tax_enabled = 0 ;
        private $indirect_tax_amount = 0.00;
        private $indirect_tax_basis = 0.00;
        private $indirect_tax_amount_j1 = 0.00;
        private $indirect_tax_basis_j1 = 0.00;
        private $indirect_tax_amount_j2 = 0.00;
        private $indirect_tax_basis_j2 = 0.00;
        private $wholsale_price_before_indirect_tax = 0.00;
       
    

        /*
        |--------------------------------------------------------------------------
        | Class Methods
        |--------------------------------------------------------------------------
         */



        public function override_wholesale_price_shop($price_arr, $product_id, $user_wholesale_role) {

      	  global $woocommerce;

      	  //Get Current User, check customer specific price
          $user_id = get_current_user_id();
          $price_arr = $this->override_wholesale_price_with_user_price($price_arr,$user_id,$product_id);

          //1.0.6 - Only show tax included in shop if woo setting dictates
          if( get_option( 'woocommerce_tax_display_shop' ) != 'incl' ) {
          	return $price_arr;
          }

          $indirect_tax_exempt = get_user_meta($user_id,'mm_field_ITX',true);
          if ($indirect_tax_exempt == '1'){
          	//Customer is exempt from indirect tax. do not add to price
          	//We don't need basis here, just return (do not do calculation)
          	return $price_arr;
          }

        
          //Get Indirect Tax for Cart State:
          $ship_state = '';
          if (isset($woocommerce->cart)) {
             $ship_state =   $woocommerce->cart->get_customer()->get_shipping_state();
          }
          if ($ship_state == '' and $user_id<>0){
          	 //If cart does not have a state, use customer state
          	 $ship_state = get_user_meta($user_id, 'shipping_state', true );
          	 //If customer does not have a shipping state, use customer billing state
          	 if ($ship_state == ''){
            	 $ship_state = get_user_meta($user_id, 'billing_state', true );
          	 }
          }
          $price_arr = $this->override_wholesale_price_with_indirect_tax($price_arr, $product_id, $ship_state, $indirect_tax_exempt);

          //Get Indirect Tax for Customer TaxCode (juridiction):
          $tax_code = get_user_meta($user_id,'mm_field_TXC',true) ;
          if ($tax_code){
             $price_arr = $this->override_wholesale_price_with_indirect_tax_jurisdiction($price_arr, $product_id, $tax_code, $indirect_tax_exempt);
          }
          return $price_arr;
       
        }

        public function override_wholesale_price_cart($price_arr, $product_id, $user_wholesale_role, $cart_item, $cart_object) {

      	  global $woocommerce;

        	// Get cart_item_key
          $cart_item_key = null;
          foreach ($woocommerce->cart->get_cart() as $key => $item) {
             if ($item === $cart_item) {
                 $cart_item_key = $key;
                 break;
             }
          }
      	 

      	  //Get Current User, check customer specific price
          $user_id = get_current_user_id();
          $price_arr = $this->override_wholesale_price_with_user_price($price_arr,$user_id,$product_id);

          //Check if customer is exempt from indirect taxes (still do calculation, because we need basis)
          $indirect_tax_exempt = get_user_meta($user_id,'mm_field_ITX',true);
      	 
          //Get State for indirect Taxes
          //a) Shipping State from Cart
          $ship_state = $cart_object->get_customer()->get_shipping_state();
          if ($ship_state == '' && $user_id<>0){
          	 //B) If cart does not have a state, use customer shipping state
          	 $ship_state = get_user_meta($user_id, 'shipping_state', true );
          	 //C) If customer does not have a shipping state, use customer billing state
          	 if ($ship_state == ''){
            	 $ship_state = get_user_meta($user_id, 'billing_state', true );
          	 }
          }
          $price_arr = $this->override_wholesale_price_with_indirect_tax($price_arr, $product_id, $ship_state, $indirect_tax_exempt);

          //Get Indirect Tax for Customer TaxCode (juridiction):
          $tax_code = get_user_meta($user_id,'mm_field_TXC',true) ;
          if ($tax_code){
             $price_arr = $this->override_wholesale_price_with_indirect_tax_jurisdiction($price_arr, $product_id, $tax_code, $indirect_tax_exempt);
          }
          
          if ($cart_item_key){
           	 $cart_object->cart_contents[$cart_item_key]['indirect_tax_amount'] = $this->indirect_tax_amount;
           	 $cart_object->cart_contents[$cart_item_key]['indirect_tax_basis'] = $this->indirect_tax_basis;
           	 $cart_object->cart_contents[$cart_item_key]['indirect_tax_amount_j1'] = $this->indirect_tax_amount_j1;
           	 $cart_object->cart_contents[$cart_item_key]['indirect_tax_basis_j1'] = $this->indirect_tax_basis_j1;
           	 $cart_object->cart_contents[$cart_item_key]['indirect_tax_amount_j2'] = $this->indirect_tax_amount_j2;
           	 $cart_object->cart_contents[$cart_item_key]['indirect_tax_basis_j2'] = $this->indirect_tax_basis_j2;
          }

          
          return $price_arr;
           
       
        }

        /**
         * Adjusts the wholesale sale price by adding a tax amount.
         *
         * @param float $wholesale_sale_price The original wholesale sale price.
         * @param int $product_id The product ID.
         * @param string $user_wholesale_role The user's wholesale role.
         * @return float The adjusted wholesale sale price.
         */
        public function override_wholesale_sale_price( $wholesale_sale_price, $product_id, $user_wholesale_role ) {

      	    global $woocommerce;

            $user_id = get_current_user_id();
            
            //Get user-specific price, if lower than current sale price
            $current_price = $wholesale_sale_price;
            $wholesale_sale_price = $this->calc_user_price($current_price, $user_id, $product_id);

            //1.0.6 - Only show tax included in shop if woo setting dictates
            if( get_option( 'woocommerce_tax_display_shop' ) != 'incl' ) {
          	  return $wholesale_sale_price;
            }
            
            $total_taxes= 0.00;

            $indirect_tax_exempt = get_user_meta($user_id,'mm_field_ITX',true);
            if ($indirect_tax_exempt == '1'){
            	//Customer is exempt from indirect tax. do not add to price (we dont need basis calculated here)
            	return $wholesale_sale_price;
            }
            
            $ship_state = '';
            if (isset($woocommerce->cart)) {
              $ship_state =   $woocommerce->cart->get_customer()->get_shipping_state();
            }
            if ($ship_state == '' and $user_id<>0){
            	 //If cart does not have a state, use customer state
            	 $ship_state = get_user_meta($user_id, 'shipping_state', true );
            	 //If customer does not have a shipping state, use customer billing state
            	 if ($ship_state == ''){
              	 $ship_state = get_user_meta($user_id, 'billing_state', true );
          	   }
            }
            $this->calc_indirect_taxes($product_id,$ship_state, $wholesale_sale_price, $indirect_tax_exempt);
            $total_taxes += $this->indirect_tax_amount;
            
            //Get Indirect Tax for Customer TaxCode (juridiction):
            $tax_code = get_user_meta($user_id,'mm_field_TXC',true) ;
            if ($tax_code){
               $this->calc_indirect_tax_jurisdiction($product_id, $tax_code, $wholesale_sale_price, $indirect_tax_exempt);
               $total_taxes += ($this->indirect_tax_amount_j1 + $this->indirect_tax_amount_j2);
            }

            // Add the amount to the original wholesale sale price
            $adjusted_price = (float)$wholesale_sale_price + (float)$total_taxes;

            // Ensure the price is returned as a string and trimmed
            return trim( strval( $adjusted_price ) );
        }

        private function override_wholesale_price_with_user_price($price_arr,$user_id,$product_id){
       	
       	   
       	    if ($price_arr['wholesale_price'] == ''){
       	    	 return $price_arr;
       	    }
            
          	 $current_price = $price_arr['wholesale_price'];
             $new_price = $this->calc_user_price($current_price,$user_id,$product_id);
     	  	   $price_arr['wholesale_price'] = $new_price;
             $this->wholsale_price_before_indirect_tax = $new_price;
             return $price_arr;
              
      	
        }

        private function override_wholesale_price_with_indirect_tax($price_arr, $product_id, $ship_state, $indirect_tax_exempt){



           // Do not add indirect tax if option not present.
        	 if ($this->indirect_tax_enabled != 1 ){
        	  	  return $price_arr;  
        	 }  


           if ($price_arr['wholesale_price'] !== ''){

              $this->calc_indirect_taxes($product_id, $ship_state, $this->wholsale_price_before_indirect_tax, $indirect_tax_exempt);
              $price_arr['wholesale_price'] += $this->indirect_tax_amount;
              
           }
           return $price_arr;
        	     
        	
        }

        private function override_wholesale_price_with_indirect_tax_jurisdiction($price_arr, $product_id, $tax_code, $indirect_tax_exempt){


           // Do not add indirect tax if option not present.
        	  if ($this->indirect_tax_enabled != 1 ){
        	  	  return $price_arr;  
        	  }  

           if ($price_arr['wholesale_price'] !== ''){
             $this->calc_indirect_tax_jurisdiction($product_id, $tax_code, $this->wholsale_price_before_indirect_tax, $indirect_tax_exempt);
             $price_arr['wholesale_price'] += ($this->indirect_tax_amount_j1 + $this->indirect_tax_amount_j2);
             return $price_arr;
            }
        }

        /**         
         * Calulates the indirect tax (by State).
         *
         * set the value for property indirect_tax_amount, basis
         *
         * @param int $product_id The product ID.
         * @param string $ship_state The user's state.
         * @return Void
         */
        private function calc_indirect_taxes($product_id, $ship_state, $wholesale_price, $indirect_tax_exempt){

             	   global $wpdb;

                 $indirect_tax_type = get_post_meta( $product_id, 'mm_indirect_tax_type', true );
                 if ($indirect_tax_type == '') {
                    $this->indirect_tax_amount = 0.00;
                    $this->indirect_tax_basis = 0.00;
                  	return;
                 }

                 // If customer is not exempt from all taxes, check if exempt from this type
                 // Note: we store a user meta field called indirect_tax_exemptions which is a pipe delimeted array of exempt tax types
                 if ($indirect_tax_exempt != 1 ){
                    $user_id = get_current_user_id();
                    $indirect_tax_exemptions = get_user_meta($user_id,'indirect_tax_exemptions',true);
                    $tax_exempt_array = explode('|', $indirect_tax_exemptions);
                    $indirect_tax_exempt = in_array($indirect_tax_type, $tax_exempt_array);
                 }


            	   //If we dont have a state from the order/customer, use shop state
            	   //Note: A wholesale user must log in in order to get prices overridden 
            	   //This will never apply to non-logged in customer.
            	   //It is a fail-safe, in case customer has an incomplete address
            	   if ($ship_state == ''){
            	  	  // The country/state
                    $store_raw_country = get_option( 'woocommerce_default_country' );
                    // Split the country/state
                    $split_country = explode( ":", $store_raw_country );
                    // Country and state separated:
                    //$store_country = $split_country[0];
                    //$store_state   = $split_country[1];
                    $ship_state = $split_country[1];
            	   }

                  //get indirect taxes for state
                 //2.1.0 - Use new database file for efficiency
                 //2.1.2 - Changed Computation to use Last Cost
                 $table_name =  $wpdb->prefix . 'mm_indirect_tax';
                 $query = 'SELECT * FROM '.$table_name.' WHERE indirect_tax_type='.$indirect_tax_type.' AND indirect_tax_state="'.$ship_state.'" LIMIT 1;';
                 $results = $wpdb->get_row($query);

                 if(empty($results)){
                    $this->indirect_tax_amount = 0.00;
                    $this->indirect_tax_basis = 0.00;
                  	return;
                  }

               	  //2.1.0 - set fields from database query result
               	  $stored_rate = $results->indirect_tax_rate;
               	  $indirect_tax_method = $results->indirect_tax_calc_method;

                  switch ($indirect_tax_method){
                  	 case 1:
                  	     //Percent of cost
                  	     $indirect_tax_basis =  get_post_meta( $product_id, 'mm_product_last_cost', true ) ;
                         $indirect_tax_rate =  $stored_rate/100;
                  	     break;
                  	 case 2:
                  	     //Flat rate
                  	     $indirect_tax_basis =  1; 
                         $indirect_tax_rate =  $stored_rate;
                  	     break;
                  	 case 3:
                  	     //Net Weight
                  	     $indirect_tax_basis =  get_post_meta( $product_id, 'mm_product_net_weight', true );
                         $indirect_tax_rate =  $stored_rate;
                  	     break;
                  	 case 4:
                  	     //Tax 1 Basis
                  	     $indirect_tax_basis =  get_post_meta( $product_id, 'mm_product_basis_1', true );
                         $indirect_tax_rate =  $stored_rate;
                  	     break;
                  	 case 5:
                  	     //Tax 2 Basis
                  	     $indirect_tax_basis =  get_post_meta( $product_id, 'mm_product_basis_2', true );
                         $indirect_tax_rate =  $stored_rate;
                  	     break;
                  	 case 6:
                  	     //Tax 3 Basis
                  	     $indirect_tax_basis =  get_post_meta( $product_id, 'mm_product_basis_3', true );
                         $indirect_tax_rate =  $stored_rate;
                  	     break;
                  	 case 10:
                  	     // Percent of Price
                  	     $indirect_tax_basis =  $wholesale_price;  
                         $indirect_tax_rate =  $stored_rate/100;
                  	     break;
                    default:
                        $indirect_tax_basis = get_post_meta( $product_id, 'mm_product_last_cost', true );
                        $indirect_tax_rate =  $stored_rate/100;
                        break;
                  }
                  $indirect_tax_basis = ( $indirect_tax_basis == '') ? 0 : $indirect_tax_basis;
                  $indirect_tax_rate = ( $indirect_tax_rate == '') ? 0 : $indirect_tax_rate;
                  $indirect_tax_amount = $indirect_tax_exempt ? 0.00: $indirect_tax_basis * $indirect_tax_rate;
        	
                  $this->indirect_tax_amount = $indirect_tax_amount;
                  $this->indirect_tax_basis = $indirect_tax_basis;
        }

        /**         
         * Calulates the indirect tax (by Jurisdiction).
         *
         * sets the values for properties indirect_tax_amount_j1, amount_j2, basis_j1 and basis_j2
         *
         * @param int $product_id The product ID.
         * @param string $ship_state The user's state.
         * @return Void
         */
         private function calc_indirect_tax_jurisdiction($product_id, $tax_code, $wholesale_price, $indirect_tax_exempt){

             global $wpdb;

       	   
       	     $this->indirect_tax_amount_j1 = 0.00;
       	     $this->indirect_tax_basis_j1 = 0.00;
             $this->indirect_tax_amount_j2 = 0.00;
             $this->indirect_tax_basis_j2 = 0.00;

             $indirect_tax_type = get_post_meta( $product_id, 'mm_indirect_tax_type', true );
             if ($indirect_tax_type == '') {
             	 return; // no tax type for this product
             }
             
             // If customer is not exempt from all taxes, check if exempt from this type
             // Note: we store a user meta field called indirect_tax_exemptions which is a pipe delimeted array of exempt tax types
             if ($indirect_tax_exempt != 1 ){
                $user_id = get_current_user_id();
                $indirect_tax_exemptions = get_user_meta($user_id,'indirect_tax_exemptions',true);
                $tax_exempt_array = explode('|', $indirect_tax_exemptions);
                $indirect_tax_exempt = in_array($indirect_tax_type, $tax_exempt_array);
             }

             //get indirect taxes for taxcode
             $table_name =  $wpdb->prefix . 'mm_indirect_tax_jurisdiction';
             $query = 'SELECT * FROM '.$table_name.' WHERE indirect_tax_type='.$indirect_tax_type.' AND indirect_tax_code="'.$tax_code.'" LIMIT 1;';
             $results = $wpdb->get_row($query);

             if(empty($results)){
              	return;  //no jurisdictions in db
             }

             for ($i = 1; $i <= 2; $i++) {
             	
                  //Set Properties from loop
                  $rate_property = "indirect_tax_rate_j" . $i;
                  $calc_method_property = "indirect_tax_calc_method_j" . $i;

                  //Set calculation variables from db query
                  $stored_rate = $results->$rate_property;
                  $indirect_tax_method = $results->$calc_method_property;

                  switch ($indirect_tax_method) {
                      case 1:
                          // Percent of cost
                          $indirect_tax_basis = get_post_meta($product_id, 'mm_product_last_cost', true);
                          $indirect_tax_rate = $stored_rate / 100;
                          break;
                      case 2:
                          // Flat rate
                          $indirect_tax_basis = 1;
                          $indirect_tax_rate = $stored_rate;
                          break;
                      case 3:
                          // Net Weight
                          $indirect_tax_basis = get_post_meta($product_id, 'mm_product_net_weight', true);
                          $indirect_tax_rate = $stored_rate;
                          break;
                      case 4:
                          // Tax 1 Basis
                          $indirect_tax_basis = get_post_meta($product_id, 'mm_product_basis_1', true);
                          $indirect_tax_rate = $stored_rate;
                          break;
                      case 5:
                          // Tax 2 Basis
                          $indirect_tax_basis = get_post_meta($product_id, 'mm_product_basis_2', true);
                          $indirect_tax_rate = $stored_rate;
                          break;
                      case 6:
                          // Tax 3 Basis
                          $indirect_tax_basis = get_post_meta($product_id, 'mm_product_basis_3', true);
                          $indirect_tax_rate = $stored_rate;
                          break;
                      case 10:
                          // Percent of Price
                          $indirect_tax_basis = $wholesale_price;
                          $indirect_tax_rate = $stored_rate / 100;
                          break;
                      default:
                          $indirect_tax_basis = '';
                          $indirect_tax_rate = '';
                          break;
                  }

                  $indirect_tax_basis = ($indirect_tax_basis == '') ? 0 : $indirect_tax_basis;
                  $indirect_tax_rate = ($indirect_tax_rate == '') ? 0 : $indirect_tax_rate;
                  $indirect_tax_amount = $indirect_tax_exempt ? 0.00: $indirect_tax_basis * $indirect_tax_rate;

                  $property_name = "indirect_tax_amount_j" . $i;
                  $this->$property_name = $indirect_tax_amount;

                  $property_name = "indirect_tax_basis_j" . $i;
                  $this->$property_name = $indirect_tax_basis;
             }

         	   return;

         }


        private function calc_user_price($current_price,$user_id,$product_id){
           	  
         	   global $wpdb;

             //Check if this option is allowed
             $customer_pricing_enable = get_option('mm_customer_pricing_enable',1);
             if ($customer_pricing_enable != 1){
       	    	 return $current_price;
             }

           	  $lowest_price = get_post_meta($product_id,'mm_product_lowest_price',true);
          	  //$allow_lower_than_minimum = get_option('mm_allow_price_lower_than_minimum',0);  This flag is no longer considered (4.6.0)
              $mm_allow_sale_price_if_lower = get_option('mm_allow_sale_price_if_lower',0);
      	
              //get user price
              //2.0.0 - Use new database file for efficiency
              $table_name =  $wpdb->prefix . 'mm_customer_price';
              $query = 'SELECT * FROM '.$table_name.' WHERE product_id='.$product_id.' AND user_id='.$user_id.' LIMIT 1;';
              $results = $wpdb->get_row($query);
          
              if ($results){

              	  //2.0.0 - set fields from database query result
              	  $amount = $results->amount;
              	  $method = $results->method;
              	  $start_date = $results->start_date;
              	  $end_date = $results->end_date;
              	  
              	  
              	  $today_date = date('Ymd');
              	  if ($amount != ''){

              	  	  //Check if there is a date range (dates are stored as YYYYMMDD) and today is outside of range
              	  	  if ($today_date < $start_date || ($end_date != '' && $end_date < $today_date)){
              	  	  	 return $current_price;
              	  	  }
              	  	  
              	  	  // Calculate New Price based on method, amount, and product data
              	  	  switch ($method) { 	
              	  	  	case '1': //Fixed Price
              	  	  	  $new_price = $amount;
              	  	  	  $new_price = ($lowest_price != '' && $new_price < $lowest_price) ? $lowest_price : $new_price ;
              	  	  	  break;
              	  	  	case '2' : //Discount Percent  
              	  	  	  $discount = $current_price * ($amount/100);
              	  	  	  $new_price = $current_price - $discount;
              	  	  	  $new_price = ($lowest_price != '' && $new_price < $lowest_price) ? $lowest_price : $new_price ;
              	  	  	  break;
              	  	   case '3' : //Markup over Last Cost
              	  	     $last_cost = get_post_meta($product_id,'mm_product_last_cost',true);
              	  	     if ($last_cost != ''){
              	  	     	  $markup = $last_cost * ($amount/100);
              	  	     	  $new_price = $current_price + $markup;
              	  	     	  $new_price = ($lowest_price != '' && $new_price < $lowest_price) ? $lowest_price : $new_price ;
              	  	     }
              	  	     break;
              	  	   case '4' : //Markup over Average Cost
              	  	     $average_cost = get_post_meta($product_id,'mm_product_cost',true);
              	  	     if ($average_cost != ''){
              	  	     	  $markup = $average_cost * ($amount/100);
              	  	     	  $new_price = $current_price + $markup;
              	  	     	  $new_price = ($lowest_price != '' && $new_price < $lowest_price) ? $lowest_price : $new_price ;
              	  	     }
              	  	     break;
              	  	   case '5' : //Fixed amount over Last Cost
              	  	     $last_cost = get_post_meta($product_id,'mm_product_last_cost',true);
              	  	     if ($last_cost != ''){
              	  	     	  $new_price = $last_cost + $amount;
              	  	     	  $new_price = ($lowest_price != '' && $new_price < $lowest_price) ? $lowest_price : $new_price ;
                	  	  }
              	  	     break;
              	  	   case '6' : //Fixed amount over Average Cost
              	  	     $average_cost = get_post_meta($product_id,'mm_product_cost',true);
              	  	     if ($average_cost != ''){
              	  	     	  $new_price = $average_cost + $amount;
              	  	     	  $new_price = ($lowest_price != '' && $new_price < $lowest_price) ? $lowest_price : $new_price ;
                	  	  }
              	  	     break;
              	  	   case '7' : //Price Level 1
              	  	     $price_level_1 = get_post_meta($product_id,'mm_price_1_wholesale_price',true);
              	  	     if ($price_level_1 != ''){
              	  	     	  $new_price = $price_level_1;
                	  	  }
              	  	     break;
              	  	   case '8' : //Price Level 2
              	  	     $price_level_2 = get_post_meta($product_id,'mm_price_2_wholesale_price',true);
              	  	     if ($price_level_2 != ''){
              	  	     	  $new_price = $price_level_2;
                	  	  }
              	  	     break;
              	  	   case '9' : //Price Level 3
              	  	     $price_level_3 = get_post_meta($product_id,'mm_price_3_wholesale_price',true);
              	  	     if ($price_level_3 != ''){
              	  	     	  $new_price = $price_level_3;
                	  	  }
              	  	     break;
              	  	   case '10' : //Price Level 4
              	  	     $price_level_4 = get_post_meta($product_id,'mm_price_4_wholesale_price',true);
              	  	     if ($price_level_4 != ''){
              	  	     	  $new_price = $price_level_4;
                	  	  }
              	  	     break;
              	  	   case '11' : //Price Level 5
              	  	     $price_level_5 = get_post_meta($product_id,'mm_price_5_wholesale_price',true);
              	  	     if ($price_level_5 != ''){
              	  	     	  $new_price = $price_level_5;
                	  	  }
              	  	     break;
              	  	   case '12' : //Price Level 6
              	  	     $price_level_6 = get_post_meta($product_id,'mm_price_6_wholesale_price',true);
              	  	     if ($price_level_6 != ''){
              	  	     	  $new_price = $price_level_6;
                	  	  }
              	  	     break;
              	  	   case '13' : //Price Level 7
              	  	     $price_level_7 = get_post_meta($product_id,'mm_price_7_wholesale_price',true);
              	  	     if ($price_level_7 != ''){
              	  	     	  $new_price = $price_level_7;
                	  	  }
              	  	     break;
              	  	   case '14' : //Price Level 8
              	  	     $price_level_8 = get_post_meta($product_id,'mm_price_8_wholesale_price',true);
              	  	     if ($price_level_8 != ''){
              	  	     	  $new_price = $price_level_8;
                	  	  }
              	  	     break;
              	  	  	default:  // Do not change price
              	  	  	  $new_price = $current_price;
              	  	  	  break;
              	  	  }
              	  	  
              	  }
              }
              else {
              	 $new_price = $current_price;
              }
              
              //2.1.4 - Check if customer price is greater than current price.
              //4.8.0 - Removed.  We no longer consider if there is a sale price that would be lower.
              //        We added a rule in mm/website that if a customer price is used, promo is not considered
              //if ($new_price > $current_price){
              //	  $new_price = $current_price;
              //}
              
              //Check if there is a sale price, and its lower than the new price.
              //This was removed because we now support Wholesale Prices Premium discounts, which are automatically applied by that plug-in (4.6.0)
              //if ($mm_allow_sale_price_if_lower == 1){
              //   $product = wc_get_product( $product_id );
              //   if( $product->is_on_sale() ) {
              //      $sale_price = $product->get_sale_price();
              //      if ($sale_price < $new_price){
              //      	  $new_price = $sale_price;
              //      }
              //   }
              //}

              //Regardless of whether new price was reset above, or still contains the price level price, be sure it is not lower than the lowest price (unless allowed)
              //This was moved to only take effect for Customer specific pricing (4.6.0)
              //if ($allow_lower_than_minimum != 1){
          	  //   $new_price = ($lowest_price != '' && $new_price < $lowest_price) ? $lowest_price : $new_price ;
              //}

        	    return $new_price;
        	
        }


    
    
        /**
         * Execute model.
         *
         * @since 1.0.0
         * @access public
         */
        public function run() {
        	

              $this->indirect_tax_enabled = get_option('mm_indirect_tax_enable',0);

              add_filter('wwp_filter_wholesale_price_shop_v2', array($this,'override_wholesale_price_shop'), 100, 3);

              add_filter('wwp_filter_wholesale_price_cart', array($this,'override_wholesale_price_cart'), 100, 5);

              add_filter( 'wwp_pass_wholesale_price_through_taxing', array( $this, 'override_wholesale_sale_price' ), 10, 3 );

 
        }

    }
}