<?php if (!defined('ABSPATH')) {
    exit;
}
// Exit if accessed directly

if (!class_exists('MM_WOO_WHOLESALE')) {

    /**
     * Model that houses the logic of MM WooCommerce Wholesale Related Functions (Custom Woo Fields, Indirect Taxes, Customer Pricing, etc.)
     * Note: This functionality requires the Wholesale Prices Premium Plug In.  The code uses filters/logic provided by that plug-in
     *
     * @since 1.0.0
     */
    class MM_WOO_WHOLESALE {

        
        /*
        |--------------------------------------------------------------------------
        | Class Properties
        |--------------------------------------------------------------------------
         */

        private $indirect_tax_enabled = false ;
        private $show_indirect_tax_shop = false ;
        private $show_restriction_notation_shop = false;
        private $customer_pricing_enable = false ;
        private $debug_active = false;
        private $indirect_tax_exempt = false;
        private $tax_exempt_array = [];
        private $indirect_tax_amount = 0.00;
        private $indirect_tax_basis = 0.00;
        private $indirect_tax_amount_j1 = 0.00;
        private $indirect_tax_basis_j1 = 0.00;
        private $indirect_tax_amount_j2 = 0.00;
        private $indirect_tax_basis_j2 = 0.00;
        private $indirect_tax_total_amount = 0.00;
        private $wholesale_price_before_indirect_tax = 0.00;
        private $indirect_tax_allow_cart_detail = false;
        private $indirect_tax_note_message = '';
        private $already_processed = [];
        private $mm_allow_cache = false;
        private $cache_duration = 30; //days
        private $user_wholesale_role = array();
        private $woo_extender = null;
       
        /**
         * Property that holds the single main instance of MM_WOO_WHOLESALE.
         *
         * @since 1.0.0
         * @access private
         * @var MM_REST_API
         */
        private static $_instance;
    

        /*
        |--------------------------------------------------------------------------
        | Class Methods
        |--------------------------------------------------------------------------
         */

        /**
         * Ensure that only one instance of MM_REST_API is loaded or can be loaded (Singleton Pattern).
         *
         * @since 1.12.0
         * @access public
         *
         * @param array $dependencies Array of instance objects of all dependencies of MM_WOO_WHOLESALE model.
         * @return MM_WOO_WHOLESALE
         */
        public static function instance() {

            if (!self::$_instance instanceof self) {
                self::$_instance = new self();
            }

            return self::$_instance;

        }

        public function mm_setup_user_properties(){
              $user_id = get_current_user_id();
              $user = get_userdata($user_id);
              if ( $user && is_object( $user ) ) {
                 $this->user_wholesale_role = $user->roles;
              }
              $indirect_tax_exemptions = get_user_meta($user_id,'indirect_tax_exemptions',true);
              $this->tax_exempt_array = explode('|', $indirect_tax_exemptions);
              $this->indirect_tax_exempt = get_user_meta($user_id,'mm_field_ITX',true);
        	
        }


        public function override_wholesale_price_shop($price_arr, $product_id, $user_wholesale_role) {


      	  global $woocommerce;



      	  //Get Current User, check customer specific price
          $user_id = get_current_user_id();

          //5.1.0 if no user, do not calculate
          if ($user_id == 0){
             return $price_arr;	
          }
          
        	if($this->debug_active){
               MM_Connector_Functions::debug_log('SHOP-------Overriding Price in Shop for user: '.$user_id.' Product Id: '.$product_id.'***********************',MM_Debug_Mode::WOOCOMMERCE);
         	}
         	
          $price_arr = $this->override_wholesale_price_with_user_price($price_arr,$user_id,$product_id);

          //1.0.6 - Only show tax included in shop if woo setting dictates
          //5.1.0 - Changed to our own plugin setting
          //if( get_option( 'woocommerce_tax_display_shop' ) != 'incl' ) {
          if (!$this->show_indirect_tax_shop){
          	return $price_arr;
          }

          if ($this->indirect_tax_exempt == '1'){
          	//Customer is exempt from indirect tax. do not add to price
          	//We don't need basis here, just return (do not do calculation)
          	return $price_arr;
          }

        
          //Get Indirect Tax for Cart State:
          $ship_state = '';
          if (isset($woocommerce->cart)) {
             $ship_state =   $woocommerce->cart->get_customer()->get_shipping_state();
          }
          if ($ship_state == '' and $user_id<>0){
          	 //If cart does not have a state, use customer state
          	 $ship_state = get_user_meta($user_id, 'shipping_state', true );
          	 //If customer does not have a shipping state, use customer billing state
          	 if ($ship_state == ''){
            	 $ship_state = get_user_meta($user_id, 'billing_state', true );
          	 }
          }
          $price_arr = $this->override_wholesale_price_with_indirect_tax($price_arr, $product_id, $ship_state);

          //Get Indirect Tax for Customer TaxCode (juridiction):
          $tax_code = get_user_meta($user_id,'mm_field_TXC',true) ;
          if ($tax_code){
             $price_arr = $this->override_wholesale_price_with_indirect_tax_jurisdiction($price_arr, $product_id, $tax_code);
          }
          return $price_arr;
       
        }

        public function override_wholesale_sale_price_shop($wholesale_sale_price, $product_id, $user_wholesale_role) {

        	 if(is_cart() || is_checkout()){
        	 	 return $wholesale_sale_price; //do not do in cart or checkout
        	 }
        	 
        	 if ($wholesale_sale_price == ''){
        	 	  //Not on sale - do not compute
        	 	  return $wholesale_sale_price;
        	 }

        	 if($this->debug_active){
               MM_Connector_Functions::debug_log('SHOP SALE-------Overriding Sale Price in Shop for user: '.$user_id.' Product Id: '.$product_id.'***********************',MM_Debug_Mode::WOOCOMMERCE);
           }


           $wholesale_sale_price = $this->override_wholesale_sale_price( $wholesale_sale_price, $product_id, $user_wholesale_role );
           return $wholesale_sale_price;
        	
        }

        public function override_wholesale_price_cart($price_arr, $product_id, $user_wholesale_role, $cart_item, $cart_object) {

      	  global $woocommerce;
          
      	  //Get Current User, check customer specific price
          $user_id = get_current_user_id();

          //5.1.0 if no user, do not calculate
          if ($user_id == 0){
             return $price_arr;	
          }

         	if($this->debug_active){
               MM_Connector_Functions::debug_log('CART-------Overriding price in Cart for user: '.$user_id.' Product Id: '.$product_id.' ******************'.$backtrace,MM_Debug_Mode::WOOCOMMERCE);
          }

        	// Get cart_item_key (for storing our values to cart, and for logic below to check already processed)
          $cart_item_key = null;
          foreach ($woocommerce->cart->get_cart() as $key => $item) {
             if ($item === $cart_item) {
                 $cart_item_key = $key;
                 break;
             }
          }
          
          //Log action if necessary.  If wholesale pricing does not exist, do not override.
          if ($price_arr['wholesale_price'] == ''){
          	if($this->debug_active){
               MM_Connector_Functions::debug_log('Cannot Override price in Cart for unique key: '.$unique_key.' user: '.$user_id.' Product Id: '.$product_id.' No wholesale pricing for this user. ***********************',MM_Debug_Mode::CUST_PRICE);
          	}
          	return $price_arr;
          }

  
          $price_arr = $this->override_wholesale_price_with_user_price($price_arr,$user_id,$product_id);

      	 
          //Get State for indirect Taxes
          //a) Shipping State from Cart
          $ship_state = $cart_object->get_customer()->get_shipping_state();
          if ($ship_state == '' && $user_id<>0){
          	 //B) If cart does not have a state, use customer shipping state
          	 $ship_state = get_user_meta($user_id, 'shipping_state', true );
          	 //C) If customer does not have a shipping state, use customer billing state
          	 if ($ship_state == ''){
            	 $ship_state = get_user_meta($user_id, 'billing_state', true );
          	 }
          }
          $price_arr = $this->override_wholesale_price_with_indirect_tax($price_arr, $product_id, $ship_state);

          //Get Indirect Tax for Customer TaxCode (juridiction):
          $tax_code = get_user_meta($user_id,'mm_field_TXC',true) ;
          if ($tax_code){
             $price_arr = $this->override_wholesale_price_with_indirect_tax_jurisdiction($price_arr, $product_id, $tax_code);
          }
          
          if ($cart_item_key){
           	 $cart_object->cart_contents[$cart_item_key]['indirect_tax_amount'] = $this->indirect_tax_amount;
           	 $cart_object->cart_contents[$cart_item_key]['indirect_tax_basis'] = $this->indirect_tax_basis;
           	 $cart_object->cart_contents[$cart_item_key]['indirect_tax_amount_j1'] = $this->indirect_tax_amount_j1;
           	 $cart_object->cart_contents[$cart_item_key]['indirect_tax_basis_j1'] = $this->indirect_tax_basis_j1;
           	 $cart_object->cart_contents[$cart_item_key]['indirect_tax_amount_j2'] = $this->indirect_tax_amount_j2;
           	 $cart_object->cart_contents[$cart_item_key]['indirect_tax_basis_j2'] = $this->indirect_tax_basis_j2;
           	 $cart_object->cart_contents[$cart_item_key]['indirect_tax_total_amount'] = $this->indirect_tax_amount + $this->indirect_tax_amount_j1 + $this->indirect_tax_amount_j2;
          }
          
          return $price_arr;
           
       
        }

        public function override_wholesale_html_price($wholesale_price, $price, $product, $user_wholesale_role, $wholesale_price_title_text, $raw_wholesale_price, $source, $return_wholesale_price_only ){

      	  global $woocommerce;

          //Do not do this in admin
          if(is_admin()){
          	return $wholesale_price;
          }


          //This filter only applies to us if this flag is true
          if(!$return_wholesale_price_only){
          	return $wholesale_price;
          }

      	  //Get Current User, check customer specific price
          $user_id = get_current_user_id();

          //5.1.0 if no user, do not calculate
          if ($user_id == 0){
             return $wholesale_price;	
          }

          $product_id = $product->get_id();
          
        	if($this->debug_active){
               MM_Connector_Functions::debug_log('HTML-------Overriding Price in Shop for user: '.$user_id.' Product Id: '.$product_id.'***********************',MM_Debug_Mode::CUST_PRICE);
         	}

          //Get user-specific price, if lower than current sale price
          $current_price = $wholesale_price;
          $wholesale_price = $this->calc_user_price($current_price, $user_id, $product_id);
          $this->wholesale_price_before_indirect_tax = $wholesale_price;

          //1.0.6 - Only show tax included in shop if woo setting dictates
          //5.1.0 - Changed to our own plugin setting
          //if( get_option( 'woocommerce_tax_display_shop' ) != 'incl' ) {
          if (!$this->show_indirect_tax_shop){
        	  return $wholesale_price;
          }
          
          $this->indirect_tax_total_amount = 0.00;

          if ($this->indirect_tax_exempt == '1'){
          	//Customer is exempt from indirect tax. do not add to price (we dont need basis calculated here)
          	return $wholesale_price;
          }
          
          $ship_state = '';
          if (isset($woocommerce->cart)) {
            $ship_state =   $woocommerce->cart->get_customer()->get_shipping_state();
          }
          if ($ship_state == '' and $user_id<>0){
          	 //If cart does not have a state, use customer state
          	 $ship_state = get_user_meta($user_id, 'shipping_state', true );
          	 //If customer does not have a shipping state, use customer billing state
          	 if ($ship_state == ''){
            	 $ship_state = get_user_meta($user_id, 'billing_state', true );
        	   }
          }
          $this->calc_indirect_taxes($product_id,$ship_state, $wholesale_price);
          $this->indirect_tax_total_amount += $this->indirect_tax_amount;
          
          //Get Indirect Tax for Customer TaxCode (juridiction):
          $tax_code = get_user_meta($user_id,'mm_field_TXC',true) ;
          if ($tax_code){
             $this->calc_indirect_tax_jurisdiction($product_id, $tax_code, $wholesale_price);
             $this->indirect_tax_total_amount += ($this->indirect_tax_amount_j1 + $this->indirect_tax_amount_j2);
          }

          // Add the amount to the original wholesale sale price
          $adjusted_price = (float)$wholesale_price + (float)$this->indirect_tax_total_amount;

          // Ensure the price is returned as a string and trimmed
          return trim( strval( $adjusted_price ) );

        	
        }

        public function maybe_add_notations_shop() {
        	 global $product;
        	 
        	 $restricted = false;
        	 $indirect_tax_found = false;

           if ( ! $product instanceof WC_Product ) {
             return; //Only applies to products
           }
           
           if ( is_a( $product, 'WC_Product_Variable' )){
           	  //Variable product
              // Check all variations for indirect tax
              $variation_ids = $product->get_children();
              foreach ( $variation_ids as $variation_id ) {

              	  if ($this->indirect_tax_enabled && $this->show_indirect_tax_shop && $this->indirect_tax_allow_cart_detail){
                     $user_wholesale_role = $this->user_wholesale_role;
                     $wholesale_price = WWP_Wholesale_Prices::getProductWholesalePrice( $variation_id, $user_wholesale_role );
                     $wholesale_price = $this->override_wholesale_sale_price( $wholesale_price, $variation_id, $user_wholesale_role );
                  
            
                     if ( $this->indirect_tax_total_amount > 0 ) {
                        $indirect_tax_found = true;
                     }
              	  }
                  if ($this->show_restriction_notation_shop){
                     if ($this->woo_extender->mm_is_product_restricted($variation_id,$problems_found)){
                     	 $restricted=true;
                     }
                  }


           	  }
           }
           else {

              $product_id = $product->get_id();


           	  if ($this->indirect_tax_enabled && $this->show_indirect_tax_shop && $this->indirect_tax_allow_cart_detail){
                 $user_wholesale_role = $this->user_wholesale_role;
           	     $wholesale_sale_price = WWP_Wholesale_Prices::getProductWholesalePrice($product_id, $user_wholesale_role) ; // Gets the current price
                 $new_price = $this->override_wholesale_sale_price( $wholesale_sale_price, $product_id, $user_wholesale_role );
                 $indirect_tax_total_amount = $this->indirect_tax_total_amount;
                 if ( $this->indirect_tax_total_amount > 0 ) {
                    $indirect_tax_found = true;
                 }
              }

              if ($this->show_restriction_notation_shop){
                 if ($this->woo_extender->mm_is_product_restricted($product_id,$problems_found)){
                 	 $restricted=true;
                 }
              }


          }
           if ($indirect_tax_found){
           	  $message = $this->indirect_tax_note_message;
              echo "<span>{$message}</span>";
           }
           
           if ($restricted){
           	  $messages = $this->woo_extender->mm_get_restriction_messages($problems_found);
           	  foreach ($messages as $message) {
                 echo "<span>{$message}</span>";
           	  } 
           }
           
           

        }
        
        public function maybe_add_restriction_message_to_stock_desc($html, $single_variation){

        	  $notations = '';
        	  $variation_id = $single_variation->get_id();
            if ($this->woo_extender->mm_is_product_restricted($variation_id,$problems_found)){
               $notations=$this->woo_extender->mm_get_restriction_notations($problems_found);
               $html = '<span>Restriction(s):'.$notations.'</span>';
            }
            return $html;
        	
        }
        
        
        /**
         * Adjusts the wholesale sale price by customer pricing and possibly adding a tax amount.
         *
         * @param float $wholesale_sale_price The original wholesale sale price.
         * @param int $product_id The product ID.
         * @param string $user_wholesale_role The user's wholesale role.
         * @return float The adjusted wholesale sale price.
         */
        public function override_wholesale_sale_price( $wholesale_sale_price, $product_id, $user_wholesale_role ) {

      	    global $woocommerce;

            $user_id = get_current_user_id();
            
            //5.1.0 if no user, do not calculate
            if ($user_id == 0){
               return $wholesale_sale_price;	
            }

          	if($this->debug_active){
               MM_Connector_Functions::debug_log('Override Wholesale Sale Price called for user: '.$user_id.' Product Id: '.$product_id.'***********************',MM_Debug_Mode::WOOCOMMERCE);
           	}

            //Get user-specific price, if lower than current sale price
            $current_price = $wholesale_sale_price;
            $wholesale_sale_price = $this->calc_user_price($current_price, $user_id, $product_id);
            $this->wholesale_price_before_indirect_tax = $wholesale_sale_price;

            //1.0.6 - Only show tax included in shop if woo setting dictates
            //5.1.0 - Changed to our own plugin setting
            //if( get_option( 'woocommerce_tax_display_shop' ) != 'incl' ) {
            if (!$this->show_indirect_tax_shop){
          	  return $wholesale_sale_price;
            }
            
            $this->indirect_tax_total_amount = 0.00;

            if ($this->indirect_tax_exempt == '1'){
            	//Customer is exempt from indirect tax. do not add to price (we dont need basis calculated here)
            	return $wholesale_sale_price;
            }
            
            $ship_state = '';
            if (isset($woocommerce->cart)) {
              $ship_state =   $woocommerce->cart->get_customer()->get_shipping_state();
            }
            if ($ship_state == '' and $user_id<>0){
            	 //If cart does not have a state, use customer state
            	 $ship_state = get_user_meta($user_id, 'shipping_state', true );
            	 //If customer does not have a shipping state, use customer billing state
            	 if ($ship_state == ''){
              	 $ship_state = get_user_meta($user_id, 'billing_state', true );
          	   }
            }
            $this->calc_indirect_taxes($product_id,$ship_state, $wholesale_sale_price);
            $this->indirect_tax_total_amount += $this->indirect_tax_amount;
            
            //Get Indirect Tax for Customer TaxCode (juridiction):
            $tax_code = get_user_meta($user_id,'mm_field_TXC',true) ;
            if ($tax_code){
               $this->calc_indirect_tax_jurisdiction($product_id, $tax_code, $wholesale_sale_price);
               $this->indirect_tax_total_amount += ($this->indirect_tax_amount_j1 + $this->indirect_tax_amount_j2);
            }

            // Add the amount to the original wholesale sale price
            $adjusted_price = (float)$wholesale_sale_price + (float)$this->indirect_tax_total_amount;

            // Ensure the price is returned as a string and trimmed
            return trim( strval( $adjusted_price ) );
        }

        private function override_wholesale_price_with_user_price($price_arr,$user_id,$product_id){

       	    if ($user_id == 0){
       	    	 return $price_arr;
       	    }
       	   
       	    if ($price_arr['wholesale_price'] == ''){
       	    	 return $price_arr;
       	    }
            
          	 $current_price = $price_arr['wholesale_price'];
             $new_price = $this->calc_user_price($current_price,$user_id,$product_id);
     	  	   $price_arr['wholesale_price'] = $new_price;
             $this->wholesale_price_before_indirect_tax = $new_price;
             return $price_arr;
              
      	
        }

        private function override_wholesale_price_with_indirect_tax($price_arr, $product_id, $ship_state){



           // Do not add indirect tax if option not present.
        	 if ($this->indirect_tax_enabled != 1 ){
        	  	  return $price_arr;  
        	 }  

         	 if($this->debug_active){
              MM_Connector_Functions::debug_log('Overriding price with indirect tax for Product Id: '.$product_id.' State: '.$ship_state.' Exempt: '.$this->indirect_tax_exempt,MM_Debug_Mode::INDIRECT_TAX);
           }

           if ($price_arr['wholesale_price'] !== ''){

              $this->calc_indirect_taxes($product_id, $ship_state, $this->wholesale_price_before_indirect_tax);

              $price_arr['wholesale_price'] += $this->indirect_tax_amount;

              if($this->debug_active){
                 MM_Connector_Functions::debug_log('Indirect Tax Calculated for Product Id: '.$product_id.' = '.number_format($this->indirect_tax_amount, 2).'New Wholesale Price: '.$price_arr['wholesale_price'],MM_Debug_Mode::INDIRECT_TAX);
              }

              
           }
           return $price_arr;
        	     
        	
        }

        private function override_wholesale_price_with_indirect_tax_jurisdiction($price_arr, $product_id, $tax_code){


           // Do not add indirect tax if option not present.
        	  if ($this->indirect_tax_enabled != 1 ){
        	  	  return $price_arr;  
        	  }  

           if($this->debug_active){
              MM_Connector_Functions::debug_log('Overriding price with indirect tax jurisdiction for Product Id: '.$product_id.' Tax Code: '.$tax_code.' Exempt: '.$this->indirect_tax_exempt,MM_Debug_Mode::INDIRECT_TAX);
           }

           if ($price_arr['wholesale_price'] !== ''){
             $this->calc_indirect_tax_jurisdiction($product_id, $tax_code, $this->wholesale_price_before_indirect_tax);
             $price_arr['wholesale_price'] += ($this->indirect_tax_amount_j1 + $this->indirect_tax_amount_j2);
          	 if($this->debug_active){
               MM_Connector_Functions::debug_log('Indirect Tax Jurisdiction Calculated for Product Id: '.$product_id.' J1 = '.number_format($this->indirect_tax_amount_j1, 2).' J2 = '.number_format($this->indirect_tax_amount_j2, 2).'New Wholesale Price: '.$price_arr['wholesale_price'],MM_Debug_Mode::INDIRECT_TAX);
             }
             return $price_arr;
            }
        }


        /**         
         * Calulates the indirect tax (by State).
         *
         * set the value for property indirect_tax_amount, basis
         *
         * @param int $product_id The product ID.
         * @param string $ship_state The user's state.
         * @return Void
         */
        private function calc_indirect_taxes($product_id, $ship_state, $wholesale_price){

             	   global $wpdb;

                 $indirect_tax_type = get_post_meta( $product_id, 'mm_indirect_tax_type', true );
                 if ($indirect_tax_type == '') {
          	        if($this->debug_active){
                       MM_Connector_Functions::debug_log('No Indirect Tax Type for Product Id: '.$product_id,MM_Debug_Mode::INDIRECT_TAX);
                    }
                    $this->indirect_tax_amount = 0.00;
                    $this->indirect_tax_basis = 0.00;
                  	return;
                 }

                 // If customer is not exempt from all taxes, check if exempt from this type
                 // Note: we store a user meta field called indirect_tax_exemptions which is a pipe delimeted array of exempt tax types
                 $indirect_tax_exempt = $this->indirect_tax_exempt;
                 if ($indirect_tax_exempt != 1 ){
                    $indirect_tax_exempt = in_array($indirect_tax_type, $this->tax_exempt_array);
                 }
                 


            	   //If we dont have a state from the order/customer, use shop state
            	   //Note: A wholesale user must log in in order to get prices overridden 
            	   //This will never apply to non-logged in customer.
            	   //It is a fail-safe, in case customer has an incomplete address
            	   if ($ship_state == ''){
            	  	  // The country/state
                    $store_raw_country = get_option( 'woocommerce_default_country' );
                    // Split the country/state
                    $split_country = explode( ":", $store_raw_country );
                    // Country and state separated:
                    //$store_country = $split_country[0];
                    //$store_state   = $split_country[1];
                    $ship_state = $split_country[1];
            	   }

                 // Check cache if used (only for products with a tax type)
                 if ($this->mm_allow_cache){
                    $cache_key = "mm_indirect_tax_{$indirect_tax_type}_{$product_id}_{$ship_state}";
                    $cached_tax = MM_Object_Cache::get($cache_key);

                    if (is_array($cached_tax)) {
             	         if($this->debug_active){
                           MM_Connector_Functions::debug_log('Cached indirect tax ['.$cache_key.'] used for Product Id: '.$product_id,MM_Debug_Mode::INDIRECT_TAX);
                       }
                       $this->indirect_tax_amount = ($indirect_tax_exempt) ? 0.00 : $cached_tax['amount'];
                       $this->indirect_tax_basis = $cached_tax['basis'];
                       return;
                    }
                 	
                 }


                 //get indirect taxes for state from database
                 //2.1.0 - Use new database file for efficiency
                 //2.1.2 - Changed Computation to use Last Cost
                 $table_name =  $wpdb->prefix . 'mm_indirect_tax';
                 $query = 'SELECT * FROM '.$table_name.' WHERE indirect_tax_type='.$indirect_tax_type.' AND indirect_tax_state="'.$ship_state.'" LIMIT 1;';
                 $results = $wpdb->get_row($query);

                 if(empty($results)){
                 	  if($this->mm_allow_cache){
                 	  	  $cache_data = ['amount' => 0.00, 'basis' => 0.00];
                 	  	  MM_Object_Cache::set($cache_key, $cache_data, [
                           'product' => $product_id,
                           'indirect_tax_type' => $indirect_tax_type,
                       ], $this->cache_duration * DAY_IN_SECONDS,'indirect_tax');
                 	  }
             	      if ($this->debug_active){
                       MM_Connector_Functions::debug_log('DB Lookup - No tax found for Indirect Tax Type: '.$indirect_tax_type.' State: '.$ship_state,MM_Debug_Mode::INDIRECT_TAX);
                    }

                    $this->indirect_tax_amount = 0.00;
                    $this->indirect_tax_basis = 0.00;
                  	return;
                  }

               	  //2.1.0 - set fields from database query result
               	  $stored_rate = $results->indirect_tax_rate;
               	  $indirect_tax_method = $results->indirect_tax_calc_method;

            	    if ($this->debug_active){
                       MM_Connector_Functions::debug_log('DB Lookup - Tax found for Indirect Tax Type: '.$indirect_tax_type.' State: '.$ship_state.' Rate: '.$stored_rate.' Method: '.$indirect_tax_method,MM_Debug_Mode::INDIRECT_TAX);
                  }
                  switch ($indirect_tax_method){
                  	 case 1:
                  	     //Percent of cost
                  	     $indirect_tax_basis =  get_post_meta( $product_id, 'mm_product_last_cost', true ) ;
                         $indirect_tax_rate =  $stored_rate/100;
                  	     break;
                  	 case 2:
                  	     //Flat rate
                  	     $indirect_tax_basis =  1; 
                         $indirect_tax_rate =  $stored_rate;
                  	     break;
                  	 case 3:
                  	     //Net Weight
                  	     $indirect_tax_basis =  get_post_meta( $product_id, 'mm_product_net_weight', true );
                  	     $indirect_tax_rate =  $stored_rate;
                  	     break;
                  	 case 4:
                  	     //Tax 1 Basis
                  	     $indirect_tax_basis =  get_post_meta( $product_id, 'mm_product_basis_1', true );
                         $indirect_tax_rate =  $stored_rate;
                  	     break;
                  	 case 5:
                  	     //Tax 2 Basis
                  	     $indirect_tax_basis =  get_post_meta( $product_id, 'mm_product_basis_2', true );
                         $indirect_tax_rate =  $stored_rate;
                  	     break;
                  	 case 6:
                  	     //Tax 3 Basis
                  	     $indirect_tax_basis =  get_post_meta( $product_id, 'mm_product_basis_3', true );
                         $indirect_tax_rate =  $stored_rate;
                  	     break;
                  	 case 10:
                  	     // Percent of Price
                  	     $indirect_tax_basis =  $wholesale_price;  
                         $indirect_tax_rate =  $stored_rate/100;
                  	     break;
                    default:
                        $indirect_tax_basis = get_post_meta( $product_id, 'mm_product_last_cost', true );
                        $indirect_tax_rate =  $stored_rate/100;
                        break;
                  }
                  $indirect_tax_basis = ( $indirect_tax_basis == '') ? 0.00 : $indirect_tax_basis;
                  $indirect_tax_rate = ( $indirect_tax_rate == '') ? 0.00 : $indirect_tax_rate;
                  $indirect_tax_calc_amount = round($indirect_tax_basis * $indirect_tax_rate,2); //added rounding 5.1.0 (04.10.2025)
                  $indirect_tax_amount = $indirect_tax_exempt ? 0.00 : $indirect_tax_calc_amount;

                  //Check Max/Min - added 5.0.1
                  $min_max_method =  $results->min_max_calc_method;
                  $max_amount_to_tax =  (float) $results->max_amount_to_tax;
                  $min_amount_to_tax =  (float) $results->min_amount_to_tax;
                  $indirect_tax_amount = $this->check_min_max($product_id, $indirect_tax_amount, $min_max_method, $max_amount_to_tax, $min_amount_to_tax);
        	
                  $this->indirect_tax_amount = $indirect_tax_amount;
                  $this->indirect_tax_basis = $indirect_tax_basis;

               	  if($this->mm_allow_cache){
               	  	  $cache_data = ['amount' => $indirect_tax_calc_amount, 'basis' => $indirect_tax_basis];
               	  	  MM_Object_Cache::set($cache_key, $cache_data, [
                         'product' => $product_id,
                         'indirect_tax_type' => $indirect_tax_type,
                     ], $this->cache_duration * DAY_IN_SECONDS,'indirect_tax');
               	  }


        }


        /**
         * Adjusts the indirect tax amount based on the specified maximum or minimum amounts to tax.
         *
         * @param WC_Product $product The WooCommerce product object.
         * @param float $indirect_tax_amount The initial indirect tax amount.
         * @param int $min_max_method The method used to determine the basis for adjustment.
         * @param float $max_amount_to_tax The maximum amount to tax.
         * @param float $min_amount_to_tax The minimum amount to tax.
         * @return float The adjusted indirect tax amount.
         */
        private function check_min_max($product_id, $indirect_tax_amount, $min_max_method, $max_amount_to_tax, $min_amount_to_tax){

            $min_max_multiple = 0;
            switch ($min_max_method){
            	  case 1;
            	      $min_max_multiple =  (float) get_post_meta( $product_id, 'mm_product_basis_1', true );
            	      break;
            	  case 2:
            	      $min_max_multiple =  (float) get_post_meta( $product_id, 'mm_product_basis_2', true );
            	      break;
            	  case 3:
            	      $min_max_multiple =  (float) get_post_meta( $product_id, 'mm_product_basis_3', true );
            	      break;
            	  case 4:
            	      $min_max_multiple = 1;
            	      break;
            	  case 5:
            	      $min_max_multiple = (float) get_post_meta( $product_id, 'mm_product_items_per_unit', true );
            	      break;
            	  default:
            	      $min_max_multiple = 1;
            	      break;
            }
            if ($min_max_multiple == 0 ){
            	$min_max_multiple = 1 ;
            }
            
            //Check if tax needs adjustment.  Note: $indirect_tax_amount is the Unit (per Quantity) Tax 
            if ($max_amount_to_tax > 0 ){
            	
            	if ($max_amount_to_tax * $min_max_multiple < $indirect_tax_amount ){
            		 $indirect_tax_amount = $max_amount_to_tax * $min_max_multiple;
            	}
            	
            }
            elseif ($min_amount_to_tax > 0){

            	if ($min_amount_to_tax * $min_max_multiple > $indirect_tax_amount ){
            		 $indirect_tax_amount = $min_amount_to_tax * $min_max_multiple;
            	}
            	
            }

            return $indirect_tax_amount; 
        	  
        }



        /**         
         * Calulates the indirect tax (by Jurisdiction).
         *
         * sets the values for properties indirect_tax_amount_j1, amount_j2, basis_j1 and basis_j2
         *
         * @param int $product_id The product ID.
         * @param string $ship_state The user's state.
         * @return Void
         */
        private function calc_indirect_tax_jurisdiction($product_id, $tax_code, $wholesale_price){

             global $wpdb;

       	   
       	     $this->indirect_tax_amount_j1 = 0.00;
       	     $this->indirect_tax_basis_j1 = 0.00;
             $this->indirect_tax_amount_j2 = 0.00;
             $this->indirect_tax_basis_j2 = 0.00;

             $indirect_tax_type = get_post_meta( $product_id, 'mm_indirect_tax_type', true );
             if ($indirect_tax_type == '') {
             	 return; // no tax type for this product
             }
             
             // If customer is not exempt from all taxes, check if exempt from this type
             // Note: we store a user meta field called indirect_tax_exemptions which is a pipe delimeted array of exempt tax types
             $indirect_tax_exempt = $this->indirect_tax_exempt;
             if ($indirect_tax_exempt != 1 ){
                $indirect_tax_exempt = in_array($indirect_tax_type, $this->tax_exempt_array);
             }

             // 5.1.0 Check cache if used (only for products with a tax type)
             if ($this->mm_allow_cache){
                $cache_key = "mm_tax_jurisdiction_{$indirect_tax_type}_{$product_id}_{$tax_code}";
                $cached_tax = MM_Object_Cache::get($cache_key);
                if (is_array($cached_tax)) {
             	     if($this->debug_active){
                      MM_Connector_Functions::debug_log('Cached jurisdiction tax ['.$cache_key.'] used for Product Id: '.$product_id,MM_Debug_Mode::INDIRECT_TAX);
                   }
                   $this->indirect_tax_amount_j1 = ($indirect_tax_exempt) ? 0.00 : $cached_tax['amount_j1'];
                   $this->indirect_tax_basis_j1 = $cached_tax['basis_j1'];
                   $this->indirect_tax_amount_j2 = ($indirect_tax_exempt) ? 0.00 : $cached_tax['amount_j2'];
                   $this->indirect_tax_basis_j2 = $cached_tax['basis_j2'];
                   return;
                }
             }



             //get indirect taxes for taxcode from db
             $table_name =  $wpdb->prefix . 'mm_indirect_tax_jurisdiction';
             $query = 'SELECT * FROM '.$table_name.' WHERE indirect_tax_type='.$indirect_tax_type.' AND indirect_tax_code="'.$tax_code.'" LIMIT 1;';
             $results = $wpdb->get_row($query);

             if(empty($results)){
    	          if ($this->debug_active){
                   MM_Connector_Functions::debug_log('DB Lookup - No Jurisdiction tax found for Indirect Tax Type: '.$indirect_tax_type.' Taxcode: '.$tax_code,MM_Debug_Mode::INDIRECT_TAX);
                }
                if($this->mm_allow_cache){
                   $cache_data = [
                    'amount_j1' => $this->indirect_tax_amount_j1, 'basis_j1' => $this->indirect_tax_basis_j1,
                    'amount_j2' => $this->indirect_tax_amount_j2, 'basis_j2' => $this->indirect_tax_basis_j2
                   ];
             	     MM_Object_Cache::set($cache_key, $cache_data, [
                    'product' => $product_id,
                    'indirect_tax_type' => $indirect_tax_type,
                   ], $this->cache_duration * DAY_IN_SECONDS,'indirect_tax_jurisdiction');
                }
              	return;  //no jurisdictions in db
             }
             $indirect_tax_calc_amount = [0.00,0.00];

             for ($i = 1; $i <= 2; $i++) {
             	
                  //Set Properties from loop
                  $rate_property = "indirect_tax_rate_j" . $i;
                  $calc_method_property = "indirect_tax_calc_method_j" . $i;

                  //Set calculation variables from db query
                  $stored_rate = $results->$rate_property;
                  $indirect_tax_method = $results->$calc_method_property;
       	          if ($this->debug_active){
                     MM_Connector_Functions::debug_log('DB Lookup - Jurisdiction tax found for Indirect Tax Type: '.$indirect_tax_type.' Taxcode: '.$tax_code.' J'.$i.' Rate: '.$stored_rate.' J'.$i.' Method: '.$indirect_tax_method,MM_Debug_Mode::INDIRECT_TAX);
                  }

                  switch ($indirect_tax_method) {
                      case 1:
                          // Percent of cost
                          $indirect_tax_basis = get_post_meta($product_id, 'mm_product_last_cost', true);
                          $indirect_tax_rate = $stored_rate / 100;
                          break;
                      case 2:
                          // Flat rate
                          $indirect_tax_basis = 1;
                          $indirect_tax_rate = $stored_rate;
                          break;
                      case 3:
                          // Net Weight
                          $indirect_tax_basis = get_post_meta($product_id, 'mm_product_net_weight', true);
                          $indirect_tax_rate = $stored_rate;
                          break;
                      case 4:
                          // Tax 1 Basis
                          $indirect_tax_basis = get_post_meta($product_id, 'mm_product_basis_1', true);
                          $indirect_tax_rate = $stored_rate;
                          break;
                      case 5:
                          // Tax 2 Basis
                          $indirect_tax_basis = get_post_meta($product_id, 'mm_product_basis_2', true);
                          $indirect_tax_rate = $stored_rate;
                          break;
                      case 6:
                          // Tax 3 Basis
                          $indirect_tax_basis = get_post_meta($product_id, 'mm_product_basis_3', true);
                          $indirect_tax_rate = $stored_rate;
                          break;
                      case 10:
                          // Percent of Price
                          $indirect_tax_basis = $wholesale_price;
                          $indirect_tax_rate = $stored_rate / 100;
                          break;
                      default:
                          $indirect_tax_basis = '';
                          $indirect_tax_rate = '';
                          break;
                  }

                  $indirect_tax_basis = ($indirect_tax_basis == '') ? 0 : $indirect_tax_basis;
                  $indirect_tax_rate = ($indirect_tax_rate == '') ? 0 : $indirect_tax_rate;
                  $indirect_tax_calc_amount[$i] = round($indirect_tax_basis * $indirect_tax_rate,2); 
                  $indirect_tax_amount = $indirect_tax_exempt ? 0.00 : $indirect_tax_calc_amount[$i];

                  //Check Max/Min - added 5.0.1
                  $field_name = 'min_max_calc_method_j' . $i ;
                  $min_max_method =  $results->$field_name;
                  $field_name = 'max_amount_to_tax_j' . $i ;
                  $max_amount_to_tax =  (float) $results->$field_name;
                  $field_name = 'min_amount_to_tax_j' . $i ;
                  $min_amount_to_tax =  (float) $results->$field_name;

                  $indirect_tax_amount = $this->check_min_max($product_id, $indirect_tax_amount, $min_max_method, $max_amount_to_tax, $min_amount_to_tax);
                  $indirect_tax_calc_amount[$i] = $this->check_min_max($product_id,  $indirect_tax_calc_amount[$i], $min_max_method, $max_amount_to_tax, $min_amount_to_tax); //for cache

                  $property_name = "indirect_tax_amount_j" . $i;
                  $this->$property_name = $indirect_tax_amount;

                  $property_name = "indirect_tax_basis_j" . $i;
                  $this->$property_name = $indirect_tax_basis;
             }

             if($this->mm_allow_cache){
                $cache_data = [
                    'amount_j1' => $indirect_tax_calc_amount[1], 'basis_j1' => $this->indirect_tax_basis_j1,
                    'amount_j2' => $indirect_tax_calc_amount[2], 'basis_j2' => $this->indirect_tax_basis_j2
                ];
             	  MM_Object_Cache::set($cache_key, $cache_data, [
                    'product' => $product_id,
                    'indirect_tax_type' => $indirect_tax_type,
                ], $this->cache_duration * DAY_IN_SECONDS,'indirect_tax_jurisdiction');
             }


         	   return;

        }


        private function calc_user_price($current_price,$user_id,$product_id){
           	  
         	   global $wpdb;

             //Check if this option is allowed
             if ($this->customer_pricing_enable != 1){
       	    	 return $current_price;
             }
             
             //Array to store rule data
             $rule_data = [];
             

             // Try cache if enabled
             if ($this->mm_allow_cache) {

                 $cache_key = "mm_cust_price_{$product_id}_{$user_id}";
                 $cached = MM_Object_Cache::get($cache_key);
                 //$cached = MM_Customer_Pricing_Cache::get($product_id, $user_id);
                 if (is_array($cached)) {

                     if($this->debug_active){
                         MM_Connector_Functions::debug_log('Cache ['.$cache_key.'] used for user price Product Id: '.$product_id.
                                                           ' User Id:'.$user_id.
                                                           ' Rule Exists: '.$cached['rule_exists'].
                                                           ' Start Date: '.$cached['start_date'].
                                                           ' End Date: '.$cached['end_date'].
                                                           ' Price: '.$cached['price'],MM_Debug_Mode::CUST_PRICE);
                     }

                 	   if ($cached['rule_exists'] === false) {
                 	   	  return $current_price; //we cached that no rule exists for this customer and product
                 	   }

                 	   $today_date = date('Ymd');
                 	   $start_date = $cached['start_date'];
                     $end_date = $cached['end_date'];
           	  	     //Check if there is a date range (dates are stored as YYYYMMDD) and today is outside of range
           	  	     if ($today_date < $start_date || ($end_date != '' && $end_date < $today_date)){
              	  	    return $current_price;
           	  	     }
                 	   
                     $new_price = $cached['price'];
                     return $new_price;

                 	   
                 }

             }
             
    	
              //get user price from db, if it exists
              //2.0.0 - Use new database file for efficiency
              $table_name =  $wpdb->prefix . 'mm_customer_price';
              $query = 'SELECT * FROM '.$table_name.' WHERE product_id='.$product_id.' AND user_id='.$user_id.' LIMIT 1;';
              $results = $wpdb->get_row($query);
          
              if ($results){

              	  $start_date = $results->start_date;
              	  $end_date = $results->end_date;
              	  $today_date = date('Ymd');
              	  $out_of_range=false;
           	  	  //Check if there is a date range (dates are stored as YYYYMMDD) and today is outside of range
           	  	  if ($today_date < $start_date || ($end_date != '' && $end_date < $today_date)){
              	  	 $out_of_range=true;
           	  	  }
  
                  //Compute Price based on rule
                  $rule_data['method'] = $results->method;
                  $rule_data['amount'] = $results->amount;
                  $new_price = $this->compute_price_rule($rule_data,$current_price,$product_id);

                  if($this->debug_active){
                    MM_Connector_Functions::debug_log('DB Lookup custom price found for Product Id: '.$product_id .' User: '.$user_id.' Rule Method: ',$rule_data['method'].' Rule Amount: '.$rule_data['amount'],MM_Debug_Mode::CUST_PRICE);
                  }

                  // Store to cache if allowed
                  if ($this->mm_allow_cache) {
                  	  /*
                      MM_Customer_Pricing_Cache::set($product_id, $user_id, [
                          'rule_exists'      => true,
                          'start_date' => $start_date,
                          'end_date'   => $end_date,
                          'price'     => $new_price,
                      ]);
                      */
                      $cache_data = [
                          'rule_exists'      => true,
                          'start_date' => $start_date,
                          'end_date'   => $end_date,
                          'price'     => $new_price,
                      ];
             	        MM_Object_Cache::set($cache_key, $cache_data, [
                          'product' => $product_id,
                          'user' => $user_id,
                      ], $this->cache_duration * DAY_IN_SECONDS,'cust_price');
                      
                      
                  }
                  if ($out_of_range){
                  	//Date is out of range, use current price
                  	return $current_price;
                  }
                  return $new_price;

              }



              else {

                 if($this->debug_active){
                    MM_Connector_Functions::debug_log('DB Lookup NO custom price found for Product Id: '.$product_id .' User: '.$user_id,MM_Debug_Mode::CUST_PRICE);
                 }

              	 $new_price = $current_price;
                 // Store to cache if allowed
                 if ($this->mm_allow_cache) {
                 	   /*
                     MM_Customer_Pricing_Cache::set($product_id, $user_id, [
                         'rule_exists'      => false,
                         'start_date' => '',
                         'end_date'   => '',
                         'price'     => '',
                     ]);
                     */
                      $cache_data = [
                          'rule_exists'      => false,
                          'start_date' => '',
                          'end_date'   => '',
                          'price'     => '',
                      ];
             	        MM_Object_Cache::set($cache_key, $cache_data, [
                          'product' => $product_id,
                          'user' => $user_id,
                      ], $this->cache_duration * DAY_IN_SECONDS,'cust_price');


                 }
              }
             
              
        	    return $new_price;
        	
        }


        private function compute_price_rule($rule_data,$current_price,$product_id){



              	  //set fields from rule data
              	  $amount = $rule_data['amount'];
              	  $method = $rule_data['method'];

           	  	  $new_price = $current_price;

              	  if ($amount != ''){

                  	  $lowest_price = get_post_meta($product_id,'mm_product_lowest_price',true);
              	  	  
              	  	  // Calculate New Price based on method, amount, and product data
              	  	  switch ($method) { 	
              	  	  	case '1': //Fixed Price
              	  	  	  $new_price = $amount;
              	  	  	  $new_price = ($lowest_price != '' && $new_price < $lowest_price) ? $lowest_price : $new_price ;
              	  	  	  break;
              	  	  	case '2' : //Discount Percent  
              	  	  	  $discount = $current_price * ($amount/100);
              	  	  	  $new_price = $current_price - $discount;
              	  	  	  $new_price = ($lowest_price != '' && $new_price < $lowest_price) ? $lowest_price : $new_price ;
              	  	  	  break;
              	  	   case '3' : //Markup over Last Cost
              	  	     $last_cost = get_post_meta($product_id,'mm_product_last_cost',true);
              	  	     if ($last_cost != ''){
              	  	     	  $markup = $last_cost * ($amount/100);
              	  	     	  $new_price = $current_price + $markup;
              	  	     	  $new_price = ($lowest_price != '' && $new_price < $lowest_price) ? $lowest_price : $new_price ;
              	  	     }
              	  	     break;
              	  	   case '4' : //Markup over Average Cost
              	  	     $average_cost = get_post_meta($product_id,'mm_product_cost',true);
              	  	     if ($average_cost != ''){
              	  	     	  $markup = $average_cost * ($amount/100);
              	  	     	  $new_price = $current_price + $markup;
              	  	     	  $new_price = ($lowest_price != '' && $new_price < $lowest_price) ? $lowest_price : $new_price ;
              	  	     }
              	  	     break;
              	  	   case '5' : //Fixed amount over Last Cost
              	  	     $last_cost = get_post_meta($product_id,'mm_product_last_cost',true);
              	  	     if ($last_cost != ''){
              	  	     	  $new_price = $last_cost + $amount;
              	  	     	  $new_price = ($lowest_price != '' && $new_price < $lowest_price) ? $lowest_price : $new_price ;
                	  	  }
              	  	     break;
              	  	   case '6' : //Fixed amount over Average Cost
              	  	     $average_cost = get_post_meta($product_id,'mm_product_cost',true);
              	  	     if ($average_cost != ''){
              	  	     	  $new_price = $average_cost + $amount;
              	  	     	  $new_price = ($lowest_price != '' && $new_price < $lowest_price) ? $lowest_price : $new_price ;
                	  	  }
              	  	     break;
              	  	   case '7' : //Price Level 1
              	  	     $price_level_1 = get_post_meta($product_id,'mm_price_1_wholesale_price',true);
              	  	     if ($price_level_1 != ''){
              	  	     	  $new_price = $price_level_1;
                	  	  }
              	  	     break;
              	  	   case '8' : //Price Level 2
              	  	     $price_level_2 = get_post_meta($product_id,'mm_price_2_wholesale_price',true);
              	  	     if ($price_level_2 != ''){
              	  	     	  $new_price = $price_level_2;
                	  	  }
              	  	     break;
              	  	   case '9' : //Price Level 3
              	  	     $price_level_3 = get_post_meta($product_id,'mm_price_3_wholesale_price',true);
              	  	     if ($price_level_3 != ''){
              	  	     	  $new_price = $price_level_3;
                	  	  }
              	  	     break;
              	  	   case '10' : //Price Level 4
              	  	     $price_level_4 = get_post_meta($product_id,'mm_price_4_wholesale_price',true);
              	  	     if ($price_level_4 != ''){
              	  	     	  $new_price = $price_level_4;
                	  	  }
              	  	     break;
              	  	   case '11' : //Price Level 5
              	  	     $price_level_5 = get_post_meta($product_id,'mm_price_5_wholesale_price',true);
              	  	     if ($price_level_5 != ''){
              	  	     	  $new_price = $price_level_5;
                	  	  }
              	  	     break;
              	  	   case '12' : //Price Level 6
              	  	     $price_level_6 = get_post_meta($product_id,'mm_price_6_wholesale_price',true);
              	  	     if ($price_level_6 != ''){
              	  	     	  $new_price = $price_level_6;
                	  	  }
              	  	     break;
              	  	   case '13' : //Price Level 7
              	  	     $price_level_7 = get_post_meta($product_id,'mm_price_7_wholesale_price',true);
              	  	     if ($price_level_7 != ''){
              	  	     	  $new_price = $price_level_7;
                	  	  }
              	  	     break;
              	  	   case '14' : //Price Level 8
              	  	     $price_level_8 = get_post_meta($product_id,'mm_price_8_wholesale_price',true);
              	  	     if ($price_level_8 != ''){
              	  	     	  $new_price = $price_level_8;
                	  	  }
              	  	     break;
              	  	  default:  // Do not change price
              	  	  	 $new_price = $current_price;
              	  	     break;
              	  	  }
              	  	  
              	  }
              	  
              	  return $new_price;
        	
        }

    
    
        /**
         * Execute model.
         *
         * @since 1.0.0
         * @access public
         */
        public function run() {
        	

              $this->indirect_tax_enabled = get_option('mm_indirect_tax_enable');
              $this->show_indirect_tax_shop = get_option('mm_show_indirect_tax_shop');
              $this->show_restriction_notation_shop = get_option('mm_show_restriction_notation_shop');
              $this->indirect_tax_allow_cart_detail = get_option('mm_indirect_tax_allow_cart_detail');
              $this->indirect_tax_note_message = get_option('mm_indirect_tax_note_message','Excise Tax included in price');
              $this->customer_pricing_enable = get_option('mm_customer_pricing_enable');
              $this->debug_active = MM_Debug_Mode::is_active([
                                                              MM_Debug_Mode::WOOCOMMERCE,
                                                              MM_Debug_Mode::INDIRECT_TAX,
                                                              MM_Debug_Mode::CUST_PRICE,
                                                              ]);
              $this->mm_allow_cache = get_option('mm_allow_cache');
              $this->woo_extender = MM_WOO_EXTENDER::instance();

              add_action('plugins_loaded',array($this,'mm_setup_user_properties'));

              add_filter('wwp_filter_wholesale_price_shop_v2', array($this,'override_wholesale_price_shop'), 100, 3);

              add_filter('wwp_filter_wholesale_price_cart', array($this,'override_wholesale_price_cart'), 100, 5);

              //add_filter( 'wwp_pass_wholesale_price_through_taxing', array( $this, 'override_wholesale_sale_price' ), 10, 3 );
              
              add_filter('wwp_before_wholesale_price_html_filter',array($this,'override_wholesale_html_price'),100,8);
              
              if ($this->indirect_tax_enabled && $this->show_indirect_tax_shop){
              	add_filter('wwpp_filter_product_raw_wholesale_sale_price', array($this,'override_wholesale_sale_price_shop'), 100, 3);
              }

              if (($this->indirect_tax_enabled && $this->show_indirect_tax_shop && $this->indirect_tax_allow_cart_detail) || $this->show_restriction_notation_shop) {
              	 add_action( 'woocommerce_product_meta_end', array($this,'maybe_add_notations_shop'),100 );
              	 if ($this->show_restriction_notation_shop){
                	 add_filter('pvtfw_table_availability_html', array($this, 'maybe_add_restriction_message_to_stock_desc' ), 10, 2);
              	 }
              }
        }

    }
}