<?php
/**
 * ManageMore Debug Mode Constants
 *
 * This file defines constants used for selecting debug modes in the ManageMore Connector.
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}

final class MM_Debug_Mode {
    public const NONE             = 0;
    public const PAYMENT_GATEWAY = 1;
    public const REST_API        = 2;
    public const WOOCOMMERCE     = 3;
    public const ULTIMATE_MEMBER = 4;
    public const INDIRECT_TAX    = 5;
    public const CUST_PRICE      = 6;
    public const REGION_RESTRICT = 7;
    public const LICENSE         = 8;
    public const CACHE           = 9;
    public const ALL             = 99;

    /**
     * Get a human-readable label for a debug mode.
     *
     * @param int $mode
     * @return string
     */
    public static function get_label( int $mode ): string {
        return match ( $mode ) {
            self::NONE             => 'None',
            self::PAYMENT_GATEWAY => 'Payment Gateway',
            self::REST_API        => 'REST API',
            self::WOOCOMMERCE     => 'WooCommerce',
            self::ULTIMATE_MEMBER => 'Ultimate Member',
            self::INDIRECT_TAX    => 'Indirect Taxes',
            self::CUST_PRICE      => 'Customer Prices',
            self::REGION_RESTRICT => 'Region Restrictions',
            self::LICENSE         => 'License Restrictions',
            self::CACHE           => 'Cache',
            self::ALL             => 'All',
            default               => 'Unknown',
        };
    }

    /**
     * Get an array of all debug mode options as [value => label].
     *
     * @return array<int, string>
     */
    public static function get_options(): array {
        return [
            self::NONE             => 'None',
            self::PAYMENT_GATEWAY => 'Payment Gateway',
            self::REST_API        => 'REST API',
            self::WOOCOMMERCE     => 'WooCommerce',
            self::ULTIMATE_MEMBER => 'Ultimate Member',
            self::INDIRECT_TAX    => 'Indirect Taxes',
            self::CUST_PRICE      => 'Customer Prices',
            self::REGION_RESTRICT => 'Region Restrictions',
            self::LICENSE         => 'License Restrictions',
            self::CACHE           => 'Cache',
            self::ALL             => 'All',
        ];
    }

    public static function is_active(int|array $modes): bool {
        $current = (int) get_option('mm_debug_mode', self::NONE);

        // If ALL mode is active, return true immediately
        if ($current === self::ALL) {
            return true;
        }

        // Normalize to array
        $modes = is_array($modes) ? $modes : [$modes];

        // Check if any of the specified modes are active
        foreach ($modes as $mode) {
            if ($current === $mode) {
                return true;
            }
        }

        return false;
    }

}
