<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * ManageMore Payment Gateway
 *
 * Provides a ManageMore Payment Gateway
 *
 * @class 		WC_Gateway_ManageMore_RevitA
 * @extends		WC_Payment_Gateway_CC
 * @version		2.01
 * @package		WooCommerce/Classes/Payment
 * @author 		Intellisoft Solutions, Inc.
 */
class WC_Gateway_ManageMore_RevitA extends WC_Payment_Gateway_CC {
	
	private $token_id;
	private $token_value; 
	private $cardexpyear;
	private $cardexpmonth;
	private $card_on_file_text = 'Remember this Card for Future Use';


    /**
     * Constructor for the gateway.
     */
	public function __construct() {
		$this->id                 = 'managemore_revita'; //RevitPay A
		$this->icon               = null;
		$this->has_fields         = true;
		// Supports the default credit card form, refunds and tokenization
		$this->supports = array( 'default_credit_card_form',
		                         'refunds',
		                         'tokenization' );
		$this->method_title       = __( 'ManageMore RevitPay A', 'woocommerce' );
		$this->method_description = __( 'Allows payments via ManageMore\'s IntelliCharge Payment Gateway for RevitPay A<br><br>
		                                 <a href="https://www.managemore.com/intellicharge" target="_blank">Sign-Up For a FREE Merchant Account</a>', 'woocommerce' );

		// Load the settings.
		$this->init_form_fields();
		$this->init_settings();

    // Turn these settings into variables we can use
		foreach ( $this->settings as $setting_key => $value ) {
			$this->$setting_key = $value;
		}

		// Define user set variables
		$this->title        = $this->get_option( 'title' );
		$this->description  = $this->get_option( 'description' );
		$this->instructions = $this->get_option( 'instructions', $this->description );


  	//Save Settings
		add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
    //Add Filter to Change Text for Card on File
    add_filter('woocommerce_payment_gateway_save_new_payment_method_option_html', array($this, 'customize_save_payment_method_checkbox_text'), 10, 2);

		
    
  }

    /**
     * Initialise Gateway Settings Form Fields
     */
    public function init_form_fields() {
      
    	$this->form_fields = array(
			'enabled' => array(
				'title'   => __( 'Enable/Disable', 'woocommerce' ),
				'type'    => 'checkbox',
				'label'   => __( 'Enable ManageMore RevitPay A', 'woocommerce' ),
				'default' => 'no'
			),
			'title' => array(
				'title'       => __( 'Title', 'woocommerce' ),
				'type'        => 'text',
				'description' => __( 'This controls the title which the user sees during checkout.', 'woocommerce' ),
				'default'     => __( 'Credit Card Payment', 'woocommerce' ),
				'desc_tip'    => true,
			),
			'loginid' => array(
				'title'       => __( 'Login Id', 'woocommerce' ),
				'type'        => 'text',
				'description' => __( 'The Merchant Login Id assigned by RevitPay', 'woocommerce' ),
				'default'     => __( '', 'woocommerce' ),
				'desc_tip'    => true,
			),
			'loginpassword' => array(
				'title'       => __( 'Login Password', 'woocommerce' ),
				'type'        => 'text',
				'description' => __( 'The Merchant Login Password assigned by RevitPay', 'woocommerce' ),
				'default'     => __( '', 'woocommerce' ),
				'desc_tip'    => true,
			),
			'authtype' => array(
				'title'       => __( 'Authorization Type', 'woocommerce' ),
				'type'        => 'select',
				'description' => __( 'Type of authorization to perform', 'woocommerce' ),
				'default'     => 'Capture',
				'options' => array(
                              'preauth' => __( 'PreAuth', 'woocommerce' ),
                              'capture' => __( 'Capture', 'woocommerce' )
                               ),
				'desc_tip'    => true,
			),
      'surcharge_enabled' => array(
          'title'   => __('Enable Surcharge', 'woocommerce'),
          'type'    => 'checkbox',
          'label'   => __('Enable a surcharge for using this payment method', 'woocommerce'),
          'default' => 'no',
      ),
      'surcharge_percent' => array(
          'title'       => __('Surcharge Percentage', 'woocommerce'),
          'type'        => 'number',
          'description' => __('Enter the percentage charge without the % sign (e.g., for 2%, enter 2).', 'woocommerce'),
          'default'     => '',
          'desc_tip'    => true,
          'custom_attributes' => array(
              'min'  => '0',
              'step' => '1.00',
          ),
      ),
      'surcharge_name' => array(
          'title'       => __('Surcharge Name', 'woocommerce'),
          'type'        => 'text',
          'description' => __('This will be displayed to the customer at checkout.', 'woocommerce'),
          'default'     => __('Credit Card Charge', 'woocommerce'),
          'desc_tip'    => true,
      ),
			'cardonfile' => array(
				'title'   => __( 'Allow Card on File', 'woocommerce' ),
				'type'    => 'checkbox',
				'label'   => __( 'Allow customer to save token of credit card on file for future checkout', 'woocommerce' ),
				'default' => 'no'
			),
			'cardonfile_text' => array(
				'title'   => __( 'Save to Account Checkbox Text', 'woocommerce' ),
        'type'        => 'text',
        'description' => __('The text to be displayed when saving Card on File.', 'woocommerce'),
        'default'     => $this->card_on_file_text,
        'desc_tip'    => true,
			),
			'sandboxmode' => array(
				'title'   => __( 'Sandbox Mode', 'woocommerce' ),
				'type'    => 'checkbox',
				'label'   => __( 'Use sandbox mode for testing (card will not be charged)', 'woocommerce' ),
				'default' => 'no'
			),
		);
    }


 
   /**
   * Override field name function
   */
   public function field_name( $name ) {

        return ' name="' . esc_attr( $this->id . '-' . $name ) . '" ' ;
     
   }
   
  /**
  * Display payment form.
  */
  function payment_fields(){

     if (is_ssl() == false ) {
     	 echo "We're sorry, payments cannot be accepted at this time.  A secure connection (HTTPS) is required.";
     	 return;
     }

     $user_id = get_current_user_id();
     $mm_cust_id =  get_user_meta($user_id,'mm_field_CID',true);
     
     if ( is_checkout() ) {
        if ($this->cardonfile=='no'){
        	 //card on file not enabled.  show cc form without tokenization options
           $this->form();
        }
        else {

        	 //card on file allowed.  show cc form with tokenization options
	         $this->tokenization_script();
           $this->saved_payment_methods();
           $this->form();
           //If no Customer Id, do not allow card on file
           if ($mm_cust_id != ''){
              //Ensure checkbox is not opaque (this was necessary for Astra theme)
              echo '<style>
                   .woocommerce-SavedPaymentMethods-saveNew, 
                   #wc-managemore_revita-new-payment-method, 
                   label[for="wc-managemore_revita-new-payment-method"] {
                   opacity: 1 !important;
                   }</style>';
              $this->save_payment_method_checkbox();
           }
           echo '<br>';
           echo '<br>';
        }
     }
     else {
 
        if ($this->cardonfile=='no'){
        	 echo "We're sorry, but you cannot save cards on file at this time.";
        }
        elseif ($mm_cust_id == ''){
        	 echo "You must have an active account to save card on file.";
        }
        else {
        	 //card on file allowed, in add payment method area.  show cc form only
	         $this->tokenization_script();
           $this->form();
      	}
     }
     
  	 return;
  	
  }

  // Validate fields
  public function validate_fields() {

   
    //if token used, no need for futher validation
    if ( isset( $_POST['wc-'.$this->id.'-payment-token'] ) && 'new' !== $_POST['wc-'.$this->id.'-payment-token'] )
      return true;  

  	$card_number = $_POST[$this->id.'-card-number'];
 
  	if ( $card_number === '' ){
  		 $error = "Credit Card Number is required.";
       wc_add_notice( $error , "error" );
       return false;
          
  	}
  	
  	if ( $this->mod_10_check($card_number) == false){
  		 $error = "Credit Card Number is invalid.";
       wc_add_notice( $error , "error" );
       return false;
 		
  	}
  	
  	$card_expiry = $_POST[$this->id.'-card-expiry'];
  	$len = strlen($card_expiry);
   	if ($len != 7){
  		 //should be "mm / yy" (with spaces and slash)
  		 $error = "Expiration Date is invalid.";
       wc_add_notice( $error , "error" );
       return false;
  	}
  	
  	$cvc = $_POST[$this->id.'-card-cvc'];
 
  	if ( $cvc === '' ){
  		 $error = "Card Security Code is required.";
       wc_add_notice( $error , "error" );
       return false;
  	}
  	

   	return true;
  }

  //mod 10 check to validate card
  function mod_10_check($number) {
  	return true;

    // Strip any non-digits (useful for credit card numbers with spaces and hyphens)
    $number=preg_replace('/\D/', '', $number);

    // Set the string length and parity
    $number_length=strlen($number);
    $parity=$number_length % 2;

    // Loop through each digit and do the maths
    $total=0;
    for ($i=0; $i<$number_length; $i++) {
      $digit=$number[$i];
      // Multiply alternate digits by two
      if ($i % 2 == $parity) {
        $digit*=2;
        // If the sum is two digits, add them together (in effect)
        if ($digit > 9) {
          $digit-=9;
        }
      }
      // Total up the digits
      $total+=$digit;
    }

    // If the total mod 10 equals 0, the number is valid
    return ($total % 10 == 0) ? TRUE : FALSE;

  }

  public function add_payment_method() {


   
            if (is_ssl() == false ) {
               wc_add_notice( "We're sorry, payment methods cannot be added at this time.  A secure connection (HTTPS) is required." , "error" );
               return false;
            }


            //Board the Card in Vault, set token or error message
            $submit_type = 'VaultCard';
            $this->card_num = str_replace(array( '-', ' '), '',$_POST[$this->id.'-card-number']); //strip spaces and dashes
            $ccexp = str_replace( array( '/', ' '), '', $_POST[$this->id.'-card-expiry'] ); //mm / yy
     	   	  $this->card_expmonth = substr($ccexp,0,2);
	          $this->card_expyear = '20'.substr($ccexp,2,2); 
            $this->card_expdate =  $this->card_expyear . '-' . $this->card_expmonth; //yyyy-mm
            unset($vault_response);
            //note, for VaultCard, we create our own array of resultCode, customerProfileId, customerPaymentProfileId, and errorMessage to return.
            $vault_response = $this->_sendCCRequest($submit_type,$order);
            $vault_token = '';
            if($this->commError!=''){
    	    		 $error_message = 'Communication Error: '.$this->commError;
            }
            elseif ($vault_response['resultCode'] == 'Ok'){
               $vault_token = $vault_response['customerProfileId'] . '|' . $vault_response['customerPaymentProfileId'];
               $error_message = '';
            }
            else{
    	    		 $error_message = $vault_response['errorMessage'];
            }
            
            
            if ($vault_token != '') {

               $token = new WC_Payment_Token_CC();
               $token_string = $vault_token;

               $cardnum = $this->card_num;
               $firstdigit = substr($cardnum, 0, 1);
               switch ($firstdigit){
                   case "3":
                      $this->card_type = 'amex';
                      break;
                   case "4":
                      $this->card_type = 'visa';
                      break;
                   case "5":
                      $this->card_type = 'mastercard';
                      break;
                   case "6":
                      $this->card_type = 'discover';
                      break;
                   default:
			                $response_msg_to_customer = 'Card Type Error';
                      wc_add_notice( __('Token error: ', 'woothemes') . $response_msg_to_customer, 'error' );
                      MM_Connector_Functions::debug_log('Token error: '. $response_msg_to_customer,1); //log payment gateway debug
    	    		        return false;
                      
               }

               $token->set_token( $token_string );
               $token->set_gateway_id( $this->id ); // `$this->id` references the gateway ID set in `__construct`
               $token->set_card_type( $this->card_type );
               $token->set_last4( substr( $cardnum ,-4) );
               $token->set_expiry_month( $this->card_expmonth );
               $token->set_expiry_year( $this->card_expyear );
               $token->set_user_id( get_current_user_id() );
               $token->save();

               //Give message that card was saved
               wc_add_notice( __('Payment Token Stored.', 'woothemes') , 'success' );
               MM_Connector_Functions::debug_log('Card Token Stored.',1); //log payment gateway debug
    	    	}
    	    	else {
               //Give message about token not being stored
               wc_add_notice( __('Token error: ', 'woothemes') . $error_message, 'error' );
               MM_Connector_Functions::debug_log('Token error: '. $error_message,1); //log payment gateway debug
     	    	   return false;

            }


  	
  	
  	return true;
  }



  /**
  * Process the payment and return the result
  *
  * @param int $payment
  * @return array
  */
	public function process_payment( $order_id ) {
      global $woocommerce;


      
      if (is_ssl() == false ) {
          wc_add_notice( "We're sorry, payments cannot be accepted at this time.  A secure connection (HTTPS) is required." , "error" );
          return;
      }


      //Get Order
 		  $order = new WC_Order( $order_id );
      $token_used = false;
		
      //Set Processing Information (Card Number, Exp Date, etc.)
      //Check if Token (Card on File) to be used
      if ( isset( $_POST['wc-'.$this->id.'-payment-token'] ) && 'new' !== $_POST['wc-'.$this->id.'-payment-token'] ) {

         $token_id = wc_clean( $_POST['wc-'.$this->id.'-payment-token'] );
         $token    = WC_Payment_Tokens::get( $token_id );

         // Token user ID does not match the current user... bail out of payment processing.
         if ( $token->get_user_id() !== get_current_user_id() ) {
         	   $response_msg_to_customer = 'The card on file selected is invalid for your user id.';
             // Add notice to the cart
             wc_add_notice( __('Payment error: ', 'woothemes') . $response_msg_to_customer, 'error' );
             return;
         }
         $token_used = true;
         $this->token_id = $token_id;
         $this->token_value = $token->get_token();
         $this->card_num = $token->get_last4();
         $this->card_type= $token->get_card_type();
         switch ($this->card_type){
              case "amex":
                 $this->card_desc = "American Express";
                break;
              case "visa":
                 $this->card_desc = "Visa";
                 break;
              case "mastercard":
                 $this->card_desc = "MasterCard";
                 break;
              case "discover":
                 $this->card_desc = "Discover";
                 break;
              default:
                 $this->card_desc = "Credit Card";
                 break;
         }

	    	 $this->card_expmonth = $token->get_expiry_month(); //mm
	       $this->card_expyear = $token->get_expiry_year();   //yyyy
         $this->card_expdate =  $token->get_expiry_year() .'-' .$token->get_expiry_month(); //yyyy-mm
         $submit_type = ($this->authtype=='preauth') ? 'PreAuthVault' : 'SaleVault' ; 


      } 
      else {

      	 //card info entered
         $this->token_id = null;
         $this->token_value = '';
         $this->card_num = str_replace(array( '-', ' '), '',$_POST[$this->id.'-card-number']); //strip spaces and dashes
         $firstdigit = substr($this->card_num, 0, 1);
         switch ($firstdigit){
             case "3":
                $this->card_desc = "American Express";
                $this->card_type = 'amex';
                break;
             case "4":
                $this->card_desc = "Visa";
                $this->card_type = 'visa';
                break;
             case "5":
                $this->card_desc = "MasterCard";
                $this->card_type = 'mastercard';
                break;
             case "6":
                $this->card_desc = "Discover";
                $this->card_type = 'discover';
                break;
             default:
                $this->card_desc = "Credit Card";
                break;
         }
         $ccexp = str_replace( array( '/', ' '), '', $_POST[$this->id.'-card-expiry'] ); //mm / yy
     		 $this->card_expmonth = substr($ccexp,0,2);
	       $this->card_expyear = '20'.substr($ccexp,2,2); 
         $this->card_expdate =  $this->card_expyear . '-' . $this->card_expmonth; //yyyy-mm
         $submit_type = ($this->authtype=='preauth') ? 'PreAuth' : 'Sale' ; 

      }


      //Call Gateway
      unset($response);
     
      $response = $this->_sendCCRequest($submit_type,$order); //send request to gateway and get response
      
      //Check Response
      $transactionResponse = $response['transactionResponse'] ?? null;
      $messages = $transactionResponse['messages'] ?? null;
      $errors = $transactionResponse['errors'] ?? null;
      $response_code = $transactionResponse['responseCode'] ?? null;
      
      
      // If the response code is not 1 (approved) then redirect back to the payment page with the appropriate error message
      // If 1, finish order
      if ($response_code == '1') {	

          //Payment Approved
          $this->gateway_refid =$transactionResponse['transId'] ?? '';
          $this->auth_code = $transactionResponse['authCode'] ?? '';
          
          //Set PreAuth -------------------------------------------------------------------
          if ($this->authtype=='preauth') {
              $this->preauth='1';
          }
          else {
              $this->preauth='0';
          }

         
			    // Add note to order
			    $order->add_order_note( 'Credit Card Payment completed. Gateway Ref Id=' . $this->gateway_refid);
          MM_Connector_Functions::debug_log('Credit Card Payment completed. Gateway Ref Id=' . $this->gateway_refid,1); //log payment gateway debug
												 

          //All OK, approved

         
          //Update Post Meta with Gateway Ref Id (for refunds), and other payment info (for REST-based importing into MM)


          //CC Info (was traditionally stored with _econnect prefix - we use these values when downloading the order for payment reference in ManageMore)
          $order->update_meta_data( '_econnect_card_desc', $this->card_desc );
          $order->update_meta_data( '_econnect_card_num', substr($this->card_num,-4) );
          $order->update_meta_data( '_econnect_auth_code', substr($this->auth_code,10)  );  //note: substring fixes rare cases (esp with test account) that auth_code is null
          $order->update_meta_data( '_econnect_auth_type', $this->preauth );
          $order->update_meta_data( '_econnect_gateway_refid', $this->gateway_refid );
          $order->update_meta_data( '_econnect_card_expdate',  $this->card_expmonth . '/' . substr($this->card_expyear,2,2) ); //mm/yy 
          if($token_used==true){
             $order->update_meta_data( '_econnect_token', $this->token_value); //New 4.7.0
          }

         // Save Token, if necessary
         $save_token = ( isset( $_POST['wc-'.$this->id.'-new-payment-method'] ) && 'true' == $_POST['wc-'.$this->id.'-new-payment-method'] ) ? 'yes' : 'no' ;
         if ($save_token=="yes") {

            //Board the Card in Vault, set token or error message
            $submit_type = 'VaultCard';
            unset($vault_response);
            //note, for VaultCard, we create our own array of resultCode, customerProfileId, customerPaymentProfileId, and errorMessage to return.
            $vault_response = $this->_sendCCRequest($submit_type,$order);
            $vault_token = '';
            if($this->commError!=''){
    	    		 $error_message = 'Communication Error: '.$this->commError;
            }
            elseif ($vault_response['resultCode'] == 'Ok'){
               $vault_token = $vault_response['customerProfileId'] . '|' . $vault_response['customerPaymentProfileId'];
               $error_message = '';
            }
            else{
    	    		 $error_message = $vault_response['errorMessage'];
            }
            
            
            if ($vault_token != '') {

               $token = new WC_Payment_Token_CC();
               $token_string = $vault_token;
               $token->set_token( $token_string );
               $token->set_gateway_id( $this->id ); // `$this->id` references the gateway ID set in `__construct`
               $token->set_card_type( $this->card_type );
               $token->set_last4( substr(str_replace( array(' ', '-' ), '', $_POST[$this->id.'-card-number'] ),-4) );
               $token->set_expiry_month( $this->card_expmonth );
               $token->set_expiry_year( $this->card_expyear );
               $token->set_user_id( get_current_user_id() );
               $token->save();

               //Save note to order about token being stored
               $tokeninfo = 'Payment Token stored.';
    	    	   $order->add_order_note( $tokeninfo );
    	    	   //Save token for ManageMore
               $order->update_meta_data( '_econnect_token', $token_string); //New 4.7.0
               MM_Connector_Functions::debug_log('Card Token Stored',1); //log payment gateway debug
    	    	}
    	    	else {
               //Save note to order about token not being stored
               $tokeninfo = 'Token requested but could NOT be stored.  Error: '. $error_message;
    	    	   $order->add_order_note( $tokeninfo ); 
    	    	   MM_Connector_Functions::debug_log($tokeninfo,1); //log payment gateway debug
            }
            	       		


         } //if saving token

   			 // Mark order as Paid
   			 $order->payment_complete();
   
   			 // Empty the cart (Very important step)
   			 $woocommerce->cart->empty_cart();
   

   			 // Redirect to thank you page
		   	 return array(
	   			'result'   => 'success',
			   	'redirect' => $this->get_return_url( $order ),
			    );
			    
			    
		} else {
		  	
			   // Transaction was not approved
	       if ($this->commError == '' ){ 
	                  	
	          $resultCode = $messages['resultCode'] ?? null;
	          $resultMessage = $messages['message'] ?? null;
	          $resultMessageDescription = $resultMessage[0]['text'] . ' (' . $resultMessage[0]['code'] . ')';
	          
	          //If "normal" error (e.g. Declined)
	          if ($resultCode == "Ok"){
          	 	 $errorMessage = $errors[0]['errorText'];
	          	 switch ($response_code){
	          	 	 case "2":
                     $response_msg_to_customer = "The transaction has been declined.";
                     break;
                 case "3":
                     $response_msg_to_customer = "There was an error processing the transaction.";
                     break;
                 case "4":
                     $response_msg_to_customer = "The transaction is being held for review.";
                     break;
                 default:
                     $response_msg_to_customer = "There was an error processing the transaction.";
                     break;
               };
               $order->update_status('failed', 'Payment Error: '. $errorMessage);
               MM_Connector_Functions::debug_log('Payment Error: ' . $errorMessage,1); //log payment gateway debug
             } 
            else{
            	 //Other error, will be in resultMessage field
            	 $response_msg_to_customer = "We are sorry, there was an error processing your card.  A note for the administrator has been added to the Order.";
               $order->update_status('failed', 'Gateway Error: '. $resultMessageDescription);
               MM_Connector_Functions::debug_log('Gateway Error: ' . $resultMessageDescription,1); //log payment gateway debug
            }
          }
          else {
            //Mark Order as Failed if gateway communication error (so admin will know)
            $response_msg_to_customer = 'There was a Credit Card Communication Error.  A note for the administrator has been added to the Order.' ;
            $error_text = $this->commError;
            $order->update_status('failed', 'Gateway Communication Error: '. $error_text );
            MM_Connector_Functions::debug_log('Gateway Communication Error: ' . $error_text,1); //log payment gateway debug
         }   

			   // Add notice to the cart
         wc_add_notice( __('Payment error: ', 'woothemes') . $response_msg_to_customer, 'error' );


         return;

      }	
	}


  /**
   * Process Refund from Admin section View Order page
   */
  public function process_refund( $order_id, $amount = null, $reason='' ) {

     //Get Order
	   $order = new WC_Order( $order_id );

     //Get Post Meta with Gateway Ref Id
     $this->gateway_refid = $order->get_meta('_econnect_gateway_refid', true);
     $this->refund_amount = $amount;
     $this->card_num = $order->get_meta('_econnect_card_num', true);
     $this->card_expdate = 'XXXX';

     
     //Check if order was paid today - if so, use a Void, else use a refund
     $datePaid = $order->get_date_paid();
     if ($datePaid){
        // Convert the order's paid date to the shop's timezone
        $paidDate = $datePaid->setTimezone(new DateTimeZone(wc_timezone_string()))->format('Y-m-d');
        // Get today's date in the shop's timezone
        $today = new DateTime('now', new DateTimeZone(wc_timezone_string()));
        $todayDate = $today->format('Y-m-d');
        if ($paidDate === $todayDate) {
          $submit_type = "Void";
        }
        else{
          $submit_type = "Refund";
        }
             	
     }
     else {
     	  //Order was not paid
     	  $errorMessage = 'Order was not paid.';
     	  $order->add_order_note( 'Refund could not be processed: ' . $errorMessage);
        MM_Connector_Functions::debug_log('Refund could not be processed: ' . $errorMessage,1); //log payment gateway debug
     	  return false;
     }
          
     $response = $this->_sendCCRequest($submit_type,$order);
     $transactionResponse = $response['transactionResponse'] ?? null;
     $messages = $transactionResponse['messages'] ?? null;
     $errors = $transactionResponse['errors'] ?? null;
     $response_code = $transactionResponse['responseCode'] ?? null;
     if($response_code != "1"){
     	  $errorMessage = $errors[0]['errorText'];
     	  if( $this->commError ==''){
        	  $order->add_order_note( 'Refund could not be processed: ' . $errorMessage);
            MM_Connector_Functions::debug_log('Refund could not be processed: ' . $errorMessage,1); //log payment gateway debug
     	  }
     	  else {
        	  $order->add_order_note( 'Refund could not be processed, due to communication error: ' .$this->commError);
            MM_Connector_Functions::debug_log('Refund could not be processed. Comm Error: ' . $this->commError,1); //log payment gateway debug
     	  }
     }
     $success = ($response_code == '1') ? true : false;
     
     return $success;
  }

    	
  /**
   * Send Credit Card communication request
   */
  public function _sendCCRequest($submit_type,$order) {


    $response = '';
    
    $authentication_array = array(
        "merchantAuthentication" => array (
             "name" => $this->loginid,
             "transactionKey" => $this->loginpassword
           )
        );
    
    MM_Connector_Functions::debug_log('Card Request Initiated. Request Type: '.$submit_type,1); //log payment gateway debug

    if($submit_type == 'Void'){

        //Build array to send for voiding
        $transaction_array = array(
                        'transactionRequest' => array(
                           'transactionType'=> 'voidTransaction',
                           'refTransId'=>  $this->gateway_refid
                        )
                     );

        $submit_data = json_encode(
                       array(
                         'createTransactionRequest' => array_merge($authentication_array,$transaction_array)
                       )
                    );

       return $this->_gateway_request($submit_data);

   
    }
    elseif($submit_type == 'Refund'){
        //Build array to send for refunding
        $transaction_array = array(
                        'refId'=> str_replace( "#", "", $order->get_order_number() ),
                        'transactionRequest' => array(
                           'transactionType'=> 'refundTransaction',
                           'amount'=> $this->refund_amount,
                           'payment'=> array(
                               'creditCard'=>array(
                                  'cardNumber'=> $this->card_num,
                                  'expirationDate'=> $this->card_expdate
                               )
                           ),
                           'refTransId'=>  $this->gateway_refid
                        )
                     );

        $submit_data = json_encode(
                       array(
                         'createTransactionRequest' => array_merge($authentication_array,$transaction_array)
                       )
                    );

       return $this->_gateway_request($submit_data);
    
    }
    elseif($submit_type == 'VaultCard'){

    	  $current_user_id = get_current_user_id() ;
    	  $fname = get_user_meta( $current_user_id, 'billing_first_name', true );
        $lname = get_user_meta( $current_user_id, 'billing_first_name', true );
        /* for now, we do not update the address fields for vaulted cards (managemore doesn't either)
        $address_1 = get_user_meta( $current_user_id, 'billing_address_1', true ); 
    	  //Prevent problems with postal code.  For US AVS only, must be no more than 5 digits, numeric.
        if (get_user_meta( $current_user_id, 'billing_country', true ) != 'US'){
        	$post_code = '';
        }
        else {
          $post_code = get_user_meta( $current_user_id, 'billing_postcode', true );
    	    $post_code = substr($post_code,5);
    	    $post_code = preg_replace("/[^0-9]/", "", $post_code);
    	  }
    	  */
    	  $company_name = get_user_meta( $current_user_id, 'billing_company', true );
    	  

        //Build array to send for getting customer profile id
        //For the Merchant Customer Id, we use ManageMore Account Number
        //This value must be unique to a particular merchant account
        $user_id = get_current_user_id();
        $mm_cust_id =  get_user_meta($user_id,'mm_field_CID',true);
        if ($mm_cust_id == ''){
        	
        }
        $merchantCustomerId = $mm_cust_id;
        unset($response);
        $vault_data = array(
                       'merchantCustomerId' => $merchantCustomerId,
                       'includeIssuerInfo' => 'true'
                       );
        $submit_data = json_encode(
                          array(
                            'getCustomerProfileRequest' => array_merge($authentication_array,$vault_data)
                          )
                       );
                    
        $response = $this->_gateway_request($submit_data);
        if($this->commError !=''){
         	 return;
        }
        $resultCode = $response['messages']['resultCode'];
        $errorCode = $response['messages']['message'][0]['code'];
        
        if ($resultCode!='Ok' && $errorCode!= 'E00040'){
        	  //error and not "The record could not be found" error
            $vault_response = array(
                              'resultCode'=>$response['messages']['resultCode'],
                              'customerProfileId'=>0,
                              'customerPaymentProfileId'=>0,
                              'errorMessage'=>'Getting Customer Profile - '.$response['messages']['message'][0]['text']
                              );
            return $vault_response;
        	 
        }
        $customerProfileId = $response['profile']['customerProfileId'] ?? null;
        $lastFourDigitsOfCurrentCard = substr($this->card_num, -4);
 
        if ($customerProfileId){
        	 //The Customer is found in the Vault
        	
           //Check if there is a payment profile for this card
           // Extract the last four digits of the current card number
           $customerPaymentProfileId = 0;  //a non-zero value indicates a match

           // Check if response exist
           if (isset($response['profile']['paymentProfiles'])) {
              foreach ($response['profile']['paymentProfiles'] as $payment_profile) {
                  // Extract last four digits of each card number in the profiles
                  if (isset($payment_profile['payment']['creditCard']['cardNumber'])) {

                     $cardNumber = $payment_profile['payment']['creditCard']['cardNumber'];
                     $lastFourDigits = substr($cardNumber, -4);
                     // Compare with the current card's last four digits
                     if ($lastFourDigits === $lastFourDigitsOfCurrentCard) {
                         $customerPaymentProfileId = $payment_profile['customerPaymentProfileId'];
                         break; // Stop the loop if a match is found
                     }
                  }
               }
           }


           if($customerPaymentProfileId == 0){
           	  //Create Payment Profile Only
           	  $vault_data = array(
           	          'customerProfileId' => strval($customerProfileId),
           	          'paymentProfile' => array(
    	                     'billTo' => array(
    	                        'firstName' => $fname,
    	                        'lastName' => $lname,
    	                        'company' => $company_name
    	                       ),
           	                'payment' => array(
           	                     'creditCard' => array(
           	                         'cardNumber'=>$this->card_num,
               	                     'expirationDate' => $this->card_expdate
               	                     )
           	                )
           	          ),
           	          'validationMode' => 'testMode'  //Just store the card, do not validate
           	        );
              $submit_data = json_encode(
                                array(
                                  'createCustomerPaymentProfileRequest' => array_merge($authentication_array,$vault_data)
                                )
                             );

              //Send Vault Request, and return custom array from response
              //Get customerProfileId from the REQUEST and customerPaymentProfileId from the response
              $response = $this->_gateway_request($submit_data);
              if($this->commError !=''){
              	 return;
              }
              $vault_response = array(
                                 'resultCode'=>$response['messages']['resultCode'],
                                 'customerProfileId'=>$customerProfileId,
                                 'customerPaymentProfileId'=>$response['customerPaymentProfileId'],
                                 'errorMessage'=>'Adding Payment Profile - '.$response['messages']['message'][0]['text']
                              );
              return $vault_response;

           }
           else {
           	  //Update Payment Profile
           	  $vault_data = array(
           	          'customerProfileId' => strval($customerProfileId),
           	          'paymentProfile' => array(
    	                     'billTo' => array(
    	                        'firstName' => $fname,
    	                        'lastName' => $lname,
    	                        'company' => $company_name
    	                       ),
           	                'payment' => array(
           	                     'creditCard' => array(
           	                         'cardNumber'=>$this->card_num,
               	                     'expirationDate' => $this->card_expdate
               	                     )
           	                ),
           	                'customerPaymentProfileId' => strval($customerPaymentProfileId)
           	          ),
           	          'validationMode' => 'testMode'  //Just store the card, do not validate
           	        );

              //Get both ids, from the REQUEST, since we already know them
              $submit_data = json_encode(
                                array(
                                  'updateCustomerPaymentProfileRequest' => array_merge($authentication_array,$vault_data)
                                )
                             );

              //Send Vault Request, and return custom array from response
              $response = $this->_gateway_request($submit_data);
              if($this->commError !=''){
              	 return;
              }
              $vault_response = array(
                                 'resultCode'=>$response['messages']['resultCode'],
                                 'customerProfileId'=>$customerProfileId,
                                 'customerPaymentProfileId'=>$customerPaymentProfileId,
                                 'errorMessage'=>'Updating Payment Profile - '.$response['messages']['message'][0]['text']
                              );
              return $vault_response;


            }

            


        }
        else{
        	    // Customer not in the Vault. Create Customer Profile with payment
            	$vault_data = array(
            	      'profile' => array(
            	         'merchantCustomerId' => $merchantCustomerId,
            	         'description' => $company_name ?? $fname . ' ' . $lname,
            	         'paymentProfiles' => array(
    	                     'billTo' => array(
    	                        'firstName' => $fname,
    	                        'lastName' => $lname,
    	                        'company' => $company_name
    	                       ),
           	                'payment' => array(
           	                     'creditCard' => array(
           	                         'cardNumber'=>$this->card_num,
               	                     'expirationDate' => $this->card_expdate
               	                     )
           	                ),
            	            )
           	          ),
           	          'validationMode' => 'testMode'  //Just store the card, do not validate
          	        );
          	        
              $submit_data = json_encode(
                                array(
                                  'createCustomerProfileRequest' => array_merge($authentication_array,$vault_data)
                                )
                             );

              //Send Vault Request, and return custom array from response
              //Get both ids, from the response.  The PaymentProfileId will be in a list, 
              //but since the Customer Profile was just created, we can assume only one.
              $response = $this->_gateway_request($submit_data);
              if($this->commError !=''){
              	 return;
              }
              $vault_response = array(
                                 'resultCode'=>$response['messages']['resultCode'],
                                 'customerProfileId'=>$response['customerProfileId'],
                                 'customerPaymentProfileId'=>$response['customerPaymentProfileIdList'][0],
                                 'errorMessage'=>'Creating Customer Profile - '.$response['messages']['message'][0]['text']
                              );
              return $vault_response;


        }
        
        
        
        
    
    }
    elseif($submit_type == 'VaultByReference'){
        //Not used for this gateway
    
    }
    elseif($submit_type == 'SaleVault' || $submit_type == 'PreAuthVault'){
    	  
    	
        //Build array to send for Vault Sale, Vault PreAuth
        $transaction_type = ($this->authtype=='preauth') ? 'authOnlyTransaction' : 'authCaptureTransaction' ;
        $token_value = $this->token_value;
        // Split the string by the pipe character
        $parts = explode("|", $token_value);
        $customerProfileId = $parts[0];
        $customerPaymentProfileId = $parts[1];

        
        $transaction_array = array(
                        'transactionRequest' => array(
                          'transactionType' => $transaction_type, 
                          'amount' => number_format($this->get_order_total(), 2,'.',''),
                          'profile' => array(
                               'customerProfileId' => $customerProfileId,
                               'paymentProfile'=> array(
                                    'paymentProfileId' => $customerPaymentProfileId
                                              )
                                     ),
                          'order' => array(
                             'invoiceNumber' => str_replace( "#", "", $order->get_order_number() ),
                                     )
                              )
                          );

           $submit_data = json_encode(
                          array(
                            'createTransactionRequest' => array_merge($authentication_array,$transaction_array)
                          )
                       );
        
           return $this->_gateway_request($submit_data);
     
    }
    else {

    	
    	  //Sale or PreAuth via Credit Card
        //Build array to send for Sale, PreAuth
        $transaction_type = ($this->authtype=='preauth') ? 'authOnlyTransaction' : 'authCaptureTransaction' ;
        $transaction_array = array(
                        'transactionRequest' => array(
                          'transactionType' => $transaction_type, 
                          'amount' => number_format($this->get_order_total(), 2,'.',''),
                          'payment' => array(
                             'creditCard' => array(
                                'cardNumber' => $this->card_num,
                                'expirationDate' => $this->card_expdate,  //yyyy-mm
                                'cardCode' => ( isset( $_POST[$this->id.'-card-cvc'] ) ) ? $_POST[$this->id.'-card-cvc'] : ''
                                )
                          ),
                          'order' => array(
                             'invoiceNumber' => str_replace( "#", "", $order->get_order_number() ),
                          ),
                          'billTo' => array(
                             'firstName' => $order->get_billing_first_name(),
                             'lastName' => $order->get_billing_last_name(),
                             'company' => $order->get_billing_company(),
                             'address' => $order->get_billing_address_1() . ' ' . $order->get_billing_address_2(),
                             'city' => $order->get_billing_city(),
                             'company' => $order->get_billing_state(),
                             'zip' => $order->get_billing_postcode(),
                          ),
                        ) 
                      );

        
           $submit_data = json_encode(
                          array(
                            'createTransactionRequest' => array_merge($authentication_array,$transaction_array)
                          )
                       );
        
           return $this->_gateway_request($submit_data);
        
    }

    

  }

  /**
   * Submit a gateway request and receive the response
   * This function is strictly for communication, sending a preformed JSon Array packet and receiving the response
  */
  private function _gateway_request($submit_data = array()){

    $this->commError = '';
    if (!extension_loaded('curl')) { 
    	 $this->commError = 'cURL is not loaded';
    	 return;
    }

    // Initialize cURL session
    $url = MM_Connector_Functions::get_revita_url($this->sandboxmode);
    //mask logged data
    $masked_loginpassword='XXXXXXX';
    $log_data = $submit_data;
    $log_data = str_replace($this->loginpassword, $masked_loginpassword, $log_data);
    if (!empty($this->card_num) && strlen($this->card_num) > 4) {
       // Prepare the masked version of the card number (XXXX + last 4 digits)
       $maskedCardNum = 'XXXX' . substr($this->card_num, -4);
    
       // Replace the card number in the JSON string with its masked version
       $log_data = str_replace($this->card_num, $maskedCardNum, $log_data);
    }
    MM_Connector_Functions::debug_log('Data Submitted to Gateway: '.print_r($log_data,true),1); //log payment gateway debug
    $ch = curl_init($url);
    // Set cURL options
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_POST, true);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $submit_data);
    curl_setopt($ch, CURLOPT_HTTPHEADER, array(
        'Content-Type: application/json; charset=UTF-8',
        'Content-Length: ' . strlen($submit_data)
    ));

    // Execute cURL session and get the response
    $response = curl_exec($ch);

    if (curl_errno($ch) != 0) {
        $this->commError = curl_error($ch);
    };
    

    // Close cURL session
    curl_close($ch);

    //Get rid of invalid chars from response (needed for json_decode to work properly)
    $response = preg_replace( '/[\x00-\x1F\x80-\xFF]/', '',$response);
    MM_Connector_Functions::debug_log('Data Returned from Gateway: '.print_r($response,true),1); //log payment gateway debug
    
    // Decode JSON response
    $decodedResponse = json_decode($response,true);
    if (json_last_error()){
    	 $this->commError = "JSON decoding error: " . json_last_error_msg();
       	  
    }
    
    return $decodedResponse ;
  	
  }

  public function customize_save_payment_method_checkbox_text($html, $gateway) {
      if ($this->id === $gateway->id) {
          $new_text = $this->get_option('cardonfile_text', $this->card_on_file_text); // Default to $this->card_on_file_text if not set
          $html = str_replace('Save to account', esc_html($new_text), $html);
      }
      return $html;
  }


	
}