<?php
if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

class WC_Email_Ready_For_Pickup extends WC_Email {

    public function __construct() {
        $this->id             = 'ready_for_pickup';
        $this->title          = __('Ready for Pickup', 'woocommerce');
        $this->description    = __('Sent to customers when their order is ready for pickup.', 'woocommerce');
        $this->heading        = __('Your Order is Ready for Pickup', 'woocommerce');
        $this->subject        = __('Your {site_title} order #{order_number} is ready for pickup!', 'woocommerce');
        $this->template_html  = 'emails/ready-for-pickup.php';
        $this->template_plain = 'emails/plain/ready-for-pickup.php';
        parent::__construct();
        $this->customer_email = true;
        $this->template_base  = MM_PLUGIN_PATH . 'templates/';
    }

    public function trigger( $order_id ) {
        if ( !$order_id ) return;
        $this->object = wc_get_order( $order_id );
        $this->recipient = $this->object->get_billing_email();
        if ( ! $this->is_enabled() || ! $this->get_recipient() ) {
            return;
        }
        $this->send( $this->get_recipient(), $this->get_subject(), $this->get_content(), $this->get_headers(), $this->get_attachments() );
    }

    public function get_content_html() {
        return wc_get_template_html( $this->template_html, array(
            'order'         => $this->object,
            'email_heading' => $this->get_heading(),
            'sent_to_admin' => false,
            'plain_text'    => false,
            'email'         => $this
        ) );
    }

    public function get_content_plain() {
        return wc_get_template_html( $this->template_plain, array(
            'order'         => $this->object,
            'email_heading' => $this->get_heading(),
            'sent_to_admin' => false,
            'plain_text'    => true,
            'email'         => $this
        ) );
    }
}