<?php

//This is the main plugin code
class ManageMoreConnector {
	

    /*
    |------------------------------------------------------------------------------------------------------------------
    | Class Properties
    |------------------------------------------------------------------------------------------------------------------
     */
     
    public $wpp_active;  // If Wholesale Prices Premium is active, more functionality is allowed


    /*
    |------------------------------------------------------------------------------------------------------------------
    | Class Members
    |------------------------------------------------------------------------------------------------------------------
     */

    public $mm_rest_api;
    public $mm_post_types;
    public $mm_customer_price_list;
    public $mm_indirect_tax_list;
    public $mm_indirect_tax_jurisdiction_list;


    /*
    |--------------------------------------------------------------------------
    | Class Methods
    |--------------------------------------------------------------------------
     */

    /**
     * ManageMoreConnector constructor.
     *
     * @since 1.0.0
     * @access public
     */
    public function __construct() {


        // REST API
        $this->mm_rest_api = MM_REST_API::instance(array());  //uses Singleton pattern
        // Post Types
        $this->mm_post_types = new MM_POST_TYPES;
        // Woo/Wholesale Pricing Related
        $this->mm_woo_wholesale = new MM_WOO_WHOLESALE;
        //Woo Only Related
        $this->mm_woo_extender = new MM_WOO_EXTENDER;
        
   	

    }

    /**
     * Main execution.
     *
     * @since 1.0.0
     * @access public
     */
    public function run() {

      //Register Plugin Settings------------------------------------------------------------------------------------
      add_action( 'admin_init',  array($this,'managemore_connector_settings') );
      //Create Plugin Menu------------------------------------------------------------------------------------------
      add_action( 'admin_menu',  array($this,'managemore_connector_menu') );
      //Setup Action Links------------------------------------------------------------------------------------------
      add_filter( 'plugin_action_links_' . MM_PLUGIN_BASENAME,  array($this,'managemore_connector_action_links') );
      //Setup Meta Links--------------------------------------------------------------------------------------------
      add_filter( 'plugin_row_meta',  array($this,'managemore_connector_meta_links'), 10, 2 );
      //Add field for last login timestamp -------------------------------------------------------------------------
      add_action( 'wp_login', array($this,'managemore_connector_record_last_login'), 10, 2 );
      //Add check if user disabled by MM ---------------------------------------------------------------------------
      //Deprecated. We now check if user is disabled/inactive at checkout
      //add_filter( 'authenticate', array($this,'managemore_connector_check_user_disabled'),100,3);
      //Add delete user logic (to remove customer pricing if user deleted ------------------------------------------
      add_action('delete_user',array($this,'managemore_connector_delete_user'));
      // Include Gateway Classes and register Payment Gateways with WooCommerce-------------------------------------
      add_action( 'plugins_loaded', array($this,'managemore_gateway_init'), 0 );
      //Set Screen Options for Customer Prices----------------------------------------------------------------------
      add_filter('set-screen-option', array($this,'customer_price_set_option'), 10, 3);

      // REST API
      $this->mm_rest_api->run();
      // Post Types
      $this->mm_post_types->run();
      // Woo/Wholesale Pricing Related
      if($this->wpp_active){
        $this->mm_woo_wholesale->run();
      }
      $this->mm_woo_extender->run();


    }
  
  
    
    /**
     * Register Plugin Settings.
     *
     * @since 1.0.0
     * @access public
     */
    public function managemore_connector_settings() {

    	//register our settings
    	//1.2.0 - Added customer pricing switch
    	//4.0.0 - Added additional settings
    	//4.1.0 - Separated messages section
    	register_setting( 'managemore-connector-settings-group', 'mm_indirect_tax_enable' );
    	register_setting( 'managemore-connector-settings-group', 'mm_customer_pricing_enable' );
    	//register_setting( 'managemore-connector-settings-group', 'mm_allow_price_lower_than_minimum' ); //depricated 4.6.0
    	//register_setting( 'managemore-connector-settings-group', 'mm_allow_sale_price_if_lower' );   //depricated 4.6.0
    	register_setting( 'managemore-connector-settings-group', 'mm_show_account_status_section' );
    	register_setting( 'managemore-connector-settings-group', 'mm_restrict_checkout_license_expired' );
    	register_setting( 'managemore-connector-settings-group', 'mm_indirect_tax_allow_cart_detail' );
    	register_setting( 'managemore-connector-settings-group', 'mm_indirect_tax_allow_cart_warning' );
    	register_setting( 'managemore-connector-settings-group', 'mm_debug_mode' );
    	register_setting( 'managemore-connector-crm-settings-group', 'mm_show_customer_balance' );
    	register_setting( 'managemore-connector-crm-settings-group', 'mm_allow_balance_payment' );
    	register_setting( 'managemore-connector-crm-settings-group', 'mm_show_credit_limit' );
      register_setting( 'managemore-connector-crm-settings-group', 'mm_balance_payment_gateway');
      register_setting( 'managemore-connector-crm-settings-group', 'mm_use_payment_gateway_exclusive' );
    	register_setting( 'managemore-connector-crm-settings-group', 'mm_show_customer_last_payment' );
    	register_setting( 'managemore-connector-crm-settings-group', 'mm_show_customer_tax_info' );
    	register_setting( 'managemore-connector-crm-settings-group', 'mm_show_customer_license_1' );
    	register_setting( 'managemore-connector-crm-settings-group', 'mm_show_customer_license_2' );
    	register_setting( 'managemore-connector-crm-settings-group', 'mm_show_customer_license_3' );
    	register_setting( 'managemore-connector-crm-settings-group', 'mm_customer_license_1_name' );
    	register_setting( 'managemore-connector-crm-settings-group', 'mm_customer_license_2_name' );
    	register_setting( 'managemore-connector-crm-settings-group', 'mm_customer_license_3_name' );
    	register_setting( 'managemore-connector-crm-settings-group', 'mm_license_change_email');
    	register_setting( 'managemore-connector-crm-settings-group', 'mm_allow_pdf_upload' );
    	register_setting( 'managemore-connector-message-settings-group', 'mm_disabled_account_message' );
    	register_setting( 'managemore-connector-message-settings-group', 'mm_inactive_account_message' );
      register_setting( 'managemore-connector-message-settings-group', 'mm_payment_checkout_message');
    	register_setting( 'managemore-connector-message-settings-group', 'mm_license_expired_message' );
    	register_setting( 'managemore-connector-message-settings-group', 'mm_indirect_tax_warning_message' );
    	register_setting( 'managemore-connector-message-settings-group', 'mm_indirect_tax_detail_message' );
    	register_setting( 'managemore-connector-message-settings-group', 'mm_license_expired_notation_1' );
    	register_setting( 'managemore-connector-message-settings-group', 'mm_license_expired_notation_2' );
    	register_setting( 'managemore-connector-message-settings-group', 'mm_license_expired_notation_3' );
    	register_setting( 'managemore-connector-message-settings-group', 'mm_region_restricted_notation' );
    	register_setting( 'managemore-connector-message-settings-group', 'mm_region_restricted_message' );
    	


      // Check if managemore-connector-crm-settings was submitted, and check if Customer Payment SKU needs to be created.
      if (isset($_POST['option_page']) && $_POST['option_page'] === 'managemore-connector-crm-settings-group') {
          // Check if the mm_allow_balance_payment option was enabled
          if (isset($_POST['mm_allow_balance_payment']) && $_POST['mm_allow_balance_payment'] === '1') {
              // Check for and create the SKU
              $this->mm_woo_extender->check_for_pay_balance_product();
          }
      }
      
      //Check if Reset button pressed on Message Settings Page
      if (isset($_POST['reset']) && $_POST['option_page'] === 'managemore-connector-message-settings-group') {

          // add a notice to inform the user that the options have been reset
          add_settings_error('managemore-connector-message-settings-group', 'settings_updated', 'Messages reset to defaults.', 'updated');
      }

    }


    /**
     * Create Plugin Menu.
     *
     * @since 1.0.0
     * @access public
     */
    public function managemore_connector_menu() {
       global $menu, $submenu;


    	 //create new top-level menu
    	 add_menu_page( 'ManageMore WooCommerce Connector', 'ManageMore Connector', 'administrator', 'mm-connector-main', array($this,'managemore_connector_settings_page'),  plugins_url('/images/mmlogo.png', __FILE__ ) );

       //Remove submenu for main item by clearing array
    	 $submenu['mm-connector-main'] = array();
       //create submenu pages (need a function for each, below)
       add_submenu_page( 'mm-connector-main', 'ManageMore Connector', 'Settings', 'administrator', 'managemore-connector-settings', array($this,'managemore_connector_settings_page'));

       //If Account Status Section Shown, show options page for it in the menu
       $mm_show_account_status_section = get_option('mm_show_account_status_section',1);
       if ($mm_show_account_status_section){
          add_submenu_page( 'mm-connector-main', 'Account Status', 'Account Status', 'administrator','managemore-connector-crm-settings', array($this,'managemore_connector_crm_settings_page')); 
       }

       add_submenu_page( 'mm-connector-main', 'Custom Messages', 'Custom Messages', 'administrator','managemore-connector-messge-settings', array($this,'managemore_connector_message_settings_page')); 

       //Customer Pricing and Indirect Taxes are only allowed if Wholesale Prices Premium is active
       if($this->wpp_active){ 
          //2.0.0 - Use List of Database Table instead of Custom Post Type       	
          //add_submenu_page( 'mm-connector-main', 'Customer Prices', 'Customer Prices', 'administrator', 'edit.php?post_type=mm-customer-price'); 
          $hook = add_submenu_page( 'mm-connector-main', 'Customer Prices', 'Customer Prices', 'administrator', 'managemore-connector-customer-pricing',array($this,'managemore_connector_customer_pricing_page')); 
          add_action( "load-$hook", array($this,'add_customer_price_options' )); //Add options to this page
          
          if (get_option('mm_indirect_tax_enable',1) == 1 ) {
              add_submenu_page( 'mm-connector-main', 'Indirect Tax Types', 'Indirect Tax Types', 'administrator', 'edit.php?post_type=mm-indirect-tax-type'); 
              //2.1.0 - Use List of Database Table instead of Custom Post Type       	
              //add_submenu_page( 'mm-connector-main', 'Indirect Tax Rules', 'Indirect Tax Rules', 'administrator', 'edit.php?post_type=mm-indirect-tax'); 
              $hook1 = add_submenu_page( 'mm-connector-main', 'Indirect Tax Rules', 'Indirect Tax Rules', 'administrator', 'managemore-connector-indirect-tax-rules', array($this,'managemore_connector_indirect_tax_rules_page')); 
              add_action( "load-$hook1", array($this,'add_indirect_tax_options' )); //Add options to this page
              $hook2 = add_submenu_page('mm-connector-main', 'Indirect Tax Jurisdiction Rules', 'Indirect Tax Jurisdiction Rules', 'administrator', 'managemore-connector-jurisdiction-tax-rules', array($this, 'managemore_connector_jurisdiction_tax_rules_page'));
              add_action("load-$hook2", array($this, 'add_jurisdiction_tax_options')); // Add options to the new page
              
          }
       }
       

    }


    /**
     * Setup Action Links.
     *
     * @since 1.0.0
     * @access public
     */
    public function managemore_connector_action_links( $links ) {

      $plugin_links = array();
    	$plugin_links = array(
    		'<a href="' . admin_url( 'admin.php?page=managemore-connector-settings' ) . '">Plugin Settings</a>'
    	);
    	// Merge our new link with the default ones
    	return array_merge(  $links, $plugin_links );	
    }


    /**
     *Setup Meta Links.
     *
     * @since 1.0.0
     * @access public
     */
    public function managemore_connector_meta_links( $links, $file ) {
    	$plugin = MM_PLUGIN_BASENAME;
    	// create link
    	if ( $file == $plugin ) {
    		return array_merge(
    			$links,
    			array( '<a href="https://www.managemore.com/tutorials/installing-econnect-plugin-wordpress-woocommerce/" target="_blank" >Documentation</a>',
    			       '<a href="https://www.managemore.com/demo" target="_blank">Get a FREE ManageMore Demo</a>' )
    		);
    	}
    	return $links;

    }


    /**
     * Setup Settings Page (called by Plugin Menu).
     *
     * @since 1.0.0
     * @access public
     */
    public function managemore_connector_settings_page() {

   	if (MM_Connector_Functions::is_plugin_active('ultimate-member/ultimate-member.php')) {
    		$account_page_name = 'Ultimate Member "Account"';
    	}
      else {
    		$account_page_name = 'WooCommerce "My Account"';
    	}

    	$mm_debug_mode = get_option('mm_debug_mode',0);

    ?>
    <div class="wrap">
    <h1><?php echo '<img src="' . plugins_url( 'images/mmpill.png', __FILE__ ) . '" > '; ?>ManageMore Connector Settings</h1>


    <?php settings_errors(); ?>

    <form method="post" action="options.php">
        <?php settings_fields( 'managemore-connector-settings-group' ); ?>
        <?php do_settings_sections( 'managemore-connector-settings-group' ); ?>
        <br>
        <br>
        <h2>General Settings</h2>
        <table class="form-table">


            <tr valign="top">
            <th scope="row">Show Account Status</th>
            <td><input type="checkbox" name="mm_show_account_status_section" value="1" <?php checked( get_option('mm_show_account_status_section',1), 1 ); ?> /></td>
            <td><i>Check to add tab(s) on the <?php echo $account_page_name; ?> page for Customer Account Status (e.g. Balance, License, Payment)</i></td>
            </tr>


    <?php if ($this->wpp_active) { ?>

            <tr valign="top">
            <th scope="row">Enable Customer Pricing</th>
            <td><input type="checkbox" name="mm_customer_pricing_enable" value="1" <?php checked( get_option('mm_customer_pricing_enable',1), 1 ); ?> /></td>
            <td><i>Check to allow customer-specific pricing</i></td>
            </tr>


            <tr valign="top">
            <th scope="row">Enable Indirect Taxes</th>
            <td><input type="checkbox" name="mm_indirect_tax_enable" value="1" <?php checked( get_option('mm_indirect_tax_enable',1), 1 ); ?> /></td>
            <td><i>Check to add excise tax to product wholesale prices</i></td>
            </tr>

            <tr valign="top">
            <th scope="row">Show Indirect Tax Notation</th>
            <td><input type="checkbox" name="mm_indirect_tax_allow_cart_detail" value="1" <?php checked( get_option('mm_indirect_tax_allow_cart_detail',0), 1 ); ?> /></td>
            <td><i>Check to add indirect tax notation to price in cart</i></td>
            </tr>

            <tr valign="top">
            <th scope="row">Show Indirect Tax Notice</th>
            <td><input type="checkbox" name="mm_indirect_tax_allow_cart_warning" value="1" <?php checked( get_option('mm_indirect_tax_allow_cart_warning',0), 1 ); ?> /></td>
            <td><i>Check to add a message to the cart when indirect tax is present</i></td>
            </tr>

 
    <?php } ?>

            <tr valign="top">
            <th scope="row">Enforce License Expiration</th>
            <td><input type="checkbox" name="mm_restrict_checkout_license_expired" value="1" <?php checked( get_option('mm_restrict_checkout_license_expired',0), 1 ); ?> /></td>
            <td><i>Check to prevent sale of license restricted products when the user's License has expired</i></td>
            </tr>

            <tr valign="top">
            <th scope="row">Debug Mode</th>
            <td>
                <select name="mm_debug_mode" id="mm_debug_mode">
                   <option value="0" <?php selected($mm_debug_mode, 0); ?>>None</option>
                   <option value="1" <?php selected($mm_debug_mode, 1); ?>>Payment Gateway</option>
                   <option value="99" <?php selected($mm_debug_mode, 99); ?>>All</option>
                </select>
            </td>
            <td><i>Select mode for debugging.  WARNING: Sensitive information may be logged.  Use temporarily only, under support guidance, and remove log files when completed.</i></td>
           </tr>


        </table>
        
        <?php 
           submit_button(); 
        ?>

    </form>

    </div>
    <?php 
    } 

    /**
     * Account Status Options Page (called by Plugin Menu).
     *
     * @since 1.0.0
     * @access public
     */
    public function managemore_connector_crm_settings_page() {
    	
    	$mm_customer_license_1_name = esc_attr(MM_Connector_Functions::mm_get_option('mm_customer_license_1_name'));
    	$mm_customer_license_2_name = esc_attr(MM_Connector_Functions::mm_get_option('mm_customer_license_2_name'));
    	$mm_customer_license_3_name = esc_attr(MM_Connector_Functions::mm_get_option('mm_customer_license_3_name'));
    	$mm_license_change_email = esc_attr(MM_Connector_Functions::mm_get_option('mm_license_change_email'));
    	$mm_allow_pdf_upload = get_option('mm_allow_pdf_upload',1);
    	
    ?>
    <div class="wrap">
    <h1><?php echo '<img src="' . plugins_url( 'images/mmpill.png', __FILE__ ) . '" > '; ?>ManageMore Connector Settings</h1>


    <?php settings_errors(); ?>

    <form method="post" action="options.php">
        <?php settings_fields( 'managemore-connector-crm-settings-group' ); ?>
        <?php do_settings_sections( 'managemore-connector-crm-settings-group' ); ?>
        <br>
        <br>
        <h2>Account Status</h2>
        <table class="form-table">

            <tr valign="top">
            <th scope="row">Show Account Balance</th>
            <td><input type="checkbox" name="mm_show_customer_balance" value="1" <?php checked( get_option('mm_show_customer_balance',0), 1 ); ?> /></td>
            <td><i>Check to show the customer account balance and as of date in the Account Status section</i></td>
            </tr>

            <tr valign="top">
            <th scope="row">Show Credit Limit</th>
            <td><input type="checkbox" name="mm_show_credit_limit" value="1" <?php checked( get_option('mm_show_credit_limit',0), 1 ); ?> /></td>
            <td><i>Check to show the customer credit limit in the Account Status section.</i></td>
            </tr>

            <tr valign="top">
            <th scope="row">Allow Balance Payment</th>
            <td><input type="checkbox" name="mm_allow_balance_payment" value="1" <?php checked( get_option('mm_allow_balance_payment',0), 1 ); ?> /></td>
            <td><i>Check to show a Pay Balance button in the Account Status section if the customer has a positive balance.</i></td>
            </tr>

            <tr valign="top">
            <th scope="row">Balance Payment Gateway</th>
            <td>
                <select name="mm_balance_payment_gateway" id="mm_balance_payment_gateway">
                    <option value="">Any Active Gateway</option>
                    <?php
                    $selected_gateway = get_option('mm_balance_payment_gateway', '');
                    $gateways = WC()->payment_gateways->payment_gateways();
                    foreach ($gateways as $gateway_id => $gateway):
                        if ($gateway->enabled == 'yes' && $gateway_id != 'managemore_onaccount'):
                    ?>
                        <option value="<?php echo esc_attr($gateway_id); ?>" <?php selected($selected_gateway, $gateway_id); ?>><?php echo esc_html($gateway->title); ?></option>
                    <?php
                        endif;
                    endforeach;
                    ?>
                </select>
            </td>
            <td><i>Select the payment gateway to be used for balance payments. (Note: The "ManageMore On Account" gateway is always excluded)</i></td>
            </tr>


            <tr valign="top">
            <th scope="row">Exclusive Use of Selected Gateway</th>
            <td><input type="checkbox" name="mm_use_payment_gateway_exclusive" value="1" <?php checked( get_option('mm_use_payment_gateway_exclusive', 0), 1 ); ?> /></td>
            <td><i>Check to only allow the selected payment gateway for balance payments and prevent its usage for other types of purchases.</i></td>
            </tr>

            <tr valign="top">
            <th scope="row">Show Last Payment</th>
            <td><input type="checkbox" name="mm_show_customer_last_payment" value="1" <?php checked( get_option('mm_show_customer_last_payment',0), 1 ); ?> /></td>
            <td><i>Check to show the customer's last payment and date in the Account Status section</i></td>
            </tr>

            <tr valign="top">
            <th scope="row">Show Tax Info</th>
            <td><input type="checkbox" name="mm_show_customer_tax_info" value="1" <?php checked( get_option('mm_show_customer_tax_info',0), 1 ); ?> /></td>
            <td><i>Check to show the customer Tax Information in the Account Status section</i></td>
            </tr>

            <tr valign="top">
            <th scope="row">Show License 1</th>
            <td><input type="checkbox" name="mm_show_customer_license_1" value="1" <?php checked( get_option('mm_show_customer_license_1',0), 1 ); ?> /></td>
            <td><i>Check to show the customer license 1 in the Account Status section</i></td>
            </tr>

            <tr valign="top">
            <th scope="row">Show License 2</th>
            <td><input type="checkbox" name="mm_show_customer_license_2" value="1" <?php checked( get_option('mm_show_customer_license_2',0), 1 ); ?> /></td>
            <td><i>Check to show the customer license 2 in the Account Status section</i></td>
            </tr>

            <tr valign="top">
            <th scope="row">Show License 3</th>
            <td><input type="checkbox" name="mm_show_customer_license_3" value="1" <?php checked( get_option('mm_show_customer_license_3',0), 1 ); ?> /></td>
            <td><i>Check to show the customer license 3 in the Account Status section</i></td>
            </tr>

            <tr valign="top">
            <th scope="row">License 1 Name</th>
            <td><input type="text" name="mm_customer_license_1_name" size="30" value="<?php echo $mm_customer_license_1_name ; ?>"></td>
            <td><i>The prompt that will show for customer license 1</i></td>
            </tr>

            <tr valign="top">
            <th scope="row">License 2 Name</th>
            <td><input type="text" name="mm_customer_license_2_name" size="30" value="<?php echo $mm_customer_license_2_name ; ?>"></td>
            <td><i>The prompt that will show for customer license 2</i></td>
            </tr>

            <tr valign="top">
            <th scope="row">License 3 Name</th>
            <td><input type="text" name="mm_customer_license_3_name" size="30" value="<?php echo $mm_customer_license_3_name ; ?>"></td>
            <td><i>The prompt that will show for customer license 3</i></td>
            </tr>

            <tr valign="top">
            <th scope="row">License Change Email</th>
            <td><input type="text" name="mm_license_change_email" size="30" value="<?php echo $mm_license_change_email ; ?>"></td>
            <td><i>Enter the email address to receive a message whenever a user uploads a new license image</i></td>
            </tr>

        <?php if (MM_Connector_Functions::is_plugin_active('ultimate-member/ultimate-member.php')) { ?>
            <tr valign="top">
                <th scope="row">Upload File Types</th>
                <td>
                    <select name="mm_allow_pdf_upload" id="mm_allow_pdf_upload">
                        <option value="0" <?php selected($mm_allow_pdf_upload, 0); ?>>Image Files Only</option>
                        <option value="1" <?php selected($mm_allow_pdf_upload, 1); ?>>Images and PDF Files</option>
                    </select>
                </td>
                <td>
                    <i>Select the type of files allowed for upload. Note: If PDF files are allowed, then only one license image will be available, and you will have to add the image field to the User Profile.</i>
                </td>
            </tr>
        <?php } ?>

        </table>
        
        <?php 
           submit_button(); 
        ?>

    </form>

    </div>
    <?php 
    } 


    /**
     * Message Options Page (called by Plugin Menu).
     *
     * @since 1.0.0
     * @access public
     */
    public function managemore_connector_message_settings_page() {
    	
    	
      $mm_disabled_account_message = esc_attr(MM_Connector_Functions::mm_get_option('mm_disabled_account_message'));
      $mm_inactive_account_message = esc_attr(MM_Connector_Functions::mm_get_option('mm_inactive_account_message'));
      $mm_license_expired_message =  esc_attr(MM_Connector_Functions::mm_get_option('mm_license_expired_message'));
      $mm_indirect_tax_warning_message = esc_attr(MM_Connector_Functions::mm_get_option('mm_indirect_tax_warning_message'));
      $mm_indirect_tax_detail_message = esc_attr(MM_Connector_Functions::mm_get_option('mm_indirect_tax_detail_message'));
    	$thank_you_message = esc_attr(MM_Connector_Functions::mm_get_option('mm_payment_checkout_message'));
      $mm_region_restricted_message = esc_attr(MM_Connector_Functions::mm_get_option('mm_region_restricted_message'));
      $mm_region_restricted_notation =esc_attr(MM_Connector_Functions::mm_get_option('mm_region_restricted_notation'));
      $mm_license_expired_notation_1 = esc_attr(MM_Connector_Functions::mm_get_option('mm_license_expired_notation_1'));
      $mm_license_expired_notation_2 = esc_attr(MM_Connector_Functions::mm_get_option('mm_license_expired_notation_2'));
      $mm_license_expired_notation_3 = esc_attr(MM_Connector_Functions::mm_get_option('mm_license_expired_notation_3'));
      
    	
    ?>
    <div class="wrap">
    <h1><?php echo '<img src="' . plugins_url( 'images/mmpill.png', __FILE__ ) . '" > '; ?>ManageMore Connector Settings</h1>


    <?php settings_errors(); ?>

    <form method="post" action="options.php">
        <?php settings_fields( 'managemore-connector-message-settings-group' ); ?>
        <?php do_settings_sections( 'managemore-connector-message-settings-group' ); ?>
        <br>
        <br>
        <h2>Custom Messages</h2>
        <table class="form-table">


            <tr valign="top">
            <th scope="row">Inactive Account Message</th>
            <td colspan="2">
                <textarea name="mm_inactive_account_message" id="mm_inactive_account_message" rows="2" style="width: 100%;"><?php echo $mm_inactive_account_message; ?></textarea><br>
                <i>The message that will be displayed to a user that has been marked "Inactive" in ManageMore.</i>
            </td>



            <tr valign="top">
            <th scope="row">Disabled Account Message</th>
            <td colspan="2">
                <textarea name="mm_disabled_account_message" id="mm_disabled_account_message" rows="2" style="width: 100%;"><?php echo $mm_disabled_account_message; ?></textarea><br>
                <i>The message that will be displayed to a user that has been disabled for checkout.</i>
            </td>


            <tr valign="top">
            <th scope="row">Payment Thank You Message</th>
            <td colspan="2">
                <textarea name="mm_payment_checkout_message" id="mm_payment_checkout_message" rows="2" style="width: 100%;"><?php echo $thank_you_message; ?></textarea><br>
                <i>Enter a message to be displayed after Customer Balance payment checkout</i>
            </td>

            </tr>
            <tr valign="top">
            <th scope="row">Expired License Message</th>
            <td colspan="2">
                <textarea name="mm_license_expired_message" id="mm_license_expired_message" rows="2" style="width: 100%;"><?php echo esc_attr($mm_license_expired_message); ?></textarea><br>
                <i>The message that will be displayed to a user whose license has expired. You can use the placeholders {license_name} and {notation} in the message.</i>
            </td>
            </tr>

            <tr valign="top">
            <th scope="row">Indirect Tax Warning Message</th>
            <td colspan="2">
                <textarea name="mm_indirect_tax_warning_message" id="mm_indirect_tax_warning_message" rows="2" style="width: 100%;"><?php echo $mm_indirect_tax_warning_message; ?></textarea><br>
                <i>Enter a message to be displayed if indirect taxes were added to any item. You can use the placeholder {indirect_tax_total} in the message.</i>
            </td>
            </tr>

            <tr valign="top">
            <th scope="row">Region Restriction Message</th>
            <td colspan="2">
                <textarea name="mm_region_restricted_message" id="mm_region_restricted_message" rows="2" style="width: 100%;"><?php echo $mm_region_restricted_message; ?></textarea><br>
                <i>Enter a message to be displayed if a product cannot be shipped to the provided state/country.</i>
            </td>
            </tr>

            <tr valign="top">
            <th scope="row">Indirect Tax Notation</th>
            <td colspan="2">
            	 <input type="text" name="mm_indirect_tax_detail_message" size="50" value="<?php echo esc_attr($mm_indirect_tax_detail_message) ; ?>">
                <br><i>The notation that will be displayed next to the price of an item with indirect tax included. You can use the placeholder {indirect_tax_amount} in the note.</i>
            </td>
            </tr>

            <tr valign="top">
            <th scope="row">Region Restriction Notation</th>
            <td colspan="2">
            	 <input type="text" name="mm_region_restricted_notation" size="10" value="<?php echo esc_attr($mm_region_restricted_notation) ; ?>">
                <br><i>The notation that will be displayed before the name of an item that cannot be shipped to the provided state/country.</i>
            </td>
            </tr>

            <tr valign="top">
            <th scope="row">License 1 Notation</th>
            <td colspan="2">
            	 <input type="text" name="mm_license_expired_notation_1" size="10" value="<?php echo esc_attr($mm_license_expired_notation_1) ; ?>">
                <br><i>The notation that will be displayed before the name of an item that cannot be sold due to License 1 expiration.</i>
            </td>
            </tr>

            <tr valign="top">
            <th scope="row">License 2 Notation</th>
            <td colspan="2">
            	 <input type="text" name="mm_license_expired_notation_2" size="10" value="<?php echo esc_attr($mm_license_expired_notation_2) ; ?>">
                <br><i>The notation that will be displayed before the name of an item that cannot be sold due to License 2 expiration.</i>
            </td>
            </tr>


            <tr valign="top">
            <th scope="row">License 3 Notation</th>
            <td colspan="2">
            	 <input type="text" name="mm_license_expired_notation_3" size="10" value="<?php echo esc_attr($mm_license_expired_notation_3) ; ?>">
                <br><i>The notation that will be displayed before the name of an item that cannot be sold due to License 3 expiration.</i>
            </td>
            </tr>

        </table>
        
        <?php 
           submit_button(); 
        ?>

    </form>

    </div>
    <?php 
    } 



    /**
     * Customer Pricing List Page (called by Plugin Menu).
     *
     * @since 2.0.0
     * @access public
     */
    public function managemore_connector_customer_pricing_page(){


     ?>
        <div class="wrap" id="managemore-connector-customer-pricing">
        <h2>Customer Specific Pricing</h2>

        <div id="poststuff">
        <div id="post-body" class="metabox-holder columns-2">
        <div id="post-body-content">
        <div class="meta-box-sortables ui-sortable">
        <form method="post">
        <?php
          $this->mm_customer_price_list->prepare_items();
          $this->mm_customer_price_list->display(); ?>
        </form>
        </div>
        </div>
        </div>
        <br class="clear">
        </div>
        </div>
        <?php     	
    }


    /**
     * Add Options to Customer Pricing List Page, and Instantiate Customer Pricing List object
     *
     * @since 2.0.0
     * @access public
     */
    public function add_customer_price_options() {
        $option = 'per_page';
        $args = array(
              'label' => 'Customer Prices',
              'default' => 50,
              'option' => 'customer_prices_per_page'
              );
         add_screen_option( $option, $args );

        // Instantiate Customer Price List object
        $this->mm_customer_price_list = new MM_Customer_Price_List;


    }
 

    /**
     * Indirect Tax Rules Page (called by Plugin Menu).
     *
     * @since 2.0.0
     * @access public
     */
    public function managemore_connector_indirect_tax_rules_page(){

        	   //Added 2.1.3 - Get Wholesales Prices Premium settings for minimum order for Wholesale Price to apply 
        	   //              (these settings can cause Wholesale Price not to override, thus causing indirect tax not to calculate
        	   //
        	   $wwpp_min_order_price =  get_option('wwpp_settings_minimum_order_price',0);
        	   $wwpp_min_order_qty = get_option('wwpp_settings_minimum_order_quantity',0);
        	   $wwpp_min_order_warning = '';
        	   if ($wwpp_min_order_price != 0 || $wwpp_min_order_qty !=0){
        	   	$wwpp_min_order_warning = '<p style="color:red;">WARNING! You have enabled the minimum order settings for Wholesale Prices Premium.  If the minimum order quantity/amount is not met, Indirect Taxes will not be added.</p><br>';
        	   }


     ?>
        <div class="wrap">
        <h2>Indirect Tax Rules</h2>
        <?php echo $wwpp_min_order_warning;?>
        <div id="poststuff">
        <div id="post-body" class="metabox-holder columns-2">
        <div id="post-body-content">
        <div class="meta-box-sortables ui-sortable">
        <form method="post">
        <?php
          $this->mm_indirect_tax_list->prepare_items();
          $this->mm_indirect_tax_list->display(); ?>
        </form>
        </div>
        </div>
        </div>
        <br class="clear">
        </div>
        </div>
        <?php     	
    }


    /**
     * Add Options to Indirect Tax Rule List Page, and Instantiate Indirect Tax List object
     *
     * @since 2.0.0
     * @access public
     */
    public function add_indirect_tax_options() {
        $option = 'per_page';
        $args = array(
              'label' => 'Rules',
              'default' => 50,
              'option' => 'indirect_taxes_per_page'
              );
         add_screen_option( $option, $args );

        // Instantiate Indirect Tax List object
        $this->mm_indirect_tax_list = new MM_Indirect_Tax_List;


    }

    /**
     * Indirect Tax Jurisdiction Rules Page (called by Plugin Menu).
     *
     * @since 2.0.0
     * @access public
     */
    public function managemore_connector_jurisdiction_tax_rules_page(){

        	   //Added 2.1.3 - Get Wholesales Prices Premium settings for minimum order for Wholesale Price to apply 
        	   //              (these settings can cause Wholesale Price not to override, thus causing indirect tax not to calculate
        	   //
        	   $wwpp_min_order_price =  get_option('wwpp_settings_minimum_order_price',0);
        	   $wwpp_min_order_qty = get_option('wwpp_settings_minimum_order_quantity',0);
        	   $wwpp_min_order_warning = '';
        	   if ($wwpp_min_order_price != 0 || $wwpp_min_order_qty !=0){
        	   	$wwpp_min_order_warning = '<p style="color:red;">WARNING! You have enabled the minimum order settings for Wholesale Prices Premium.  If the minimum order quantity/amount is not met, Indirect Taxes will not be added.</p><br>';
        	   }


     ?>
        <div class="wrap">
        <h2>Indirect Tax Jurisdiction Rules</h2>
        <?php echo $wwpp_min_order_warning;?>
        <div id="poststuff">
        <div id="post-body" class="metabox-holder columns-2">
        <div id="post-body-content">
        <div class="meta-box-sortables ui-sortable">
        <form method="post">
        <?php
          $this->mm_indirect_tax_jurisdiction_list->prepare_items();
          $this->mm_indirect_tax_jurisdiction_list->display(); ?>
        </form>
        </div>
        </div>
        </div>
        <br class="clear">
        </div>
        </div>
        <?php     	
    }

    /**
     * Add Options to Indirect Tax Jurisdiction Rule List Page, and Instantiate Indirect Tax List object
     *
     * @since 2.0.0
     * @access public
     */
    public function add_jurisdiction_tax_options() {
        $option = 'per_page';
        $args = array(
              'label' => 'Rules',
              'default' => 50,
              'option' => 'indirect_taxes_per_page'
              );
         add_screen_option( $option, $args );

        // Instantiate Indirect Tax List object
        $this->mm_indirect_tax_jurisdiction_list = new MM_Indirect_Tax_Jurisdiction_List;


    }


    /**
     * Save Options for Customer Pricing List
     *
     * @since 2.0.0
     * @access public
     */
    public static function customer_price_set_option($status, $option, $value) {
       return $value;
    }


    /**
     * Record extra fields at login, like last login date, IP Address, Browser, etc. (for REST API).
     *
     * @since 1.0.0
     * @access public
     */
    public function managemore_connector_record_last_login( $user_login, $user ) {
    	
        //Record Login Timestamp   	
        update_user_meta( $user->ID, 'mm_last_login', time() );
        //Record IP Address
        $ip_address = '';
        if ( ! empty( $_SERVER['HTTP_CLIENT_IP'] ) ) {
            //IP from share internet
            $ip_address = $_SERVER['HTTP_CLIENT_IP'];
         } elseif ( ! empty( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) {
            //to check if IP is passed from proxy
            $ip_address = $_SERVER['HTTP_X_FORWARDED_FOR'];
         } else {
            $ip_address = $_SERVER['REMOTE_ADDR'];
         }
         update_user_meta( $user->ID, 'mm_last_login_ip', $ip_address );
         //Record Browser Info
         $browser_info = $_SERVER['HTTP_USER_AGENT'];
         update_user_meta( $user->ID, 'mm_last_login_browser', $browser_info );
         


    }

    /**
     * Check user disable status
     *
     *
     * @since 1.0.0
     * @access public
     * 
     */
    public function managemore_connector_check_user_disabled ($user,$username,$password){
      //Corrected detection of wp_error 2.1.2 (5.12.2022)
    	if($user == NULL || is_wp_error($user)){
    		return $user;
    	}

     /*
     * This was changed (3.0.4) - We no longer prevent login, due to disabled status.  We prevent checkout instead.
     * This is so user can still update their info, such as license image, even when locked out. 
     */

    	//$mm_disable_user =  get_user_meta($user->ID,'mm_disable_user',true);
    	$mm_field_STAT =  get_user_meta($user->ID,'mm_field_STAT',true);
    	//added inactive check 1.0.2
    	if ($mm_field_STAT == 'Inactive'){
    		$mm_inactive_account_message = MM_Connector_Functions::mm_get_option('mm_inactive_account_message');
    		return new WP_Error( 'authentication_failed',$mm_inactive_account_message);
    	}
    	//elseif ($mm_disable_user == '1'){
    	//	$mm_disabled_account_message = MM_Connector_Functions::mm_get_option('mm_disabled_account_message');
    	//	return new WP_Error( 'authentication_failed',$mm_disabled_account_message);
    	//}
    	else {
    		return $user;
    	}
    } 



    //Initialize Gateways
    public function managemore_gateway_init() {

      //Woo Gateways
      require_once 'includes/class-wc-gateway-managemore-onaccount.php' ;
      require_once 'includes/class-wc-gateway-managemore-tsys.php' ;
      require_once 'includes/class-wc-gateway-managemore-revita.php' ;
      //require_once 'includes/class-wc-gateway-managemore-vantiv.php' ;
     


    	add_filter( 'woocommerce_payment_gateways', 'add_managemore_onaccount_gateway' );
    	function add_managemore_onaccount_gateway( $methods ) {
    		$methods[] = 'WC_Gateway_ManageMore_Onaccount';
    		return $methods;
    	}

//TSYS Gateway depricated 4.7.0
/*    	add_filter( 'woocommerce_payment_gateways', 'add_managemore_tsys_gateway' );
    	function add_managemore_tsys_gateway( $methods ) {
    		$methods[] = 'WC_Gateway_ManageMore_TSYS';
    		return $methods;
    	}
*/


//Vantiv gateway no longer supported
    	//add_filter( 'woocommerce_payment_gateways', 'add_managemore_vantiv_gateway' );
/*    	function add_managemore_vantiv_gateway( $methods ) {
    		$methods[] = 'WC_Gateway_ManageMore_Vantiv';
    		return $methods;
    	}
*/

    	add_filter( 'woocommerce_payment_gateways', 'add_managemore_revita_gateway' );
    	function add_managemore_revita_gateway( $methods ) {
    		$methods[] = 'WC_Gateway_ManageMore_RevitA';
    		return $methods;
    	}


    }

    public function managemore_connector_delete_user($user_id){

        $customer_pricing  = ''; //Sending blank string will cause all pricing to be removed
    	  MM_Connector_Functions::update_customer_pricing($customer_pricing,$user_id,1);
    	  
    }

	
}



