<?php

//This is the main plugin code
class ManageMoreConnector {
	

    /*
    |------------------------------------------------------------------------------------------------------------------
    | Class Properties
    |------------------------------------------------------------------------------------------------------------------
     */
     
    public $wpp_active;  // If Wholesale Prices Premium is active, more functionality is allowed
    public $mm_rest_api;
    public $mm_woo_wholesale;
    public $mm_woo_extender;
    public $mm_post_types;
    public $mm_woo_order_status;
    public $mm_customer_price_list;
    public $mm_indirect_tax_list;
    public $mm_indirect_tax_jurisdiction_list;
    public $mm_trade_class_restriction_list;
    private $default_messages;
    private $default_settings;
    private $settings_fields;
    private $tabs;

    /*
    |--------------------------------------------------------------------------
    | Class Methods
    |--------------------------------------------------------------------------
     */

    /**
     * ManageMoreConnector constructor.
     *
     * @since 1.0.0
     * @access public
     */
    public function __construct() {


        // REST API
        $this->mm_rest_api = MM_REST_API::instance(array());  //uses Singleton pattern
        // Post Types
        $this->mm_post_types = new MM_POST_TYPES;
        //Woo Only Related
        //$this->mm_woo_extender = new MM_WOO_EXTENDER;
        $this->mm_woo_extender = MM_WOO_EXTENDER::instance(); //uses Singleton pattern
        // Woo/Wholesale Pricing Related
        //$this->mm_woo_wholesale = new MM_WOO_WHOLESALE;
        $this->mm_woo_wholesale = MM_WOO_WHOLESALE::instance(); //uses Singleton pattern
        //Woo Order Status Related
        $this->mm_woo_order_status = new MM_WOO_ORDER_STATUS;
        
        $this->default_settings = [
            // General Settings
            'mm_show_account_status_section' => true,
            'mm_restrict_checkout_license_expired' => false,
            'mm_customer_pricing_enable' => false,
            'mm_indirect_tax_enable' => false,
            'mm_show_indirect_tax_shop' => false,
            'mm_show_restriction_notation_shop' => false,
            'mm_indirect_tax_allow_cart_detail' => false,
            'mm_indirect_tax_allow_cart_warning' => false,

            // Account Settings
            'mm_show_customer_balance' => false,
            'mm_show_customer_last_payment' => false,
            'mm_show_customer_tax_info' => false,
            'mm_show_credit_limit' => false,
            'mm_allow_balance_payment' => false,
            'mm_balance_payment_gateway' => '',
            'mm_use_payment_gateway_exclusive' => false,

            // License Settings
            'mm_show_customer_license_1' => false,
            'mm_show_customer_license_2' => false,
            'mm_show_customer_license_3' => false,
            'mm_show_customer_license_4' => false,
            'mm_customer_license_1_name' => 'License 1',
            'mm_customer_license_2_name' => 'License 2',
            'mm_customer_license_3_name' => 'License 3',
            'mm_customer_license_4_name' => 'License 4',
            'mm_license_change_email' => '',
            'mm_allow_pdf_upload' => '0', // Default to Image Files Only
            
            //Cache Settings
            'mm_allow_cache' => false,

            // Advanced Settings
            'mm_debug_mode' => '0', // Default to "None"
          
        ];
        
        $this->default_messages = [
            'mm_disabled_account_message' => 'This account has been temporarily disabled.',
            'mm_inactive_account_message' => 'This account has been deactivated.',
            'mm_login_required_message' => 'You must be logged in to purchase this product.',
            'mm_license_expired_message' => '{notation} This item requires a valid {license_name}.',
            'mm_payment_checkout_message' => 'Thank you for your payment. Your online balance will not reflect this payment immediately. Please allow 24-48 hours for your balance to be updated online.',
            'mm_indirect_tax_warning_message' => '{notation} The price shown on the denoted cart items includes applicable excise taxes for your location.',
            'mm_indirect_tax_note_message' => 'Price includes excise tax',
            'mm_indirect_tax_detail_notation' => '&#x1F4CC;',
            'mm_license_expired_notation_1' => '&#x1F4E2;',
            'mm_license_expired_notation_2' => '&#x1F4E3;',
            'mm_license_expired_notation_3' => '&#x1F514;',
            'mm_region_restricted_notation' => '&#128205;',
            'mm_region_restricted_message' => '{notation} This item cannot be shipped to the location provided.',
        ];
        
        $this->settings_fields = [
            'general' => [
                ['mm_show_account_status_section', 'Show Account Status', 'checkbox', 'Check to display the Account Status tab.'],
                ['mm_restrict_checkout_license_expired', 'Restrict Checkout on License Expired', 'checkbox', 'Prevent checkout if the customer\'s license is expired.'],
                ['mm_customer_pricing_enable', 'Enable Customer Pricing', 'checkbox', 'Check to allow customer-specific pricing'],
                ['mm_indirect_tax_enable', 'Enable Indirect Taxes', 'checkbox', 'Check to add excise tax to product wholesale prices.', !$this->wpp_active],
                ['mm_show_indirect_tax_shop', 'Show Indirect Taxes in Shop', 'checkbox', 'Check to add excise tax to product wholesale prices on shop pages.'],
                ['mm_show_restriction_notation_shop', 'Show Restriction Notations in Shop', 'checkbox', 'Check to add notation to restricted products on shop pages.'],
                ['mm_indirect_tax_allow_cart_detail', 'Show Indirect Tax Notation', 'checkbox', 'Check to add excise tax notation when excise tax is present'],
                ['mm_indirect_tax_allow_cart_warning', 'Show Indirect Tax Notice', 'checkbox', 'Check to add a notice to the cart page when excise tax is present (Classic Cart Only)'],
            ],
            'account' => [
                ['mm_show_customer_balance', 'Show Customer Balance', 'checkbox', 'Display the account balance in the Account Status section.'],
                ['mm_show_customer_last_payment', 'Show Last Payment', 'checkbox', 'Display the last payment and date in the Account Status section.'],
                ['mm_show_customer_tax_info', 'Show Tax Info', 'checkbox', 'Display customer Tax Information in the Account Status section.'],
                ['mm_show_credit_limit', 'Show Credit Limit', 'checkbox', 'Display customer credit limit in the Account Status section.'],
                ['mm_allow_balance_payment', 'Allow Balance Payment', 'checkbox', 'Display a Pay Balance button if the customer has a positive balance.'],
                ['mm_balance_payment_gateway', 'Balance Payment Gateway', 'select_dynamic_gateway', 'Select the payment gateway for balance payments.'],
                ['mm_use_payment_gateway_exclusive', 'Exclusive Use of Selected Gateway', 'checkbox', 'Check to only allow the selected payment gateway for balance payments and prevent its usage for other types of purchases.'],
            ],
            'license' => [
                ['mm_show_customer_license_1', 'Show License 1', 'checkbox', 'Display customer license 1 in the Account Status section.'],
                ['mm_show_customer_license_2', 'Show License 2', 'checkbox', 'Display customer license 2 in the Account Status section.'],
                ['mm_show_customer_license_3', 'Show License 3', 'checkbox', 'Display customer license 3 in the Account Status section.'],
                ['mm_show_customer_license_4', 'Show License 4', 'checkbox', 'Display customer license 4 in the Account Status section.'],
                ['mm_customer_license_1_name', 'License Name 1', 'text', 'Custom name for License 1.'],
                ['mm_customer_license_2_name', 'License Name 2', 'text', 'Custom name for License 2.'],
                ['mm_customer_license_3_name', 'License Name 3', 'text', 'Custom name for License 3.'],
                ['mm_customer_license_4_name', 'License Name 4', 'text', 'Custom name for License 4.'],
                ['mm_license_change_email', 'License Change Notification Email', 'text', 'Email to receive license change notifications.'],
                ['mm_allow_pdf_upload', 'Upload File Types', 'select', 'Select the type of files allowed for upload. Note: If PDF files are allowed, then you will have to add the image field(s) to the User Profile Form in Ultimate Member.', [
                    '0' => 'Image Files Only',
                    '1' => 'Images and PDF Files'
                ]]
            ],
            'messages' => [
                ['mm_disabled_account_message', 'Disabled Account Message', 'textarea', 'The message that will be displayed to a user that has been marked "Inactive" in ManageMore.'],
                ['mm_inactive_account_message', 'Inactive Account Message', 'textarea', 'The message displayed when an account is inactive.'],
                ['mm_login_required_message', 'Login Required Message', 'textarea', 'Message shown when a user tries to purchsae a product requiring login.'],
                ['mm_license_expired_message', 'License Expired Message', 'textarea', 'Message shown when a user tries to purchsae a product requiring a valid license.'],
                ['mm_payment_checkout_message', 'Payment Checkout Message', 'textarea', 'Message displayed after a successful payment.'],
                ['mm_indirect_tax_warning_message', 'Indirect Tax Warning Message', 'textarea', 'Notification about excise taxes included in item prices.'],
                ['mm_indirect_tax_note_message', 'Indirect Tax Notation Message', 'text', 'Prompt included on exise tax item notations.'],
                ['mm_region_restricted_message', 'Region Restricted Message', 'textarea', 'Message shown when a user tries to purchsae a product in restricted regions. This also applies to Trade Class Retrictions.'],
                ['mm_indirect_tax_detail_notation', 'Indirect Tax Detail Notation', 'small-text', 'Symbol or notation used for excise tax items.'],
                ['mm_license_expired_notation_1', 'License Expired Notation 1', 'small-text', 'Symbol or notation used for expired licenses.'],
                ['mm_license_expired_notation_2', 'License Expired Notation 2', 'small-text', 'Second notation symbol for expired licenses.'],
                ['mm_license_expired_notation_3', 'License Expired Notation 3', 'small-text', 'Third notation symbol for expired licenses.'],
                ['mm_region_restricted_notation', 'Region Restricted Notation', 'small-text', 'Symbol or message used for region restrictions. This also applies to Trade Class Retrictions.']
            ],
            'performance' => [
                ['mm_allow_cache', 'Allow Caching', 'checkbox', 'Allow object caching of database queries for efficiency.']
            ],
            'advanced' => [
                ['mm_debug_mode', 'Debug Mode', 'select', 'Select mode for debugging.',MM_Debug_Mode::get_options()]
           ]
        ];



        $this->tabs['general'] = 'General';
        if (get_option('mm_show_account_status_section', 1) != 0) {
            $this->tabs['account'] = 'Account Status';
            $this->tabs['license'] = 'License';
        }
        $this->tabs['messages'] = 'Message';
        $this->tabs['performance'] = 'Performance';
        $this->tabs['advanced'] = 'Advanced';

    }

    /**
     * Register Plugin Settings.
     *
     * @since 1.0.0
     * @access public
     */
    public function managemore_connector_settings() {

       $added_sections = [];

       foreach ($this->settings_fields as $tab => $fields) {
           foreach ($fields as $field) {
           	   if (isset($this->default_settings[$field[0]])){
                  $default_value = $this->default_settings[$field[0]];
           	   }
           	   elseif (isset($this->default_messages[$field[0]])){
                  $default_value = $this->default_messages[$field[0]];
           	   }
           	   else {
           	    	$default_value = '';
           	   }
               register_setting('managemore_connector_' . $tab . '_settings_group', $field[0], ['default' => $default_value]);

               if (!isset($added_sections[$tab])) {
                   add_settings_section($tab . '_section', ucfirst($tab) . ' Settings', null, 'managemore_connector_' . $tab);
                   $added_sections[$tab] = true;
               }

               add_settings_field($field[0], $field[1], [$this, 'render_field'], 'managemore_connector_' . $tab, $tab . '_section', [
                   'option_name' => $field[0],
                   'type' => $field[2],
                   'description' => $field[3],
                   'choices' => $field[4] ?? [],
                   'hidden' => $field[4] ?? false
               ]);
           }
       }



    }


    /**
     * Main execution.
     *
     * @since 1.0.0
     * @access public
     */
    public function run() {

      //Register Plugin Settings------------------------------------------------------------------------------------
      add_action( 'admin_init',  array($this,'managemore_connector_settings') );
      //Register action to reset messages
      add_action('admin_post_reset_managemore_messages', array($this, 'reset_message_defaults') );
      //Create Plugin Menu------------------------------------------------------------------------------------------
      add_action( 'admin_menu',  array($this,'managemore_connector_menu') );
      //Setup Action Links------------------------------------------------------------------------------------------
      add_filter( 'plugin_action_links_' . MM_PLUGIN_BASENAME,  array($this,'managemore_connector_action_links') );
      //Setup Meta Links--------------------------------------------------------------------------------------------
      add_filter( 'plugin_row_meta',  array($this,'managemore_connector_meta_links'), 10, 2 );
      //Add field for last login timestamp -------------------------------------------------------------------------
      add_action( 'wp_login', array($this,'managemore_connector_record_last_login'), 10, 2 );
      //Add check if user disabled by MM ---------------------------------------------------------------------------
      //Deprecated. We now check if user is disabled/inactive at checkout
      //add_filter( 'authenticate', array($this,'managemore_connector_check_user_disabled'),100,3);
      //Add delete user logic (to remove customer pricing if user deleted ------------------------------------------
      add_action('delete_user',array($this,'managemore_connector_delete_user'));
      // Include Gateway Classes and register Payment Gateways with WooCommerce-------------------------------------
      add_action( 'plugins_loaded', array($this,'managemore_gateway_init'), 0 );
      //Set Screen Options for Customer Prices----------------------------------------------------------------------
      add_filter('set-screen-option', array($this,'customer_price_set_option'), 10, 3);

      // REST API
      $this->mm_rest_api->run();
      // Post Types
      $this->mm_post_types->run();
      // Woo/Wholesale Pricing Related
      if($this->wpp_active){
        $this->mm_woo_wholesale->run();
      }
      $this->mm_woo_extender->run();
      $this->mm_woo_order_status->run();

      // Add warning for debug mode
      if (get_option('mm_debug_mode', 0) != 0) {
          add_action('admin_notices', function () {
              $settings_url = admin_url('admin.php?page=managemore-connector-settings&tab=advanced');
              echo '<div class="notice notice-warning is-dismissible">
                      <p>
                          <strong>ManageMore Connector:</strong> Debug Mode is currently <em>enabled</em>, which may cause performance slowdowns on your site.
                          You can <a href="' . esc_url($settings_url) . '">go to the settings page</a> to disable it.
                      </p>
                    </div>';
          });
      }

    } 


    
  
  
    


    /**
     * Create Plugin Menu.
     *
     * @since 1.0.0
     * @access public
     */
    public function managemore_connector_menu() {
       global $menu, $submenu;


    	 //create new top-level menu
    	 add_menu_page( 'ManageMore WooCommerce Connector', 'ManageMore Connector', 'administrator', 'mm-connector-main', array($this,'managemore_connector_settings_page'),  plugins_url('/assets/images/mmlogo.png', __FILE__ ) );

       //Remove submenu for main item by clearing array
    	 $submenu['mm-connector-main'] = array();
       //create submenu pages (need a function for each, below)
       add_submenu_page( 'mm-connector-main', 'ManageMore Connector', 'Settings', 'administrator', 'managemore-connector-settings', array($this,'managemore_connector_settings_page'));

       //Customer Pricing and Indirect Taxes are only allowed if Wholesale Prices Premium is active
       if($this->wpp_active){ 
          //2.0.0 - Use List of Database Table instead of Custom Post Type       	
          //add_submenu_page( 'mm-connector-main', 'Customer Prices', 'Customer Prices', 'administrator', 'edit.php?post_type=mm-customer-price'); 
          $hook = add_submenu_page( 'mm-connector-main', 'Customer Prices', 'Customer Prices', 'administrator', 'managemore-connector-customer-pricing',array($this,'managemore_connector_customer_pricing_page')); 
          add_action( "load-$hook", array($this,'add_customer_price_options' )); //Add options to this page
          
          if (get_option('mm_indirect_tax_enable',1) == 1 ) {
              add_submenu_page( 'mm-connector-main', 'Indirect Tax Types', 'Indirect Tax Types', 'administrator', 'edit.php?post_type=mm-indirect-tax-type'); 
              //2.1.0 - Use List of Database Table instead of Custom Post Type       	
              //add_submenu_page( 'mm-connector-main', 'Indirect Tax Rules', 'Indirect Tax Rules', 'administrator', 'edit.php?post_type=mm-indirect-tax'); 
              $hook1 = add_submenu_page( 'mm-connector-main', 'Indirect Tax Rules', 'Indirect Tax Rules', 'administrator', 'managemore-connector-indirect-tax-rules', array($this,'managemore_connector_indirect_tax_rules_page')); 
              add_action( "load-$hook1", array($this,'add_indirect_tax_options' )); //Add options to this page
              $hook2 = add_submenu_page('mm-connector-main', 'Indirect Tax Jurisdiction Rules', 'Indirect Tax Jurisdiction Rules', 'administrator', 'managemore-connector-jurisdiction-tax-rules', array($this, 'managemore_connector_jurisdiction_tax_rules_page'));
              add_action("load-$hook2", array($this, 'add_jurisdiction_tax_options')); // Add options to the new page
              
          }
          $hook3 = add_submenu_page( 'mm-connector-main', 'Trade Class Restrictions', 'Trade Class Restrictions', 'administrator', 'managemore-connector-trade-class=restrictions', array($this,'managemore_connector_trade_class_restriction_page')); 
          add_action( "load-$hook3", array($this,'add_trade_class_options' )); //Add options to this page
       }
       

    }

    public function reset_message_defaults() {
       
        foreach ($this->default_messages as $key => $default) {
            update_option($key, $default);
        }
        
        wp_redirect(admin_url('admin.php?page=managemore-connector-settings&tab=messages&reset=true'));
        exit;
    }

    /**
     * Setup Action Links.
     *
     * @since 1.0.0
     * @access public
     */
    public function managemore_connector_action_links( $links ) {

      $plugin_links = array();
    	$plugin_links = array(
    		'<a href="' . admin_url( 'admin.php?page=managemore-connector-settings' ) . '">Settings</a>'
    	);
    	// Merge our new link with the default ones
    	return array_merge(  $links, $plugin_links );	
    }


    /**
     *Setup Meta Links.
     *
     * @since 1.0.0
     * @access public
     */
    public function managemore_connector_meta_links( $links, $file ) {
    	$plugin = MM_PLUGIN_BASENAME;
    	// create link
    	if ( $file == $plugin ) {
    		return array_merge(
    			$links,
    			array( '<a href="https://www.managemore.com/tutorials/installing-econnect-plugin-wordpress-woocommerce/" target="_blank" >Documentation</a>',
    			       '<a href="https://www.managemore.com/demo" target="_blank">Get a FREE ManageMore Demo</a>' )
    		);
    	}
    	return $links;

    }

    /**
     * Setup Settings Page (called by Plugin Menu).
     *
     * @since 1.0.0
     * @updated 5.0.1
     * @access public
     */
    public function managemore_connector_settings_page() {
    	   global $_wp_using_ext_object_cache;
  
         $active_tab = isset($_GET['tab']) ? sanitize_text_field($_GET['tab']) : 'general';
         echo '<div class="wrap">';
         echo '<h1><img src="' . plugins_url( 'assets/images/mmpill.png', __FILE__ ) . '" >ManageMore Connector Settings</h1>';
         echo '<h2 class="nav-tab-wrapper">';
         foreach ($this->tabs as $tab => $label) {
             $class = ($active_tab === $tab) ? 'nav-tab-active' : '';
             echo '<a class="nav-tab ' . esc_attr($class) . '" href="?page=managemore-connector-settings&tab=' . esc_attr($tab) . '">' . esc_html($label) . '</a>';
         }
         echo '</h2>';

         echo '<form method="post" action="options.php">';
         settings_fields('managemore_connector_' . $active_tab . '_settings_group');
         do_settings_sections('managemore_connector_' . $active_tab);
         if ($active_tab == 'performance'){
         	  if (!$_wp_using_ext_object_cache) {
        	  	  echo "</br></br><span>\u{26A0}\u{FE0F}</span> This feature is not available because no persistent object cache is activated on this site.";
         	  }
         	  echo "</br></br><span>\u{2139}\u{FE0F}</span>" . ' For more information about the benefits of object caching and how to use it, see <a href="https://www.managemore.com/ecommerce/using-persistent-object-caching/" target="_blank">this article</a>.';
         	  echo "</br></br><span>\u{2139}\u{FE0F}</span>" . ' If your Cart page is still using the legacy shortcode ([woocommerce_cart]), cart loading may be slow for carts with over 50 items. We strongly recommend switching to the Block Cart by editing the Cart page and inserting the Cart block from the WooCommerce blocks collection. For more information about switching to the Block Cart, see <a href="https://www.managemore.com/ecommerce/how-to-speed-up-your-woocommerce-cart/" target="_blank">this article</a>.';
         }
         elseif ($active_tab == 'advanced'){
         	    echo "\u{26A0}\u{FE0F} WARNING: Sensitive information may be logged in debug mode.  Debug mode may cause slowdown in website functionality. Use temporarily only, under support guidance, and remove log files when completed.";
         }
         submit_button();
         echo '</form>';
         
         if ($active_tab === 'messages') {
             echo '<form method="post" action="' . esc_url(admin_url('admin-post.php')) . '">';
             echo '<input type="hidden" name="action" value="reset_managemore_messages">';
             submit_button('Reset to Defaults', 'secondary', 'reset_defaults');
             echo '</form>';
         }
         echo '</div>';
    }

    public function render_field($args) {
    	  global $_wp_using_ext_object_cache;
 
        $option = get_option($args['option_name'], $this->default_messages[$args['option_name']] ?? '');

        // WooCommerce Tooltip
        /*
        $tooltip = !empty($args['description']) 
           ? '<span class="woocommerce-help-tip" tabindex="0" aria-label="' . esc_attr($args['description']) . '"></span>' 
           : '';

        echo '<label for="' . esc_attr($args['option_name']) . '">' . esc_html($args['label']). $tooltip ;
        */

        //Check if caching option is allowed
        $disabled = '';
        if (esc_attr($args['option_name']) == 'mm_allow_cache' && !$_wp_using_ext_object_cache) {
            $disabled = 'disabled';
        }


        switch ($args['type']) {
            case 'checkbox':
                echo '<input type="checkbox" name="' . esc_attr($args['option_name']) . '" value="1" ' . checked(1, $option, false) .' '.$disabled.' />';
                echo '</label><i> '.esc_attr($args['description']).'</i><br>';
                break;
            case 'textarea':
                echo '<textarea name="' . esc_attr($args['option_name']) . '" class="large-text" rows="3">' . esc_textarea($option) . '</textarea>';
                echo '</label><br><i> '.esc_attr($args['description']).'</i><br>';
                break;
            case 'select':
                echo '<select name="' . esc_attr($args['option_name']) . '" id="' . esc_attr($args['option_name']) . '">';
                foreach ($args['choices'] as $value => $label) {
                    echo '<option value="' . esc_attr($value) . '" ' . selected($option, $value, false) . '>' . esc_html($label) . '</option>';
                }
                echo '</select>';
                echo '</label><i> '.esc_attr($args['description']).'</i><br>';
                break;
            case 'select_dynamic_gateway':
                echo '<select name="' . esc_attr($args['option_name']) . '" id="' . esc_attr($args['option_name']) . '">';
                echo '<option value="">Any Active Gateway</option>';
                $selected_gateway = get_option($args['option_name'], '');
                $gateways = WC()->payment_gateways->payment_gateways();
                foreach ($gateways as $gateway_id => $gateway) {
                    if ($gateway->enabled == 'yes' && $gateway_id != 'managemore_onaccount') {
                        echo '<option value="' . esc_attr($gateway_id) . '" ' . selected($selected_gateway, $gateway_id, false) . '>' . esc_html($gateway->title) . '</option>';
                    }
                }
                echo '</select>';
                echo '</label><i> '.esc_attr($args['description']).'</i><br>';
                break;
            case 'small-text':
                echo '<input type="text" name="' . esc_attr($args['option_name']) . '" value="' . esc_attr($option) . '" class="regular-text small-text" />';
                echo '</label><i> '.esc_attr($args['description']).'</i><br>';
                break;
            default:
                echo '<input type="text" name="' . esc_attr($args['option_name']) . '" value="' . esc_attr($option) . '" class="regular-text" />';
                echo '</label><br><i> '.esc_attr($args['description']).'</i><br>';
        }
    }



    /**
     * Customer Pricing List Page (called by Plugin Menu).
     *
     * @since 2.0.0
     * @access public
     */
    public function managemore_connector_customer_pricing_page(){


     ?>
        <div class="wrap" id="managemore-connector-customer-pricing">
        <h2>Customer Specific Pricing</h2>

        <div id="poststuff">
        <div id="post-body" class="metabox-holder columns-2">
        <div id="post-body-content">
        <div class="meta-box-sortables ui-sortable">
        <form method="post">
        <?php
          $this->mm_customer_price_list->prepare_items();
          $this->mm_customer_price_list->display(); ?>
        </form>
        </div>
        </div>
        </div>
        <br class="clear">
        </div>
        </div>
        <?php     	
    }


    /**
     * Add Options to Customer Pricing List Page, and Instantiate Customer Pricing List object
     *
     * @since 2.0.0
     * @access public
     */
    public function add_customer_price_options() {
        $option = 'per_page';
        $args = array(
              'label' => 'Customer Prices',
              'default' => 50,
              'option' => 'customer_prices_per_page'
              );
         add_screen_option( $option, $args );

        // Instantiate Customer Price List object
        $this->mm_customer_price_list = new MM_Customer_Price_List;


    }
 

    /**
     * Indirect Tax Rules Page (called by Plugin Menu).
     *
     * @since 2.0.0
     * @access public
     */
    public function managemore_connector_indirect_tax_rules_page(){

        	   //Added 2.1.3 - Get Wholesales Prices Premium settings for minimum order for Wholesale Price to apply 
        	   //              (these settings can cause Wholesale Price not to override, thus causing indirect tax not to calculate
        	   //
        	   $wwpp_min_order_price =  get_option('wwpp_settings_minimum_order_price',0);
        	   $wwpp_min_order_qty = get_option('wwpp_settings_minimum_order_quantity',0);
        	   $wwpp_min_order_warning = '';
        	   if ($wwpp_min_order_price != 0 || $wwpp_min_order_qty !=0){
        	   	$wwpp_min_order_warning = '<p style="color:red;">WARNING! You have enabled the minimum order settings for Wholesale Prices Premium.  If the minimum order quantity/amount is not met, Indirect Taxes will not be added.</p><br>';
        	   }


     ?>
        <div class="wrap">
        <h2>Indirect Tax Rules</h2>
        <?php echo $wwpp_min_order_warning;?>
        <div id="poststuff">
        <div id="post-body" class="metabox-holder columns-2">
        <div id="post-body-content">
        <div class="meta-box-sortables ui-sortable">
        <form method="post">
        <?php
          $this->mm_indirect_tax_list->prepare_items();
          $this->mm_indirect_tax_list->display(); ?>
        </form>
        </div>
        </div>
        </div>
        <br class="clear">
        </div>
        </div>
        <?php     	
    }


    /**
     * Add Options to Indirect Tax Rule List Page, and Instantiate Indirect Tax List object
     *
     * @since 2.0.0
     * @access public
     */
    public function add_indirect_tax_options() {
        $option = 'per_page';
        $args = array(
              'label' => 'Rules',
              'default' => 50,
              'option' => 'indirect_taxes_per_page'
              );
         add_screen_option( $option, $args );

        // Instantiate Indirect Tax List object
        $this->mm_indirect_tax_list = new MM_Indirect_Tax_List;


    }

    /**
     * Indirect Tax Jurisdiction Rules Page (called by Plugin Menu).
     *
     * @since 2.0.0
     * @access public
     */
    public function managemore_connector_jurisdiction_tax_rules_page(){

        	   //Added 2.1.3 - Get Wholesales Prices Premium settings for minimum order for Wholesale Price to apply 
        	   //              (these settings can cause Wholesale Price not to override, thus causing indirect tax not to calculate
        	   //
        	   $wwpp_min_order_price =  get_option('wwpp_settings_minimum_order_price',0);
        	   $wwpp_min_order_qty = get_option('wwpp_settings_minimum_order_quantity',0);
        	   $wwpp_min_order_warning = '';
        	   if ($wwpp_min_order_price != 0 || $wwpp_min_order_qty !=0){
        	   	$wwpp_min_order_warning = '<p style="color:red;">WARNING! You have enabled the minimum order settings for Wholesale Prices Premium.  If the minimum order quantity/amount is not met, Indirect Taxes will not be added.</p><br>';
        	   }


     ?>
        <div class="wrap">
        <h2>Indirect Tax Jurisdiction Rules</h2>
        <?php echo $wwpp_min_order_warning;?>
        <div id="poststuff">
        <div id="post-body" class="metabox-holder columns-2">
        <div id="post-body-content">
        <div class="meta-box-sortables ui-sortable">
        <form method="post">
        <?php
          $this->mm_indirect_tax_jurisdiction_list->prepare_items();
          $this->mm_indirect_tax_jurisdiction_list->display(); ?>
        </form>
        </div>
        </div>
        </div>
        <br class="clear">
        </div>
        </div>
        <?php     	
    }

    /**
     * Trade Class Restriction List Page (called by Plugin Menu).
     *
     * @since 5.0.1
     * @access public
     */
    public function managemore_connector_trade_class_restriction_page(){
        echo '<div class="wrap">';
        echo '<h1 class="wp-heading-inline">' . __('Trade Class Restrictions', 'managemore-connector') . '</h1>';
        echo '<form method="post">';
        $this->mm_trade_class_restriction_list->prepare_items();
        $this->mm_trade_class_restriction_list->display();
        echo '</form></div>';
    }

   /**
     * Add Options to Trade Class Restrictions Page, and Instantiate Trade Class Restrictions List object
     *
     * @since 2.0.0
     * @access public
     */
    public function add_trade_class_options() {
        $option = 'per_page';
        $args = array(
              'label' => 'Rules',
              'default' => 50,
              'option' => 'restrictions_per_page'
              );
         add_screen_option( $option, $args );

        // Instantiate Indirect Tax List object
        $this->mm_trade_class_restriction_list = new MM_Trade_Class_Restriction_List;


    }

    /**
     * Add Options to Indirect Tax Jurisdiction Rule List Page, and Instantiate Indirect Tax List object
     *
     * @since 2.0.0
     * @access public
     */
    public function add_jurisdiction_tax_options() {
        $option = 'per_page';
        $args = array(
              'label' => 'Rules',
              'default' => 50,
              'option' => 'indirect_taxes_per_page'
              );
         add_screen_option( $option, $args );

        // Instantiate Indirect Tax List object
        $this->mm_indirect_tax_jurisdiction_list = new MM_Indirect_Tax_Jurisdiction_List;


    }


    /**
     * Save Options for Customer Pricing List
     *
     * @since 2.0.0
     * @access public
     */
    public static function customer_price_set_option($status, $option, $value) {
       return $value;
    }


    /**
     * Record extra fields at login, like last login date, IP Address, Browser, etc. (for REST API).
     *
     * @since 1.0.0
     * @access public
     */
    public function managemore_connector_record_last_login( $user_login, $user ) {
    	
        //Record Login Timestamp   	
        update_user_meta( $user->ID, 'mm_last_login', time() );
        //Record IP Address
        $ip_address = '';
        if ( ! empty( $_SERVER['HTTP_CLIENT_IP'] ) ) {
            //IP from share internet
            $ip_address = $_SERVER['HTTP_CLIENT_IP'];
         } elseif ( ! empty( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) {
            //to check if IP is passed from proxy
            $ip_address = $_SERVER['HTTP_X_FORWARDED_FOR'];
         } else {
            $ip_address = $_SERVER['REMOTE_ADDR'];
         }
         update_user_meta( $user->ID, 'mm_last_login_ip', $ip_address );
         //Record Browser Info
         $browser_info = $_SERVER['HTTP_USER_AGENT'];
         update_user_meta( $user->ID, 'mm_last_login_browser', $browser_info );
         


    }

    /**
     * Check user disable status
     *
     *
     * @since 1.0.0
     * @access public
     * 
     */
    public function managemore_connector_check_user_disabled ($user,$username,$password){
      //Corrected detection of wp_error 2.1.2 (5.12.2022)
    	if($user == NULL || is_wp_error($user)){
    		return $user;
    	}

     /*
     * This was changed (3.0.4) - We no longer prevent login, due to disabled status.  We prevent checkout instead.
     * This is so user can still update their info, such as license image, even when locked out. 
     */

    	//$mm_disable_user =  get_user_meta($user->ID,'mm_disable_user',true);
    	$mm_field_STAT =  get_user_meta($user->ID,'mm_field_STAT',true);
    	//added inactive check 1.0.2
    	if ($mm_field_STAT == 'Inactive'){
    		$mm_inactive_account_message = MM_Connector_Functions::mm_get_option('mm_inactive_account_message');
    		return new WP_Error( 'authentication_failed',$mm_inactive_account_message);
    	}
    	//elseif ($mm_disable_user == '1'){
    	//	$mm_disabled_account_message = MM_Connector_Functions::mm_get_option('mm_disabled_account_message');
    	//	return new WP_Error( 'authentication_failed',$mm_disabled_account_message);
    	//}
    	else {
    		return $user;
    	}
    } 



    //Initialize Gateways
    public function managemore_gateway_init() {

      //Woo Gateways
      require_once 'includes/class-wc-gateway-managemore-onaccount.php' ;
      require_once 'includes/class-wc-gateway-managemore-tsys.php' ;
      require_once 'includes/class-wc-gateway-managemore-revita.php' ;
     


    	add_filter( 'woocommerce_payment_gateways', 'add_managemore_onaccount_gateway' );
    	function add_managemore_onaccount_gateway( $methods ) {
    		$methods[] = 'WC_Gateway_ManageMore_Onaccount';
    		return $methods;
    	}

      //Global Payments Gateway 
      //depricated 4.7.0 (Was TSYS)
      //reinstated 4.9.3 (as Global Payments)
    	add_filter( 'woocommerce_payment_gateways', 'add_managemore_tsys_gateway' );
    	function add_managemore_tsys_gateway( $methods ) {
    		$methods[] = 'WC_Gateway_ManageMore_TSYS';
    		return $methods;
    	}




    	add_filter( 'woocommerce_payment_gateways', 'add_managemore_revita_gateway' );
    	function add_managemore_revita_gateway( $methods ) {
    		$methods[] = 'WC_Gateway_ManageMore_RevitA';
    		return $methods;
    	}


    }

    public function managemore_connector_delete_user($user_id){

        $customer_pricing  = ''; //Sending blank string will cause all pricing to be removed
    	  MM_Connector_Functions::update_customer_pricing($customer_pricing,$user_id,1);
    	  
    }




	
}



