<?php
/*
Plugin Name: ManageMore Connector
Plugin URI:  http://www.managemore.com
Description: Helps to connect WordPress and WooCommerce with ManageMore Business Software.
Version:     4.8.6
Author:      Intellisoft Solutions, Inc.
Author URI:  http://www.managemore.com
License:     GPL2
License URI: https://www.gnu.org/licenses/gpl-2.0.html
Text Domain: managemore-connector
Domain Path: /languages
*/



if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly
define( 'MM_CURRENT_VERSION', '4.8.6' );    // IMPORTANT ********************************************************   This needs to be kept in sync with the version above. (also change in info.json)
define( 'MM_UPDATE_URL', 'https://www.managemore.com/library/managemore-connector/get_update_info.php' );   
define( 'MM_PLUGIN_BASENAME',      plugin_basename(__FILE__) );
define( 'MM_PLUGIN_SLUG',          plugin_basename(__DIR__) );
define( 'MM_PLUGIN_PATH',          plugin_dir_path( __FILE__ ) );
define( 'MM_PLUGIN_URL',           plugins_url() . '/managemore-connector/' );
define( 'MM_CSS_PATH',             MM_PLUGIN_PATH . 'css/' );
define( 'MM_CSS_URL',              MM_PLUGIN_URL . 'css/' );
define( 'MM_IMAGES_PATH',          MM_PLUGIN_PATH . 'images/' );
define( 'MM_IMAGES_URL',           MM_PLUGIN_URL . 'images/' );
define( 'MM_INCLUDES_PATH',        MM_PLUGIN_PATH . 'includes/' );
define( 'MM_INCLUDES_URL',         MM_PLUGIN_URL . 'includes/' );

global $mm_connector_db_version;
$mm_connector_db_version = '1.4'; //Note: Change this value if you make changes to db structure in managemore-connector-db

$ok_to_run = true;
$wpp_active = true;

//Include and instantiate the mUpdateChecker class for plugin Updates
//Introduced in 2.1.1
require_once 'includes/class-mm-update-checker.php';
$mmupdate_checker = new mmUpdateChecker();

//Include Database Functions
require_once 'managemore-connector-db.php';

//Hooks for Database Initialization/Update
register_activation_hook( __FILE__, 'mm_connector_db_install' );
add_action( 'plugins_loaded', 'mm_connector_update_db_check' ); 
add_action('mm_connector_upgrade_dates_event', 'mm_connector_reformat_dates');

//Include general static functions - can be called anywhere from the plugin without instantiation
//Methods are called like this MM_Connector_Functions::is_plugin_active('mainpluginfile.php')
require_once MM_INCLUDES_PATH.'class-mm-connector-functions.php';

/**
 * Check if WooCommerce and Wholesale Prices Premium are both active
 */
if (!MM_Connector_Functions::is_plugin_active('woocommerce/woocommerce.php')) {


    /**
     * Provide admin notice when plugin dependency is missing.
     *
     * @since 1.0.0
     */
    function mm_missing_woo_plugin_dependency_notice() {

        ?>

        <div class="notice notice-error">
            <p>
                <?php _e('<b>ManageMore Connector</b> plugin is missing a dependency.<br/><br/>Please ensure you have the <a href="https://wordpress.org/plugins/woocommerce/" target="_blank">WooCommerce</a> plugin installed and activated.<br/>', 'managemore-connector'); ?>
            </p>
        </div>

<?php

    }

    add_action('admin_notices', 'mm_missing_woo_plugin_dependency_notice');
    $ok_to_run = false; //Cannot use this plug in without Woo


}

if (!MM_Connector_Functions::is_plugin_active('woocommerce-wholesale-prices-premium/woocommerce-wholesale-prices-premium.bootstrap.php')) {


    /**
     * Provide admin notice when plugin dependency is missing.
     *
     * @since 1.0.0
     */
    function mm_missing_wpp_plugin_dependency_notice() {

        ?>

        <div class="notice notice-error is-dismissible">
            <p>
                <?php _e('<b>ManageMore Connector</b> plugin is missing a dependency. Some features will not be available, unless installed.<br/><br/>Please ensure you have the <a href="https://wholesalesuiteplugin.com/woocommerce-wholesale-prices-premium/?ref=157" target="_blank">WooCommerce Wholesale Prices Premium</a> plugin installed and activated.<br/>', 'managemore-connector'); ?>
            </p>
        </div>

<?php


    }

    add_action('admin_notices', 'mm_missing_wpp_plugin_dependency_notice');
    $wpp_active = false;
    
} 


// check for Legacy Plug in -  This will conflict with the new plug in and should be deactivated/removed
if(in_array('managemore-econnect/managemore-econnect.php', apply_filters('active_plugins', get_option('active_plugins')))){ 
	
    /**
     * Provide admin notice when plugin conflict exists.
     *
     * @since 1.0.3
     */
    function mm_legacy_plug_in_exists() {

        ?>

        <div class="notice notice-error">
            <p>
                <?php _e('<b>ManageMore Connector</b> has a conflict with the Legacy ManageMore eConnect Plug-In.  You must deactivate that plug-in first.<br/>', 'managemore-connector'); ?>
            </p>
        </div>

<?php

    }

    add_action('admin_notices', 'mm_legacy_plug_in_exists');
    $ok_to_run = false; //Cannot use this plug in

}


if ($ok_to_run) {
	
  //Ok to run our plug in
	
  //Main Plugin
  require_once 'managemore-connector-plugin.php';
  // REST API
  require_once 'includes/api/class-mm-connector-rest-api.php';
  //Post Types
  require_once 'includes/class-mm-connector-post-types.php';
  //Customer Price List
  require_once 'includes/class-mm-connector-customer-price-list.php';
  //Indirect Tax List
  require_once 'includes/class-mm-connector-indirect-tax-list.php';
  //Indirect Tax Jurisiction List
  require_once 'includes/class-mm-connector-indirect-tax-jurisdiction-list.php';
  //Woo Wholesale Related Functionality (for Indiret Taxes, Pricing. Requires Wholesale Prices Premium)
  require_once 'includes/class-mm-connector-woo-wholesale.php';
  //Woo Only Functionality (does not require other plug-ins)
  require_once 'includes/class-mm-connector-woo-extender.php';

  //Instantiate and run
  $managemore_connector = new ManageMoreConnector;
  $managemore_connector->wpp_active = $wpp_active;
  $managemore_connector->run();
	
}



