<?php
/*
Plugin Name: ManageMore Connector
Plugin URI:  http://www.managemore.com
Description: Helps to connect WordPress and WooCommerce with ManageMore Business Software.
Version:     5.1.2
Author:      Intellisoft Solutions, Inc.
Author URI:  http://www.managemore.com
Requires Plugins: woocommerce
License:     GPL2
License URI: https://www.gnu.org/licenses/gpl-2.0.html
Text Domain: managemore-connector
Domain Path: /languages
*/



if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly
define( 'MM_CURRENT_VERSION', '5.1.2' );    // IMPORTANT ********************************************************   This needs to be kept in sync with the version above. (also change in info.json)
define( 'MM_UPDATE_URL', 'https://www.managemore.com/library/managemore-connector/get_update_info.php' );   
define( 'MM_PLUGIN_BASENAME',      plugin_basename(__FILE__) );
define( 'MM_PLUGIN_SLUG',          plugin_basename(__DIR__) );
define( 'MM_PLUGIN_PATH',          plugin_dir_path( __FILE__ ) );
define( 'MM_PLUGIN_URL',           plugins_url() . '/managemore-connector/' );
define( 'MM_CSS_PATH',             MM_PLUGIN_PATH . 'assets/css/' );
define( 'MM_CSS_URL',              MM_PLUGIN_URL . 'assets/css/' );
define( 'MM_JS_PATH',              MM_PLUGIN_PATH . 'assets/js/' );
define( 'MM_JS_URL',               MM_PLUGIN_URL . 'assets/js/' );
define( 'MM_IMAGES_PATH',          MM_PLUGIN_PATH . 'assets/images/' );
define( 'MM_IMAGES_URL',           MM_PLUGIN_URL . 'assets/images/' );
define( 'MM_INCLUDES_PATH',        MM_PLUGIN_PATH . 'includes/' );
define( 'MM_INCLUDES_URL',         MM_PLUGIN_URL . 'includes/' );

global $mm_connector_db_version;
$mm_connector_db_version = '1.9'; //Note: Change this value if you make changes to db structure in managemore-connector-db

$ok_to_run = true;
$wpp_active = true;

//Include and instantiate the mUpdateChecker class for plugin Updates
//Introduced in 2.1.1
require_once 'includes/class-mm-update-checker.php';
$mmupdate_checker = new mmUpdateChecker();

//Include Database Functions
require_once 'managemore-connector-db.php';

//Hooks for Database Initialization/Update
register_activation_hook( __FILE__, 'mm_connector_db_install' );
add_action( 'plugins_loaded', function() {
    if ( is_admin() && current_user_can( 'manage_options' ) ) {
        mm_connector_update_db_check();
    }
});
add_action('mm_connector_upgrade_dates_event', 'mm_connector_reformat_dates');

//Include general static functions - can be called anywhere from the plugin without instantiation
//Methods are called like this MM_Connector_Functions::is_plugin_active('mainpluginfile.php')
require_once MM_INCLUDES_PATH.'class-mm-connector-functions.php';

//Include class to define debug modes, check debug status (introduced 5.1.0)
require_once MM_INCLUDES_PATH.'class-mm-debug-mode.php';

//Include Cache Class (introduced 5.1.0)
require_once MM_INCLUDES_PATH.'class-mm-object-cache.php';

/**
 * Check if WooCommerce and Wholesale Prices Premium are both active
 */
if (!MM_Connector_Functions::is_plugin_active('woocommerce/woocommerce.php')) {


    /**
     * Provide admin notice when plugin dependency is missing.
     *
     * @since 1.0.0
     */
    function mm_missing_woo_plugin_dependency_notice() {

        ?>

        <div class="notice notice-error">
            <p>
                <?php _e('<b>ManageMore Connector</b> plugin is missing a dependency.<br/><br/>Please ensure you have the <a href="https://wordpress.org/plugins/woocommerce/" target="_blank">WooCommerce</a> plugin installed and activated.<br/>', 'managemore-connector'); ?>
            </p>
        </div>

<?php

    }

    add_action('admin_notices', 'mm_missing_woo_plugin_dependency_notice');
    $ok_to_run = false; //Cannot use this plug in without Woo


}

if (!MM_Connector_Functions::is_plugin_active('woocommerce-wholesale-prices-premium/woocommerce-wholesale-prices-premium.bootstrap.php')) {


    /**
     * Provide admin notice when plugin dependency is missing.
     *
     * @since 1.0.0
     */
    function mm_missing_wpp_plugin_dependency_notice() {

        ?>

        <div class="notice notice-error is-dismissible">
            <p>
                <?php _e('<b>ManageMore Connector</b> plugin is missing a dependency. Some features will not be available, unless installed.<br/><br/>Please ensure you have the <a href="https://wholesalesuiteplugin.com/woocommerce-wholesale-prices-premium/?ref=157" target="_blank">WooCommerce Wholesale Prices Premium</a> plugin installed and activated.<br/>', 'managemore-connector'); ?>
            </p>
        </div>

<?php


    }

    add_action('admin_notices', 'mm_missing_wpp_plugin_dependency_notice');
    $wpp_active = false;
    
} 


// check for Legacy Plug in -  This will conflict with the new plug in and should be deactivated/removed
if(in_array('managemore-econnect/managemore-econnect.php', apply_filters('active_plugins', get_option('active_plugins')))){ 
	
    /**
     * Provide admin notice when plugin conflict exists.
     *
     * @since 1.0.3
     */
    function mm_legacy_plug_in_exists() {

        ?>

        <div class="notice notice-error">
            <p>
                <?php _e('<b>ManageMore Connector</b> has a conflict with the Legacy ManageMore eConnect Plug-In.  You must deactivate that plug-in first.<br/>', 'managemore-connector'); ?>
            </p>
        </div>

<?php

    }

    add_action('admin_notices', 'mm_legacy_plug_in_exists');
    $ok_to_run = false; //Cannot use this plug in

}


function mm_check_ultimate_member_and_settings() {
    // Ensure this is in the admin area
    if (!is_admin()) {
        return;
    }

    // Check if Ultimate Member plugin is active
    if (!MM_Connector_Functions::is_plugin_active('ultimate-member/ultimate-member.php')) {
        return;
    }

    // Check the mm_allow_pdf_upload setting
    if (!MM_Connector_Functions::mm_get_option('mm_allow_pdf_upload')) {
        return;
    }

    // Define roles to check
    $roles_to_check = ['wholesale_customer', 'mm_price_2', 'mm_price_3', 'mm_price_4', 'mm_price_5', 'mm_price_6', 'mm_price_7', 'mm_price_8'];

    foreach ($roles_to_check as $role) {
        // Fetch role data for the current role
        $role_data = UM()->roles()->role_data($role);
        if (!$role_data) {
            continue; // Skip if role does not exist
        }

        // Get the title of the role from WordPress
        $role_title = $role ? wp_roles()->get_names()[ $role ] : '';


        // Determine if users with this role cannot edit profiles
        $can_edit_profile = isset($role_data['can_edit_profile']) && $role_data['can_edit_profile'] ? 1 : 0;
        if ($can_edit_profile === 0) {
            // Add a warning message specific to the role title
            echo '<div class="notice notice-error is-dismissible">
                    <p><strong>ManageMore Connector Warning:</strong> Ultimate Member is active, but users with the role <strong>' . esc_html($role_title) . '</strong> cannot edit profiles. These users will not be able to upload new license images. Please edit the "Can edit their profile?" setting for this role in the <a href="' . esc_url(admin_url('admin.php?page=um_roles')) . '" target="_blank">Ultimate Member User Roles</a> section.</p>
                  </div>';
        }
    }
}

if ($ok_to_run) {
	
  //Ok to run our plug in
  //Check for proper Ultimate Member Settings
  add_action('admin_notices', 'mm_check_ultimate_member_and_settings');
	
  //Main Plugin
  require_once 'managemore-connector-plugin.php';
  // REST API
  require_once 'includes/api/class-mm-connector-rest-api.php';
  //Post Types
  require_once 'includes/class-mm-connector-post-types.php';
  //Customer Price List
  require_once 'includes/class-mm-connector-customer-price-list.php';
  //Indirect Tax List
  require_once 'includes/class-mm-connector-indirect-tax-list.php';
  //Indirect Tax Jurisiction List
  require_once 'includes/class-mm-connector-indirect-tax-jurisdiction-list.php';
  //Trade Class Restriction List
  require_once 'includes/class-mm-connector-trade-class-restriction-list.php';

  //Woo Only Functionality (does not require other plug-ins)
  require_once 'includes/class-mm-connector-woo-extender.php';

  //Woo Wholesale Related Functionality (for Indiret Taxes, Pricing. Requires Wholesale Prices Premium)
  require_once 'includes/class-mm-connector-woo-wholesale.php';


  //Barcode Scanning
  require_once 'includes/class-mm-connector-woo-order-status.php';

  //Barcode Scanning
  require_once 'includes/class-mm-connector-barcode-scanner.php';
 

  //Instantiate and run
  $managemore_connector = new ManageMoreConnector;
  $managemore_connector->wpp_active = $wpp_active;
  $managemore_connector->run();
	
}



